#!/bin/sh
#
# This script will be run at end of month to create last months logs from the 
# current logs.  It simply copies the current log, then greps for instances of
# lastmonth and walla.
#

# Load the environment
. /etc/profile

# set a variable for logging in this script
log=/data/nmis/log/rollover.log
loglist=/data/nmis/files/logs.dat

# set date variables
year=`date | awk '{print $6}'`
month=`date | awk '{print $2}'|cut -c 1-3`
weekday=`date | awk '{print $1}'`
day=`date | awk '{print $3}'`
time=`date | awk '{print $4}'`
date="$day-$month-$year"     

case $month in
	Jan)	lastmonth=12
		month=01
		year=1999;;
	Feb)	month=02
		lastmonth=01;;
	Mar)	month=03
		lastmonth=02;;
	Apr)	month=04
		lastmonth=03;;
	May)	month=05
		lastmonth=04;;
	Jun)	month=06
		lastmonth=05;;
	Jul)	month=07
		lastmonth=06;;
	Aug)	month=08
		lastmonth=07;;
	Sep)	month=09
		lastmonth=08;;
	Oct)	month=10
		lastmonth=09;;
	Nov)	month=11
		lastmonth=10;;
	Dec)	month=12
		lastmonth=11;;
esac


# check the command line arguements for options
if [ $1 == "cisco" ]
then
	logdir=/data/nmis/log
	logfile=cisco.log
	newlog=cisco-$year$month$day.log
	desc="Cisco Syslog ending $year-$month-$day"
	
	cd $logdir

	if [ -f $newlog ]
	then
		echo "Cisco log already rolled over"
		exit
	else 
	  	# Copy the old log to a backup log
		cp $logfile $logfile.bak


	  	# Move the old log to a new log
		mv $logfile $newlog

	  	# Create a new empty log and set permissions
		touch $logfile
		chmod 666 $logfile

		# add an entry to the Log list for the web pages
		# format is shortname, filename, description
		echo "$newlog,$logdir/$newlog,$desc" >> $loglist

		kill -HUP `cat /etc/syslog.pid`

	fi
elif [ $1 == "event" ]
then
	# Do the Stats Event Log
	logdir=/data/nmis/log
	logfile=event.log
	newlog=event-$year$month$day.log
	desc="Event Log ending $year-$month-$day"
	
	cd $logdir

	if [ -f $newlog ]
	then
		echo "Event log already rolled over"
		exit
	else 
	  	# Copy the old log to a backup log
		cp $logfile $logfile.bak

	  	# Move the old log to a new log
		mv $logfile $newlog

	  	# Create a new empty log and set permissions
		touch $logfile
		chmod 666 $logfile

		# add an entry to the Log list for the web pages
		# format is shortname, filename, description
		echo "$newlog,$logdir/$newlog,$desc" >> $loglist

	fi
elif [ $1 == "apache" ]
then
	# Do the Apache Error Log
	logdir=/var/log/httpd
	logfile=access_log
	newlog=access-$year$month$day.log
	desc="Apache Access log ending $year-$month-$day"
	
	cd $logdir

	if [ -f $newlog ]
	then
		echo "Apache access log already rolled over"
		exit
	else 
	  	# Copy the old log to a backup log
		cp $logfile $logfile.bak

	  	# Move the old log to a new log
		mv $logfile $newlog

	  	# Create a new empty log and set permissions
		touch $logfile
		chmod 666 $logfile

		# add an entry to the Log list for the web pages
		# format is shortname, filename, description
		echo "$newlog,$logdir/$newlog,$desc" >> $loglist

		apachectl restart 1>/dev/null 2>/dev/null

	fi
elif [ $1 == "nmis" ]
then
	# Do the Apache Error Log
	logdir=/data/nmis/log
	logfile=nmis.log
	newlog=nmis-$year$month$day.log
	desc="NMIS Log ending $year-$month-$day"
	
	cd $logdir

	if [ -f $newlog ]
	then
		echo "NMIS log already rolled over"
		exit
	else 
	  	# Copy the old log to a backup log
		cp $logfile $logfile.bak

	  	# Move the old log to a new log
		mv $logfile $newlog

	  	# Create a new empty log and set permissions
		touch $logfile
		chmod 666 $logfile

		# add an entry to the Log list for the web pages
		# format is shortname, filename, description
		echo "$newlog,$logdir/$newlog,$desc" >> $loglist

	fi
fi


exit 0
