#!/usr/bin/perl
#
#    mibdump.pl - NMIS CGI Program - Network Mangement Information System
#    Copyright (C) 2000 Sinclair InterNetworking Services Pty Ltd
#    Keith Sinclair <keith@sinclair.org.au>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#*****************************************************************************
use lib "/usr/share/nmis/lib";
# 
#****** Shouldn't be anything else to customise below here *******************

# Check if there are any arguements
if ( $#ARGV < 0 ) {
        print "$0 NMIS MIB Dumping Tool\n";
        print "command line options are:\n";
        print "\tmibdir=<directory>     Location of the MIBS\n";
        print "\tmibs=<standard|router|switch|server>]  Which MIBs to load\n";
        print "\toutfile=<filename>]    Where to dump the OIDs to.\n";
        print "\n";
        
        exit(1);
}

use strict;
use SNMP_MIB;
use NMIS;
								       
# Variables for command line munging
my (%argue, $name, $value, $line, $index, $debug);

# Command Line Arguments:
for ($index=0; $index <= $#ARGV; ++$index) {
        if ($ARGV[$index] =~ /.+=/) {
                $name = $ARGV[$index];
                $value = $name;
                $name =~ s/(.*)=.*/$1/;
                $value =~ s/(.*)=(.*)/$2/;
                $argue{$name} = $value;
                if ( $debug eq "true" ) {
                        print "$name=$value\n"
                }
        } else {
                print "Invalid command argument: $ARGV[$index]\n";
		&errorQuit;
        }
}

if ( $argue{mibdir} eq "" ) { $argue{mibdir} = "/usr/share/snmp/mibs"; }

# Always load the default mib
SNMP_MIB::loadmib($argue{mibdir}, "RFC1213-MIB.my");

&loadMibs();
&dumpOID();

sub loadMibs {
	if ( $argue{mibs} eq "standard" ) { 
		# Standard Mibs
		SNMP_MIB::loadmib($argue{mibdir}, "IF-MIB-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "IANAifType-MIB-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "ETHERLIKE-MIB.my");
		SNMP_MIB::loadmib($argue{mibdir}, "RFC1315-MIB.my");
		SNMP_MIB::loadmib($argue{mibdir}, "ENTITY-MIB-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "SNMPv2-MIB-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "SNMPv2-SMI-V1SMI.my ");
		
	}
	elsif ( $argue{mibs} eq "switch" ) { 
		# Cisco Switch MIBS
		SNMP_MIB::loadmib($argue{mibdir}, "BRIDGE-MIB.my");
		SNMP_MIB::loadmib($argue{mibdir}, "CISCO-SMI-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "CISCO-PRODUCTS-MIB-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "CISCO-STACK-MIB-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "CISCO-PROCESS-MIB-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "CISCO-MEMORY-POOL-MIB-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "CISCO-ENVMON-MIB-V1SMI.MY");
		SNMP_MIB::loadmib($argue{mibdir}, "OLD-CISCO-INTERFACES-MIB.my");
	}
	elsif ( $argue{mibs} eq "router" ) { 
		SNMP_MIB::loadmib($argue{mibdir}, "CISCO-SMI-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "CISCO-PRODUCTS-MIB-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "OLD-CISCO-SYS-MIB.my");
		SNMP_MIB::loadmib($argue{mibdir}, "OLD-CISCO-CHASSIS-MIB.my");
		SNMP_MIB::loadmib($argue{mibdir}, "OLD-CISCO-INTERFACES-MIB.my");
		SNMP_MIB::loadmib($argue{mibdir}, "CISCO-PROCESS-MIB-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "CISCO-MEMORY-POOL-MIB-V1SMI.my");
		SNMP_MIB::loadmib($argue{mibdir}, "CISCO-ENVMON-MIB-V1SMI.MY");
		SNMP_MIB::loadmib($argue{mibdir}, "CISCO-FRAME-RELAY-MIB-V1SMI.my");
	}	
	elsif ( $argue{mibs} eq "server" ) { 
		# Cisco Switch MIBS
		SNMP_MIB::loadmib($argue{mibdir}, "UCD-SNMP-MIB.txt");
		SNMP_MIB::loadmib($argue{mibdir}, "sun.mib");
		SNMP_MIB::loadmib($argue{mibdir}, "sun-snmpdx.mib");
		SNMP_MIB::loadmib($argue{mibdir}, "MS-LMMIB2.MIB");
		SNMP_MIB::loadmib($argue{mibdir}, "MS-Perfmib.mib");
		SNMP_MIB::loadmib($argue{mibdir}, "HOST-RESOURCES-MIB.txt");
		SNMP_MIB::loadmib($argue{mibdir}, "HOST-RESOURCES-TYPES.txt");
	}
}

sub dumpOID {

	open (STDOUT,">$argue{outfile}");

	SNMP_MIB::dump_oids_file();

	close (STDOUT);

}


exit(0);

