#!/usr/bin/perl
#
#    nmis.pl - NMIS Perl Program - Network Mangement Information System
#    Copyright (C) 2000 Sinclair InterNetworking Services Pty Ltd
#    Keith Sinclair <keith@sinclair.org.au>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#*****************************************************************************
# Auto configure to the <nmis-base>/lib and <nmis-base>/files/nmis.conf
use lib "/usr/share/nmis/lib";
# 
#****** Shouldn't be anything else to customise below here *******************
# best to customise in the nmis.conf file.
#

# Check if there are any arguements
if ( $#ARGV < 0 ) {
	print <<EO_TEXT;
$0 NMIS Polling Engine - Network Mangement Information System
Copyright (C) 2000 Sinclair InterNetworking Services Pty Ltd

command line options are:
    type=<option>             Where <option> is one of the following:
                                   collect   NMIS will collect all statistics;
                                   update    Update all the dynamic NMIS configuration
                                   threshold Check collected stats for thresholds
                                   config    Validate the chosen configuration file
                                   apache    Produce Apache configuration for NMIS
    [file=<file name>]             Optional alternate configuation files;
    [node=<node name>]             Run operations on a single node;
    [debug=true|false|verbose|0-9] default=false - Show debuging information, quite handy;
    [collect=true|false]           default=true - Do the SNMP collection or not;

EO_TEXT
	exit(0);
}

require 5;

use Time::HiRes;
my $startTime = Time::HiRes::time();

use strict;
use csv;
use BER;
use SNMP_Session;
use SNMP_MIB;
use SNMP_Simple;
use SNMPv2c_Simple;
use RRDs;
use NMIS;
use func;
use ip;
use ping;

# Global Interpreted Variables ie stuff we figured out. 
my $sysObjectName;
my $ifIndex;
my $ifSpeed;

# Variable used to control writing the system file to ensure that it doesn't get blanked.
my $writeSystem = "false";

# Time Variables
my $time;
my $datestamp;
my $datestamp_2;
my $endTime;
# Global SNMP Variables once we get em keep em global
my %snmpTable;

# Variables for command line munging
my %nvp = getArguements(@ARGV);

# See if customised config file required.
my $conf;
if ( $nvp{file} ne "" ) { $conf = $nvp{file}; }
else { $conf = "nmis.conf"; }
my $configfile = "/etc/nmis/$conf";
if ( -f $configfile ) { loadConfiguration($configfile); }
else { die "Can't access configuration file $configfile.\n"; }

#Load the configuration
loadConfiguration($configfile);

# all arguments are now stored in nvp (name value pairs)
my $type=$nvp{type};
my $graph=$nvp{graph};
my $collect=$nvp{collect};
my $node_search=$nvp{node};
my $rmefile=$nvp{rmefile};

# Set debugging level.
my $debug = setDebug($nvp{debug});
$NMIS::debug = $debug;

# Create a hash for storing runReachability stats
my %reach;

# Create a hash for storing interface stats
my %ifStats;

# An array of Interface Status for AdminStatus and OperStatus
my @interfaceStatus = ("null","up","down","testing","unknown","dormant","notPresent");

# Set the default collect option
if ($collect eq "") { $collect = "true"; }

# Set the default graph option
if ($graph eq "") { $graph = "false"; }

# Other Global Variables
my $database;
my $databaseOriginal = "";
my @interfaceTable;
my $tmpifDescr;
my $ping;
my $timeout = 10;
my $pingresult;
my $snmpresult;
my $run;
my $statsDir;
my $devicetype;
my $onenode;
my $index;
my $nodecount;

# The node to turn on debugging for no matter what
my $debug_watch = "";
my $debug_global = $debug;

# How long to run for in seconds thus 30 * 60 is 1800 seconds
my $start_time = time;
my $run_time = 60 * 60;

if ($debug) { print returnTime ." Starting $0 $NMIS::VERSION: type=$type file=$configfile node=$node_search run_time=$run_time\n"; }

# Load the configuration and the node file!!!!!!!
loadConfiguration($configfile);
loadNodeDetails;
	
##############################################################
# SNMP Setup, debug options, load base mibs etc.
##############################################################
# set debug on or not depending on global debug 
if ($debug) {
	#You can turn off SNMP debug but still leave debug 
	#messages on just comment the following line for no 
	#SNMP debug at all
	#$SNMP_Simple::debug=1;
	
	# If you want SNMP warnings comment these lines
	#$SNMP_Simple::suppress_warnings=1;
	#$SNMP_Session::suppress_warnings=1;
}
else {
	$SNMP_Simple::suppress_warnings=1;
	$SNMP_Session::suppress_warnings=1;
	$SNMP_MIB::suppress_warnings=1;
}

# initalizes the OID mappings for the SNMP_Simple library.
if ($debug) { print returnTime." Loading All the Mibs\n"; }

if ($debug > 1) { print "\tLoading $NMIS::config{mib_root}/$NMIS::config{standard_mib}\n"; }
if ( -f "$NMIS::config{mib_root}/$NMIS::config{standard_mib}" ) { 
	SNMP_MIB::loadoids_file("$NMIS::config{mib_root}", "$NMIS::config{standard_mib}"); 
} else { warn returnTime." nmis.pl, mib file $NMIS::config{mib_root}/$NMIS::config{standard_mib} not found.\n"; }	

if ($debug > 1) { print "\tLoading $NMIS::config{mib_root}/$NMIS::config{router_mib}\n"; }
if ( -f "$NMIS::config{mib_root}/$NMIS::config{router_mib}" ) { 
	SNMP_MIB::loadoids_file("$NMIS::config{mib_root}", "$NMIS::config{router_mib}"); 
} else { warn returnTime." nmis.pl, mib file $NMIS::config{mib_root}/$NMIS::config{router_mib} not found.\n"; }	

if ($debug > 1) { print "\tLoading $NMIS::config{mib_root}/$NMIS::config{switch_mib}\n"; }
if ( -f "$NMIS::config{mib_root}/$NMIS::config{switch_mib}" ) { 
	SNMP_MIB::loadoids_file("$NMIS::config{mib_root}", "$NMIS::config{switch_mib}");
} else { warn returnTime." nmis.pl, mib file $NMIS::config{mib_root}/$NMIS::config{switch_mib} not found.\n"; }

# If this is an update, check the config and report any errors
if ( $type eq "update" ) { &checkConfig; }

# If this is an escalation run, do it
if ( $type eq "escalate" ) { &runEscalate; }
# If this is an config run, do it
elsif ( $type eq "config" ) { &checkConfig("fix"); }
# If this is an apache run, do it
elsif ( $type eq "apache" ) { &printApache; }
# If this is a threshold run, do it
elsif ( $type eq "threshold" ) {
	#######################################################################
	# Do the threshold thing
	#######################################################################
	if ( $node_search eq "" ) {
		foreach $onenode ( keys (%NMIS::nodeTable) ) {
			&runThreshold($onenode);
		}
	}
	else {
		&runThreshold($node_search);
	}
}      
# Read a Cisco Works file and produce an NMIS Nodes file
elsif ( $type eq "rme" ) {
	if ( $rmefile eq "" ) {
		errorQuit ("$0 the type=rme option requires a file arguement for source rme CSV file\ni.e. $0 type=rme rmefile=/data/file/rme.csv");
	}
	else {
		loadRMENodes($rmefile);
		writeNodesFile("$NMIS::config{Nodes_Table}.new");
	}
}
# Must be a collect run or something on one node
elsif ( $node_search ne "" ) {
	if ( $node_search eq $NMIS::nodeTable{$node_search}{node} ) {
		&runNodeStats($node_search);
	}
	else {
		print "Invalid node $node_search: No node of that name!\n";
	}
}
# Must be a collect run or something so do all the nodes
else {
	##############################################################
	# Processing Loop for all nodes
	##############################################################	
	$index = 0;
	foreach $onenode ( keys (%NMIS::nodeTable) ) {
		# on each loop verify if runstats has been running too
		# long, each pass should always be less then 30 minutes
		if ( time - $start_time > $run_time and $type ne "update" ) {
			$start_time = returnDateStamp($start_time);
			&errorQuit ("$0 has been running for more then $run_time seconds exiting....\nStart Time $start_time. Processed $index nodes. Current node=$onenode.");
		}
		
		# This will allow debugging to be turned on for a  
		# specific node where there is a problem
		if ( $onenode eq "$debug_watch" ) { $debug = "true"; }
		else { $debug = $debug_global; }

		undef %reach;
		&runNodeStats($onenode);

		++$index;
	} # foreach $onenode
}

if ($debug) { 
	$endTime = sprintf( "%.2f", Time::HiRes::tv_interval([$startTime]));
	print "\n".returnTime ." Polled $nodecount nodes current runtime $endTime seconds.\n";
};

# Couple of post processing things.
if ( $type eq "collect" ) { &runEscalate; }
elsif ( $type eq "update" ) { &createInterfaceInfo; }

if ($debug) { 
	$endTime = sprintf( "%.2f", Time::HiRes::tv_interval([$startTime]));
	print "\n".returnTime ." End of $0 Ran for $endTime seconds.  BYE!\n\n";
};

exit 0;

sub runNodeStats {
	my $runnode = shift;

	my $timestring;
	my $nodeCollect;
	my $start_run;
	my $end_run;

	++$nodecount;

	if ($debug) { 
		$start_run = Time::HiRes::time();
		print "\n";
		print returnTime." Starting runNodeStats for $runnode\n";
	};

	undef %reach;
	
	# set the global devicetype from the configured device type
	$devicetype = $NMIS::nodeTable{$runnode}{devicetype};
	
	## KS 1/4/2001 - Load the node file if it exists.
	if ( -f "$NMIS::config{'<nmis_var>'}/$runnode.dat" ) {
		loadSystemFile($runnode);
	}
	if ($debug) { 
		print returnTime." node=$runnode role=$NMIS::systemTable{roleType} type=$NMIS::systemTable{nodeType}\n";
		print "\tvendor=$NMIS::systemTable{nodeVendor} model=$NMIS::systemTable{nodeModel} interfaces=$NMIS::systemTable{ifNumber}\n";
	}

	#######################################################################
	# Lets do a ping to see if the device is reachable or not
	#######################################################################
	if ( $type eq "collect" or $type eq "interface" or $type eq "health" or $type eq "update" or $type eq "threshold" ) {
		&runPing($runnode);
	}

	#######################################################################
	# make sure that the interface directories exist
	#######################################################################
	if ( $type eq "interface" or $type eq "collect" and $pingresult == 100 ) {
		# if the directory for $database doesn't exist create it
		if (    not -d "$NMIS::config{database_root}/interface/$NMIS::nodeTable{$runnode}{devicetype}/$runnode" 
			and not -f "$NMIS::config{database_root}/interface/$NMIS::nodeTable{$runnode}{devicetype}/$runnode" 
			and -w "$NMIS::config{database_root}/interface/$NMIS::nodeTable{$runnode}{devicetype}" 
		) { 
			logMessage("runNodeStats, $runnode, creating interface database directory $database");
			mkdir("$NMIS::config{database_root}/interface/$NMIS::nodeTable{$runnode}{devicetype}/$runnode",0775) 
				or warn "cannot mkdir $NMIS::config{database_root}/interface/$NMIS::nodeTable{$runnode}{devicetype}/$runnode: $!";
		}
	}

	#######################################################################
	# Are we supposed to collect on this node set a temp boolean for this
	#######################################################################
	if ( $collect eq "true" and $NMIS::nodeTable{$runnode}{collect} eq "true" and $pingresult == 100 ) { 
		$nodeCollect = "true"; 
	}
	else { $nodeCollect = "false"; }
	 
	#######################################################################
	# Load the system info which is kept on file if it exists otherwise create it.
	#######################################################################
	if ( -f "$NMIS::config{'<nmis_var>'}/$runnode.dat" 
		and $NMIS::nodeTable{$runnode}{collect} eq "true" 
		and $pingresult == 100 
		and $type ne "update"
	) {
		loadSystemFile($runnode);
		&updateUptime($runnode);
	}
	elsif ( not -f "$NMIS::config{'<nmis_var>'}/$runnode.dat" ) {
		if ($debug) { print returnTime." No system file exists creating one.\n"; }
		&createSystemFile($runnode);
	}
	
	if ($debug>2) {
		if ( $NMIS::systemTable{sysDescr} ne "" ) {
			print "$NMIS::systemTable{sysDescr}\n";
			print "$NMIS::systemTable{sysUpTime}, $NMIS::systemTable{sysObjectID}, $NMIS::systemTable{sysServices}\n";
		}
	}

	if ($debug==9) { &runEventDebug; }
	
	#######################################################################
	# Now depending on the type of job we are doing.
	# Just update the interface file for the device.
	#######################################################################
	if ( $type eq "" ) {
		&errorQuit("No type defined for collection or update");
	}
	
	#######################################################################
	# update all the interface stats
	#######################################################################
	elsif ( $type eq "update" ) {
		&runUpdate($runnode);
	}
	
	#######################################################################
	# Just do health
	#######################################################################
	elsif ( $type eq "health" 
			and $NMIS::nodeTable{$runnode}{collect} eq "true" 
			and  $nodeCollect eq "true" 
			and $pingresult == 100 
			and $snmpresult == 100
	) {   
		&runHealth($runnode);
	} # end type eq health
	
	#######################################################################
	# type = collect so do health interface and reachability
	#######################################################################
	elsif ( $type eq "collect" 
		and $NMIS::nodeTable{$runnode}{collect} eq "true" 
		and $NMIS::systemTable{supported} eq "true" 
		and $pingresult == 100 
		and $snmpresult == 100
	) {      
	if ($debug) { print returnTime." $runnode running collect for interface and health\n"; }
		if ( $nodeCollect eq "true" ) {
			$type = "health";
			&runHealth($runnode);
		}
	
		$type = "interface";
		&runInterface($runnode);
		&runReachability($runnode);
		$type = "collect";
	} # end type eq collect
	
	#######################################################################
	# Do interface stuff then reachability
	#######################################################################
	elsif ( $type eq "interface" 
		and $NMIS::nodeTable{$runnode}{collect} eq "true" 
		and $NMIS::systemTable{supported} eq "true" 
		and $pingresult == 100 
		and $snmpresult == 100
	) {    
		&runInterface($runnode);
		&runReachability($runnode);
	} # end type eq interface
	
	#######################################################################
	# Only do Reachabillity
	#######################################################################
	elsif ( $NMIS::systemTable{supported} eq "false" 
			or $NMIS::nodeTable{$runnode}{collect} eq "false" 
	) {
		&runReachability($runnode);
	} 
	elsif ( $pingresult != 100 ) {
		if ($debug) { print returnTime." $runnode is down not collecting skipping to next node\n"; }
	}
	elsif ( $snmpresult != 100 and $pingresult == 100 ) {
		&runReachability($runnode);
		if ($debug) { print returnTime." $runnode SNMP is not responding skipping to next node\n"; }
	}
	#######################################################################
	# invalid type
	#######################################################################
	elsif ( ( $type ne "health" or $type ne "interface" or $type ne "update" or $collect ne "update" ) and 
		( $nodeCollect eq "true" and $NMIS::nodeTable{$runnode}{collect} eq "true" )
	) {
		print "Invalid TYPE arguement ie type=collect or type=threshold (node=$runnode type=$type collect=$nodeCollect)\n";
		exit -1;
	}
	else { 
		print "No Node as arguement\n"; 
		&errorQuit;
	}
	
	if ($debug) { 
		$end_run = sprintf( "%.2f", Time::HiRes::tv_interval([$start_run]));
		print returnTime." Ending poll of $runnode; Run Time = $end_run seconds\n";
	};
} # end runNodeStats

# Handle some simple debuging
sub errorQuit {
	my $string = shift;

	print returnDateStamp." $string\n";

	exit -1;
}

sub runInterface {
	my $node = shift;
	my $session;
	my $snmpError;
	my $message;
	my $interface;
	my %interfaceTable;
	my $createdone = "false";
	
	my $interfacefile = "$NMIS::config{'<nmis_var>'}/$node-interface.dat";
	
	if ( $NMIS::systemTable{snmpVer} eq "SNMPv2" ) {
		($session) = SNMPv2c_Simple->open($node, $NMIS::nodeTable{$node}{community});
	}
	else {
		($session) = SNMP_Simple->open($node, $NMIS::nodeTable{$node}{community});
	}
	if ( not defined($session) ) { 
		warn returnTime." runInterface, Session is not Defined Goodly to $node.\n"; 
		goto END_runInterface;
	}
	
	# See if an interface definition file exists
	if ( -r $interfacefile ) {
		#Interface file exists load the definitions
	
		if ($debug) { print returnTime." Loading the Interface File with $NMIS::systemTable{ifNumber} entries\n"; }
		%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","interface","\t");
	
		# This bit makes sure that the device wasn't down the last time it refreshed its interface files
		if ( not defined %interfaceTable ) {
			$message = "$node, interface file empty regenerating: $interfacefile";
			logMessage("runInterface, $message");
		    if ($debug) { print returnTime." $message\n"; }
			&createInterfaceFile($node); 
			%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","interface","\t");
		}
	}
	else {
		# Interface file does not exist or we can't read it better create it better not be owned by someone else
		&createInterfaceFile($node); 
		%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","interface","\t");
	}
	
	# Start a loop which go through the interface table

	## KS 9/4/01 Various changes to handle interfaces which get shutdown
	## during the day.
	
	#for ( $index = 1; $index <= $#interfaceTable; ++$index ) {
	# it would be nice to do some parallell interface processing here ie snmp 2 or 3 interfaces each hit.
	foreach $interface (keys %interfaceTable ) {
		# only collect on interfaces that are defined, with collection turned on globally
		# and for that interface and that are Admin UP
		if (    $collect eq "true" and 
			$interfaceTable{$interface}{ifAdminStatus} eq "up" and 
			$interfaceTable{$interface}{collect} eq "true"
			#and ( $NMIS::config{bad_interfaces} !~ /$interfaceTable{$interface}{Description}/ )
		) {
			# Get the bad characters out of the interface name
			$tmpifDescr = convertIfName($interfaceTable{$interface}{ifDescr});
			
			$database = "$NMIS::config{database_root}/interface/$NMIS::systemTable{nodeType}/$node/$node-$tmpifDescr.rrd";
			
			# Check if the RRD Database exists already if not create it
			# set the database name to be the stats directory with the router name 
			# and the interface name which should have been passed in by runstats
			if ( not -w $database) {
				if ( $interfaceTable{$interface}{ifType} =~ /frame-relay/ ) {
					$ifSpeed = 2000000;
				}
				else {
					$ifSpeed = $interfaceTable{$interface}{ifSpeed}
				}
				if ($debug) { print "Create Database $database\n"; }
				&createRRDDB("interface",$node);
			}
	
			# do the SNMP stuffy to get the standard stats
			(	$ifStats{ifDescr},
				$ifStats{ifOperStatus},
				$ifStats{ifAdminStatus},
				$ifStats{ifInOctets},
				$ifStats{ifOutOctets}
			) = $session->snmpget(  
				'ifDescr'.".$interfaceTable{$interface}{ifIndex}",
				'ifOperStatus'.".$interfaceTable{$interface}{ifIndex}",
				'ifAdminStatus'.".$interfaceTable{$interface}{ifIndex}",
				'ifInOctets'.".$interfaceTable{$interface}{ifIndex}",
				'ifOutOctets'.".$interfaceTable{$interface}{ifIndex}"
			);
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("runInterface, $message");
				if ($debug) { print returnTime." $message\n"; }
				goto END_runInterface;
			}
	
			if  ( $ifStats{ifInOctets} eq "" and $ifStats{ifOutOctets} eq "" ) {
				$message = "$node, interface $interfaceTable{$interface}{ifDescr} ifIn and Out Octets are blank; maybe be an interface which does not collect stats or bad SNMP configuration or access control on the router";
				logMessage("runInterface, $message");
				if ($debug) { print returnTime." $message\n"; }
				$snmpError = "true";
			}
			else {
				$snmpError = "false";
			}
	
			if ($debug) { print "    Int=$interfaceTable{$interface}{ifDescr} ifIndex=$interfaceTable{$interface}{ifIndex} OperStatus=$ifStats{ifOperStatus} in=$ifStats{ifInOctets} out=$ifStats{ifOutOctets}\n"; }
	
			if ( $snmpError eq "false" ) {
				# Now Compare Real Oper Status to Admin Status if different Log
				# Interface is down but it is supposed to be up
				if ( 	( 	$ifStats{ifOperStatus} == 2 
							and $ifStats{ifAdminStatus} == 2 
							and $interfaceTable{$interface}{ifAdminStatus} eq "up" 
					) or ( 
							$ifStats{ifOperStatus} == 1 
							and $interfaceTable{$interface}{ifOperStatus} eq "down" 
					)
				) {
					# This interface was up before but not now so recreate the interface file.
					if ($debug) { print "\tCreating an Interface file as $interfaceTable{$interface}{ifDescr} has changed ifAdminStatus.\n"; }
					# Reload the interface config won't get that one right but should get the next one right
					logMessage("runInterface, $node, $interfaceTable{$interface}{ifDescr} has changed ifAdminStatus ifOperStatus"); 
					if ( $createdone eq "false" ) {
						&createInterfaceFile($node); 
						$createdone = "true";
					}
					checkEvent(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Interface Down", details => "interface=$interfaceTable{$interface}{ifDescr}");
					# could also perform a notify here if you wanted to.
				}
				elsif ( $ifStats{ifOperStatus} != 1 and
						$interfaceTable{$interface}{ifAdminStatus} eq "up" 
				) {
					# ignore up/down events for ISDN type interfaces.
					if ( 	$interfaceTable{$interface}{ifDescr} !~ /bri|virtual|ser.*[0-9]:.*/i and
						$interfaceTable{$interface}{ifType} !~ /isdn|loopback|ds0|ds1|other|ppp/i
					) {
						notify(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Interface Down", details =>"interface=$interfaceTable{$interface}{ifDescr}");
					}
				}
				else {
					# Check if interface was down before
					checkEvent(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Interface Down", details => "interface=$interfaceTable{$interface}{ifDescr}");
				}
			
				# If new ifDescr is different from old ifDescr and its a cisco router
				# run createInterfaceFile and loadInterfaceFile and everything is back to date.
				# this handles auto discovery etc on the fly. 
				if (    $NMIS::systemTable{nodeModel} !~ /Catalyst/ and 
					$ifStats{ifDescr} ne $interfaceTable{$interface}{ifDescr} and 
					$ifStats{ifDescr} ne "" 
				) {
					if ($debug==2) { print "Creating an Interface file\n"; }
					# Reload the interface config won't get that one right but should get the next one right
					logMessage("runInterface, $node, ifIndex to ifDescr has Changed bad updating Interface File"); 
					&createInterfaceFile($node); 
					undef %interfaceTable;
					%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","interface","\t");

					# Maybe issue and alert for reboot and possibly other bad things like config change etc
				}
			
			# Check if the RRD Database Exists and update it
				if ( -w $database ) {
					&updateRRDDB("interface",$node,"$interfaceTable{$interface}{ifDescr}");
				}
			}
		} # collect eq true
		else {
			if ($debug==5) { print "    NOT Interface:  $interfaceTable{$interface}{ifDescr}: collect=$collect, ifAdminStatus=$interfaceTable{$interface}{ifAdminStatus}, Interface Collect=$interfaceTable{$interface}{collect}\n"; }
		}
	} # FOR LOOP
	# Finished with the SNMP
	END_runInterface:
	$session->close();
	if ($debug) { print returnTime." Finished getting Interface stats\n"; }
} # runInterface

sub runHealth {
	my $node = shift;
	my $session;
	my $interface;
	my $message;

	undef %snmpTable;
	
	if ($debug) { print returnTime." Starting Health Stats\n"; }

	# OPEN the SNMP Session 
	if ( $NMIS::systemTable{snmpVer} eq "SNMPv2" ) {
		($session) = SNMPv2c_Simple->open($node, $NMIS::nodeTable{$node}{community});
	}
	else {
		($session) = SNMP_Simple->open($node, $NMIS::nodeTable{$node}{community});
	}
	if ( not defined($session) ) { 
		warn returnTime." runHealth, Session is not Defined Goodly to $node.\n"; 
		goto END_runHealth;
	}

	$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";

	# Now depending on the node type do different things.
	if ( $NMIS::systemTable{nodeModel} eq "Catalyst6000" ) {
		# Get all the Cisco Health Stuff unless not doing collection
		if ($collect eq "true") {
			(       $snmpTable{avgBusy1},
				$snmpTable{avgBusy5},
				$snmpTable{MemoryUsedDRAM},
				$snmpTable{MemoryFreeDRAM},
				$snmpTable{MemoryUsedMBUF},
				$snmpTable{MemoryFreeMBUF},
				$snmpTable{MemoryUsedCLUSTER},
				$snmpTable{MemoryFreeCLUSTER},
				$snmpTable{sysTraffic},
				$snmpTable{TopChanges}
			) = $session->snmpget(  
				'cpmCPUTotal1min'.".9", 
				'cpmCPUTotal5min'.".9",
				'ciscoMemoryPoolUsed'.".1",
				'ciscoMemoryPoolFree'.".1",
				'ciscoMemoryPoolUsed'.".8",
				'ciscoMemoryPoolFree'.".8",
				'ciscoMemoryPoolUsed'.".9",
				'ciscoMemoryPoolFree'.".9",
				'sysTraffic',
				'dot1dStpTopChanges'
			);
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("runHealth, $message");
				if ($debug) { print returnTime." runHealth, $message\n"; }
				goto END_runHealth;
			}

			$reach{cpu} = $snmpTable{avgBusy5}; 
						if ( $snmpTable{MemoryFreeDRAM} > 0 and $snmpTable{MemoryUsedDRAM} > 0 ) {
							$reach{mem} = ( $snmpTable{MemoryFreeDRAM} * 100 ) / ($snmpTable{MemoryUsedDRAM} + $snmpTable{MemoryFreeDRAM}) ;
						}	
						else {	
							$reach{mem} = "U";	
						}

			if ($debug) { 
				print returnTime." Health Stats Summary\n";
				print "\tCPU:1=$snmpTable{avgBusy1}, 5=$snmpTable{avgBusy5} \n\tProcMem:$snmpTable{MemoryUsedDRAM}, $snmpTable{MemoryFreeDRAM} \n\tIOMem:$snmpTable{MemoryUsedMBUF}, $snmpTable{MemoryFreeMBUF} \n\tsysTraffic=$snmpTable{sysTraffic}, TopChanges=$snmpTable{TopChanges}\n";			    
			}

			# Check if the RRD Database Exists
			if ( -w $database ) { 
				if ($debug) { print "Update Database $database\n"; }
				&updateRRDDB("health",$node);
			}
		} # collect eq true

		# If database doesn't exist then create it
		if ( not -w $database) { 
			if ($debug) { print "Create Database $database\n"; }
			&createRRDDB("health",$node);
		}
	} # nodeModel eq Catalyst6000
	elsif ( $NMIS::systemTable{nodeModel} =~ /CiscoRouter|CatalystIOS/ ) {
		
		# Get all the Cisco Health Stuff unless not doing collection
		if ($collect eq "true") {
			(       $snmpTable{avgBusy1},
				$snmpTable{avgBusy5},
				$snmpTable{MemoryUsedPROC},
				$snmpTable{MemoryFreePROC},
				$snmpTable{MemoryUsedIO},
				$snmpTable{MemoryFreeIO},
				$snmpTable{bufferElFree},
				$snmpTable{bufferElHit},
				$snmpTable{bufferFail},
				$snmpTable{ipInReceives},
				$snmpTable{ipForwDatagrams},
				$snmpTable{ipInDelivers},
				$snmpTable{ipOutNoRoutes}
			) = $session->snmpget(  
				'avgBusy1',
				'avgBusy5',
				'ciscoMemoryPoolUsed'.".1",
				'ciscoMemoryPoolFree'.".1",
				'ciscoMemoryPoolUsed'.".2",
				'ciscoMemoryPoolFree'.".2",
				'bufferElFree',
				'bufferElHit',
				'bufferFail',
				'ipInReceives',
				'ipForwDatagrams',
				'ipInDelivers',
				'ipOutNoRoutes'
			);
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("runHealth, $message");
				if ($debug) { print returnTime." runHealth, $message\n"; }
				$SNMP_Simple::errmsg = "";
				goto END_runHealth;
			}
			if ($debug) { 
				print returnTime." Health Stats Summary\n";
				print "\tCPU:1=$snmpTable{avgBusy1}, 5=$snmpTable{avgBusy5}\n\tProcMem:$snmpTable{MemoryUsedPROC}, $snmpTable{MemoryFreePROC}\n\tIOMem:$snmpTable{MemoryUsedIO}, $snmpTable{MemoryFreeIO}\n\tBuffer:free=$snmpTable{bufferElFree}, hit=$snmpTable{bufferElHit}, fail=$snmpTable{bufferFail}\n\tIPIN: rec=$snmpTable{ipInReceives}, for=$snmpTable{ipForwDatagrams}, deliv=$snmpTable{ipInDelivers}, norout=$snmpTable{ipOutNoRoutes}\n";			      
			}
			
			$reach{cpu} = $snmpTable{avgBusy5}; 

			if ( $snmpTable{MemoryUsedPROC} == 0 or $snmpTable{MemoryFreePROC} == 0 ) {
				$reach{mem} = 100;
			}
			else {
				$reach{mem} = ( $snmpTable{MemoryFreePROC} * 100 ) / ($snmpTable{MemoryUsedPROC} + $snmpTable{MemoryFreePROC}) ; 
			}


			# Check if the RRD Database Exists
			if ( -w $database ) { 
				if ($debug) { print "Update Database $database\n"; }
				&updateRRDDB("health",$node);
			}
		} # collect eq true

		# If database doesn't exist then create it
		if ( not -w $database) { 
			if ($debug) { print "Create Database $database\n"; }
			&createRRDDB("health",$node);
		}
	} # nodeModel eq CiscoRouter

	# Finished with the SNMP
	END_runHealth:
	$session->close();
} # end runHealth

sub thresholdProcess {
	## KS 16/4/01 - used by runThreshold.
	my %args = @_;
	if ($debug) { print returnTime." Threshold: $args{event} level=$args{level} value=$args{value}\n"; }
	if ( $args{value} !~ /NaN/i and $args{value} != 0 ) {
		if ( $args{level} =~ /Normal/ ) { 
			checkEvent(node => $args{node}, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $args{event}, level => $args{level}, details => "Threshold=$args{value}");
		}
		else {
			notify(node => $args{node}, role => $args{role}, type => $args{type}, event => $args{event}, level => $args{level}, details => "Threshold=$args{value}");
		}
	}
}

sub runThreshold {
	## KS 16/4/01 - Optimised the subroutine a bit, got rid of some redundant code.
	my $node = shift;

	my %threshold;
	my $reportStats;
	my @tmparray;
	my @tmpsplit;
	my $level;
	my $event;
	my $operAvailability;
	my $totalUtil;
	my $inputUtil;
	my $outputUtil;
	my $reportStats;
	my @tmparray;
	my $interface;
	my %interfaceTable;

	if ($debug) { 
		print "\n";
		print returnTime." Starting Thresholding node=$node collect=$NMIS::nodeTable{$node}{collect}\n"; 
	}
	
	loadSystemFile($node);

	# Get the device health thresholds
	$reportStats = summaryStats($node,"health","-15 minutes",time);
	if ( $reportStats != 1 and $reportStats ne "" ) {
		@tmparray = split " ","@$reportStats";
		for ( $index = 0; $index <= $#tmparray; ++$index ) {
			@tmpsplit = split "=",@tmparray[$index];
			$threshold{$tmpsplit[0]} = $tmpsplit[1];
		}
	}
	
	# Now we have a nicely populated threshold table, lets compare them and issue some alerts
	$level = thresholdResponse($node,$NMIS::systemTable{roleType},$threshold{response});
	$event = "Proactive Response Time Threshold";
	thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $threshold{response});

	$level = thresholdHighPercent($node,$NMIS::systemTable{roleType},$threshold{reachable});
	$event = "Proactive Reachability Threshold";
	thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $threshold{reachable});

	$level = thresholdHighPercentLoose($node,$NMIS::systemTable{roleType},$threshold{available});
	$event = "Proactive Interface Availability Threshold";
	thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $threshold{available});

	# there is extra stuff to threshold!
	# Will need to add something to summary stats to threshold switch stats
	if ( 	$NMIS::systemTable{nodeModel} =~ /CiscoRouter|CatalystIOS|CiscoATM/ 
		and $NMIS::nodeTable{$node}{collect} eq "true"
	) {
		$reportStats = summaryStats($node,"cpu","-15 minutes",time);
		if ( $reportStats != 1 and $reportStats ne "" ) {
			@tmparray = split " ","@$reportStats";
			for ( $index = 0; $index <= $#tmparray; ++$index ) {
				@tmpsplit = split "=",@tmparray[$index];
				$threshold{$tmpsplit[0]} = $tmpsplit[1];
			}
	
			$level = thresholdLowPercent($node,$NMIS::systemTable{roleType},$threshold{avgBusy5min});
			$event = "Proactive CPU Threshold";
			thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $threshold{avgBusy5min});
	
			$level = thresholdMemory($node,$NMIS::systemTable{roleType},$threshold{ProcMemFree});
			$event = "Proactive Memory Threshold";
			thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $threshold{ProcMemFree});
		}
		else {
			if ($debug) { print returnTime." Problem with Health Stats on $node.\n"; }
		}
	}
	
	# Threshold interface utilisation!  Easy to do, just take an hour or so.
	# hmmmm I was wrong, about 20mins plus some testing. (I love reusable code)
	if ( 	$NMIS::systemTable{nodeModel} =~ /Catalyst6000|Catalyst5000|CiscoRouter|CatalystIOS|CiscoATM|generic|Sun/ 
		and $NMIS::nodeTable{$node}{collect} eq "true"
	) {
		%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","interface","\t");
		# Extract the interface statics and summaries for display in a second.
		foreach $interface (keys %interfaceTable) {
			# Don't do any stats cause the interface is not one we collect
			if ( $interfaceTable{$interface}{collect} eq "true" ) { 
				# Get the link availability from the local node!!!
				$reportStats = summaryStats($node,"util","-15 minutes",time,$interfaceTable{$interface}{ifDescr},$interfaceTable{$interface}{ifSpeed});
				if ( $reportStats != 1 and $reportStats ne "" ) {
					@tmparray = split " ","@$reportStats";
					$operAvailability = $tmparray[0];
					$inputUtil = $tmparray[1];
					$outputUtil = $tmparray[2];
					$totalUtil = $tmparray[3];
					$operAvailability =~ s/(.*)=(.*)/$2/;
					$inputUtil =~ s/(.*)=(.*)/$2/;
					$outputUtil =~ s/(.*)=(.*)/$2/;
					$totalUtil =~ s/(.*)=(.*)/$2/;
				}

				# Interface Input Utilisation
				$level = thresholdInterfaceUtil($node,$NMIS::systemTable{roleType},$inputUtil);
				$event = "Proactive Interface Input Utilisation Threshold $interfaceTable{$interface}{ifDescr}";
				thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $inputUtil);

				# Interface Output Utilisation
				$level = thresholdInterfaceUtil($node,$NMIS::systemTable{roleType},$outputUtil);
				$event = "Proactive Interface Output Utilisation Threshold $interfaceTable{$interface}{ifDescr}";
				thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $outputUtil);

				# Interface Availability
				$level = thresholdInterfaceAvail($node,$NMIS::systemTable{roleType},$operAvailability);
				$event = "Proactive Availability Threshold Interface $interfaceTable{$interface}{ifDescr}";
				thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $operAvailability);
			}
		} # FOR LOOP
	}
} # end runThreshold

sub runEscalate {

	my $outage_time;
	my $planned_outage;
	my $event_hash;
	my $send = "false";
	my %location_data;
	my $target;
	my $time;
	my $escalate;
	my $event_age;
	my %contact_table;
	my @x;
	my $i;
	
	if ($debug) { print "\n".returnTime." Running escalate...\n"; }
	
	%contact_table = loadCSV($NMIS::config{Contacts_Table},$NMIS::config{Contacts_Key},"\t");

	# Load the event table into the hash
	loadEventState;
	
	foreach $event_hash ( sort(keys(%NMIS::eventTable)) )  {
	    $outage_time = time - $NMIS::eventTable{$event_hash}{startdate};
	    
	    # The event must not be part of a planned outage
	    if ( 	outageCheck($NMIS::eventTable{$event_hash}{node},time) ne "true" 
	    		and $NMIS::eventTable{$event_hash}{ack} eq "false"
	    ) {
			$escalate = $NMIS::eventTable{$event_hash}{escalate};
			# now depending on the event escalate the event up a level or so depending on how long it has been active
			# now would be the time to notify as to the event. node down every 15 minutes, interface down every 4 hours?
			# maybe a descreasing run 15,30,60,2,4,etc
			# proactive events would be escalated daily
			# when escalation hits 10 they could auto delete?
			# core, distrib and access could escalate at different rates.
			if ( $NMIS::eventTable{$event_hash}{event} eq "Node Down" ) {
				if (    $outage_time >= 12 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 10; $send = "true"; }
				elsif ( $outage_time >= 8 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 9; $send = "true"; }
				elsif ( $outage_time >= 4 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 8; $send = "true"; }
				elsif ( $outage_time >= 3 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 7; $send = "true"; }
				elsif ( $outage_time >= 2 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 6; $send = "true"; }
				elsif ( $outage_time >= 1 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 5; $send = "true"; }
				elsif ( $outage_time >= 50 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 4; $send = "true"; }
				elsif ( $outage_time >= 40 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 3; $send = "true"; }
				elsif ( $outage_time >= 30 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 2; $send = "true"; }
				elsif ( $outage_time >= 15 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 1; $send = "true"; }
			}
			elsif ( $NMIS::eventTable{$event_hash}{event} eq "Interface Down" ) {
				if (    $outage_time >= 24 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 10; $send = "true"; }
				elsif ( $outage_time >= 12 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 9; $send = "true"; }
				elsif ( $outage_time >= 6 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 8; $send = "true"; }
				elsif ( $outage_time >= 3 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 7; $send = "true"; }
				elsif ( $outage_time >= 2 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 6; $send = "true"; }
				elsif ( $outage_time >= 1 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 5; $send = "true"; }
				elsif ( $outage_time >= 50 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 4; $send = "true"; }
				elsif ( $outage_time >= 40 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 3; $send = "true"; }
				elsif ( $outage_time >= 30 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 2; $send = "true"; }
				elsif ( $outage_time >= 15 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 1; $send = "true"; }
			}
			elsif ( $NMIS::eventTable{$event_hash}{event} =~ /proactive/i ) {
				if (    $outage_time >= 24 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 10; $send = "true"; }
				elsif ( $outage_time >= 12 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 9; $send = "true"; }
				elsif ( $outage_time >= 6 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 8; $send = "true"; }
				elsif ( $outage_time >= 3 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 7; $send = "true"; }
				elsif ( $outage_time >= 2 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 6; $send = "true"; }
				elsif ( $outage_time >= 1 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 5; $send = "true"; }
				elsif ( $outage_time >= 50 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 4; $send = "true"; }
				elsif ( $outage_time >= 40 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 3; $send = "true"; }
				elsif ( $outage_time >= 30 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 2; $send = "true"; }
				elsif ( $outage_time >= 15 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 1; $send = "true"; }
			}
		
			if ($debug>4) { print "$NMIS::eventTable{$event_hash}{node} Event $NMIS::eventTable{$event_hash}{event} is $outage_time seconds old escalation is $NMIS::eventTable{$event_hash}{escalate}\n"; }
		}
		# send a new email message as the escalation again.
		if ( 	$send eq "true" 
				and $escalate != $NMIS::eventTable{$event_hash}{escalate} 
				and $NMIS::eventTable{$event_hash}{event} !~ /Proactive/
		) {
			loadSystemFile($NMIS::eventTable{$event_hash}{node});
			
			$event_age = convertSecsHours(time - $NMIS::eventTable{$event_hash}{startdate});
			$time = &returnDateStamp;
			
			$target = lc(escalationPolicy( group => $NMIS::nodeTable{$NMIS::eventTable{$event_hash}{node}}{group}, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, level => $NMIS::eventTable{$event_hash}{escalate} ));
			
			if ( $target ne "" ) {
				# Now we have a target, just in case, check for multiple contacts
				@x = split(":",$target);
				$target = "";
				if ( $x[0] eq "email" ) {
					for ( $i = 1; $i <= $#x; ++$i ) {
						if ( $contact_table{$x[$i]}{Email} ne "" ) {
							$target = $target." ".$contact_table{$x[$i]}{Email};
						}
						#print "\tx=$x[$i] email=$contact_table{$x[$i]}{Email}\n"
					}
					if ( $target eq "" ) { $target = $contact_table{default}{Email}; }
					#print returnTime." runEscalate x=@x hash=$contact_table{$x[1]}{Email} target=$target\n";
							
					&sendEmail( target => $target, 
							subject => "$time Escalation $NMIS::eventTable{$event_hash}{escalate} $event_age $NMIS::eventTable{$event_hash}{node} $NMIS::eventTable{$event_hash}{event_level} $NMIS::eventTable{$event_hash}{event} $NMIS::eventTable{$event_hash}{details}", 
							body => "$time Escalation $NMIS::eventTable{$event_hash}{escalate} $event_age $NMIS::eventTable{$event_hash}{node} $NMIS::eventTable{$event_hash}{event_level} $NMIS::eventTable{$event_hash}{event} $NMIS::eventTable{$event_hash}{details}\nhttp://$NMIS::config{nmis_host}$NMIS::config{nmis}?type=event&node=$NMIS::eventTable{$event_hash}{node}"
					);
				}
				else {
					print STDERR "\tERROR runEscalate problem with escalation target @x\n";
				}
			}
			if ($debug>4) { print "\tESCALATION NOTIFICATION node=$NMIS::eventTable{$event_hash}{node} target=$target level=$NMIS::eventTable{$event_hash}{event_level} event=$NMIS::eventTable{$event_hash}{event} details=$NMIS::eventTable{$event_hash}{details} group=$NMIS::nodeTable{$NMIS::eventTable{$event_hash}{node}}{group}\n"; }
		}
		$send = "false";
	} # foreach $event_hash
	writeEventState;
} # end runEscalate

sub runUpdate {
	my $node = shift;
	
	if ($debug) { print returnTime." Running an Update for node=$node, type=$NMIS::systemTable{nodeType}\n"; }
		
	# Update supported by all devices
	&createSystemFile($node);
	
	# Interface files only support for supported platforms
	if ( $NMIS::systemTable{supported} eq "true" and $pingresult == 100 ) {
		&createInterfaceFile($node);
	}
}

sub runPing{
	my $node = shift;
	
	if ($debug) { print returnTime." Starting Pinging\n"; }
	
	# Create a ping and check if the device is up.
	$pingresult = ping(node => $node, timeout => 5, debug => $debug );
	if ( $pingresult != 100 ) { 
		# Sleep a bit and try a second time.
		if ($debug>1) { print "Sleeping 15 seconds\n"; }
		sleep 15;
		$pingresult = ping(node => $node, timeout => 5, debug => $debug );
		if ( $pingresult != 100 ) {
			# It is still down
			$pingresult=0; 
			if ($debug) { print returnTime." Pinging Failed $node $NMIS::systemTable{roleType} $NMIS::systemTable{nodeType}\n"; }
				notify(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Node Down");	
				# Device is down only update the runReachability if its an interface.
				if ( $type eq "interface" ) {
					&runReachability;
				}
				#exit(1);
			}
		else {
				# It is up!!
				checkEvent(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Node Down");
	      }
	}
	else {
		# It is up!!
		checkEvent(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Node Down");
	}
}

sub runReachability {
	my $node = shift;
	
	my $cpuWeight;
	my $memWeight;
	my $responseWeight;
	my $interfaceWeight;
	my %interfaceTable;
	my $interface;
	my $inputUtil;
	my $outputUtil;
	my $totalUtil;
	my $index;
	my $reportStats;
	my @tmparray;
	my @tmpsplit;
	my %util;
	my $intcount;
	my $intsummary;
	my $intWeight;
	
	$database = "$NMIS::config{database_root}/health/$devicetype/$node-reach.rrd";
	
	if ($debug) { print returnTime." Running Reachability node=$node type=$devicetype\n"; }
	
	# Things which don't do collect get 100 for availability
	if ( $reach{availability} eq "" and $collect eq "false" ) { 
		$reach{availability} = "100"; 
	}
	elsif ( $reach{availability} eq "" ) { $reach{availability} = "U"; }
	
	# Health should actually reflect a combination of these values
	# ie if response time is high health should be decremented.
	if ( $pingresult == 100 and $snmpresult == 100 ) {
		$reach{reachability} = 100;
		if ( $reach{operCount} > 0 ) {
			$reach{availability} = $reach{operStatus} / $reach{operCount}; 
		}
		$reach{responsetime} = responseTime(node => "$node", count => 5, debug => "$debug"); 
		if ( $reach{responsetime} eq "" ) { 
			$reach{responsetime} = "U";
			$responseWeight = 0; 
		}
		elsif ( $reach{responsetime} !~ /^[0-9]/ ) { 
			$reach{responsetime} = "U";
			$responseWeight = 0; 
		}
		elsif ( $reach{responsetime} == 0 ) { 
			$reach{responsetime} = 1; 
			$responseWeight = 100; 
		}
		elsif ( $reach{responsetime} >= 1500 ) { $responseWeight = 0; }
		elsif ( $reach{responsetime} >= 1000 ) { $responseWeight = 10; }
		elsif ( $reach{responsetime} >= 900 ) { $responseWeight = 20; }
		elsif ( $reach{responsetime} >= 800 ) { $responseWeight = 30; }
		elsif ( $reach{responsetime} >= 700 ) { $responseWeight = 40; }
		elsif ( $reach{responsetime} >= 600 ) { $responseWeight = 50; }
		elsif ( $reach{responsetime} >= 500 ) { $responseWeight = 60; }
		elsif ( $reach{responsetime} >= 400 ) { $responseWeight = 70; }
		elsif ( $reach{responsetime} >= 300 ) { $responseWeight = 80; }
		elsif ( $reach{responsetime} >= 200 ) { $responseWeight = 90; }
		elsif ( $reach{responsetime} >= 0 ) { $responseWeight = 100; }
		
		if ( $NMIS::nodeTable{$node}{collect} eq "true" and $NMIS::systemTable{nodeModel} !~ /generic|sun/i ) {
			if    ( $reach{cpu} <= 10 ) { $cpuWeight = 100; }
			elsif ( $reach{cpu} <= 20 ) { $cpuWeight = 90; }
			elsif ( $reach{cpu} <= 30 ) { $cpuWeight = 80; }
			elsif ( $reach{cpu} <= 40 ) { $cpuWeight = 70; }
			elsif ( $reach{cpu} <= 50 ) { $cpuWeight = 60; }
			elsif ( $reach{cpu} <= 60 ) { $cpuWeight = 50; }
			elsif ( $reach{cpu} <= 70 ) { $cpuWeight = 40; }
			elsif ( $reach{cpu} <= 80 ) { $cpuWeight = 30; }
			elsif ( $reach{cpu} <= 90 ) { $cpuWeight = 20; }
			elsif ( $reach{cpu} <= 100 ) { $cpuWeight = 10; }
			
			if    ( $reach{mem} >= 40 ) { $memWeight = 100; }
			elsif ( $reach{mem} >= 35 ) { $memWeight = 90; }
			elsif ( $reach{mem} >= 30 ) { $memWeight = 80; }
			elsif ( $reach{mem} >= 25 ) { $memWeight = 70; }
			elsif ( $reach{mem} >= 20 ) { $memWeight = 60; }
			elsif ( $reach{mem} >= 15 ) { $memWeight = 50; }
			elsif ( $reach{mem} >= 10 ) { $memWeight = 40; }
			elsif ( $reach{mem} >= 5 )  { $memWeight = 25; }
			elsif ( $reach{mem} >= 0 )  { $memWeight = 0; }
		}
		elsif ( $NMIS::nodeTable{$node}{collect} eq "true" and $NMIS::systemTable{nodeModel} eq "generic" ) {
			$cpuWeight = 100;
			$memWeight = 100;
		}
		else {
			$cpuWeight = 100;
			$memWeight = 100;
			$reach{availability} = 100;
		}
		
		## KS 13/4/01 - Added little fix for when no interfaces are collected.
		if ( $reach{availability} !~ /\d+/ ) {
			$reach{availability} = "100";
		}
		
		## KS 1/5/01 - Makes CatalystIOS memory health weighting always 100.
		if ( $NMIS::systemTable{nodeModel} eq "CatalystIOS" ) {
			$memWeight = 100;
		}
		
		if ( $NMIS::nodeTable{$node}{collect} eq "true" ) {
			if ($debug) { print "\tGetting Interface Utilisation Health\n"; }
			$intcount = 0;
			$intsummary = 0;
			%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","interface","\t");
			foreach $interface (keys %interfaceTable) {
				# Don't do any stats cause the interface is not one we collect
				if ( $interfaceTable{$interface}{collect} eq "true" ) { 
				# Get the link availability from the local node!!!
					$reportStats = summaryStats($node,"util","-15 minutes",time,$interfaceTable{$interface}{ifDescr},$interfaceTable{$interface}{ifSpeed});
					if ( $reportStats != 1 and $reportStats ne "" ) {
						@tmparray = split " ","@$reportStats";
						for ( $index = 0; $index <= $#tmparray; ++$index ) {
							@tmpsplit = split "=",@tmparray[$index];
							$util{$tmpsplit[0]} = $tmpsplit[1];
						}
					}
					$intsummary = $intsummary + ( 100 - $util{inputUtil} ) + ( 100 - $util{outputUtil} );
					++$intcount;
					#print "in=$util{inputUtil} out=$util{outputUtil} intsumm=$intsummary count=$intcount\n";
				}
			} # FOR LOOP
			if ( $intsummary != 0 ) {
				$intWeight = sprintf( "%.2f", $intsummary / ( $intcount * 2 ));
			}
			else {
				$intWeight = "NaN"
			}
		}
		else {
			$intWeight = 100;	
		}
		
		# if the interfaces are unhealthy and lost stats, whack a 100 in there
		if ( $intWeight eq "NaN" ) { $intWeight = 100; }
		
		# Would be cool to collect some interface utilisation bits here.
		# Maybe thresholds are the best way to handle that though.  That
		# would pickup the peaks better.
		
		# Health is made up of a weighted values:
		#	reachability is 30% 
		#	interface is 20% 
		#	response time is 20% 
		#	availability is 10% 
		#	CPU is 10% 
		#	MEM is 10% 
		$reach{health} = 	($reach{reachability} * 0.3) + 
							($intWeight * 0.2) +
							($responseWeight * 0.20) + 
							($reach{availability} * 0.1) + 
							($cpuWeight * 0.1) + 
							($memWeight * 0.1)
							;
	}
	elsif ( $pingresult == 0 and NMIS::outageCheck($node,time) eq "true" ) {
		if ($debug) { print returnTime." Planned Outage Exists\n"; }
		$reach{reachability} = "U"; 
		$reach{availability} = "U"; 
		$reach{responsetime} = "U"; 
		$reach{health} = "U"; 
	} 
	elsif ( $pingresult == 100 and $snmpresult == 0 ) {
		$reach{reachability} = 100; 
		$reach{availability} = "U"; 
		$reach{responsetime} = responseTime(node => "$node", count => 5, debug => "$debug"); 
		$reach{health} = "U"; 
	}
	else {
		$reach{reachability} = 0; 
		$reach{availability} = "U"; 
		$reach{responsetime} = "U"; 
		$reach{health} = 0; 
	}
	
	if ($debug) { print returnTime." Health: reach=$reach{reachability} avail=$reach{availability} rt=$reach{responsetime} cpu=$cpuWeight mem=$memWeight int=$intWeight health=$reach{health}\n"; }
	
	if ($debug>2) { print returnTime." Doing Reachability database stuff!\n"; }
	# If database doesn't exist then create it
	if ( not -w $database) { &createRRDDB("reach",$node); }
	
	# Check if the RRD Database Exists and update it
	if ( -w $database ) { &updateRRDDB("reach",$node); }
}

sub createSystemFile {
	my $node = shift;
	
	undef %NMIS::systemTable;
	if ( $NMIS::nodeTable{$node}{collect} eq "true" and $pingresult == 100 ) {
		&getNodeInfo($node);
	}
	else {
		$writeSystem = "true"	
	}
	#This makes the nodes file overwrite the getNodeInfo bits.
	$NMIS::systemTable{netType} = $NMIS::nodeTable{$node}{net};
	$NMIS::systemTable{roleType} = $NMIS::nodeTable{$node}{role};
	$NMIS::systemTable{nodeType} = $NMIS::nodeTable{$node}{devicetype};
	
	&writeSystemFile($node);
}

sub getNodeInfo {
	my $node = shift;
	my $message;
	
	my $session;
	my %enterprise = loadCSV($NMIS::config{Enterprise_Table},$NMIS::config{Enterprise_Key},"\t");
	my $OID;
	my $oid_key;
	my @x;
	
	if ($debug) { print returnTime." Getting Node Info using SNMP Ver: $NMIS::systemTable{snmpVer}\n"; }
	
	# only test for SNMPv2 support the first time the device is discovered
	# delete the system file or edit the snmpVer attribute to rediscover SNMP support.
	if ( $NMIS::systemTable{snmpVer} eq "" ) {
		# now open SNMPv2 and see if the device talks that!
		if ($debug) { print returnTime." Testing for SNMPv2 support\n"; }
		($session) = SNMPv2c_Simple->open($node, $NMIS::nodeTable{$node}{community});
		if ( not defined($session) ) { 
			warn returnTime." getNodeInfo, Session is not Defined Goodly to $node.\n"; 
			goto END_getNodeInfo;
		}

		(	$NMIS::systemTable{sysObjectID},
			$NMIS::systemTable{sysUpTime}
		) = $session->snmpget (
			'sysObjectID', 
			'sysUpTime'
		);		
		if ( $SNMP_Simple::errmsg !~ /No answer from/ ) {
			$NMIS::systemTable{snmpVer} = "SNMPv2";
			$session->close();
		}
		else {
			$session->close();
			if ($debug) { print "\tSNMPv2 doesn't work. errmsg=$SNMP_Simple::errmsg\n"; }
			$SNMP_Simple::errmsg = "";
			if ($debug) { print returnTime." Testing for SNMPv1 support\n"; }
			($session) = SNMP_Simple->open($node, $NMIS::nodeTable{$node}{community});
			if ( not defined($session) ) { 
				warn returnTime." getNodeInfo, Session is not Defined Goodly to $node.\n"; 
				goto END_getNodeInfo;
			}
			(	$NMIS::systemTable{sysObjectID},
				$NMIS::systemTable{sysUpTime}
			) = $session->snmpget (
				'sysObjectID', 
				'sysUpTime'
			);
			
			if ( $SNMP_Simple::errmsg !~ /No answer from/ ) {
				$session->close();
				$NMIS::systemTable{snmpVer} = "SNMPv1";
			}
			else {
				if ($debug) { print "\tSNMPv1 doesn't work. errmsg=$SNMP_Simple::errmsg\n"; }
				logMessage("getNodeInfo, $node, SNMP Error. community or SNMP agent problem. errmsg=$SNMP_Simple::errmsg");
				$SNMP_Simple::errmsg = "";
				$NMIS::systemTable{snmpVer} = "none";
				goto END_getNodeInfo;
			}
		}
		if ($debug) { print returnTime." $node supports $NMIS::systemTable{snmpVer} version.\n"; }
	}

	if ($debug) { print returnTime." getNodeInfo opening SNMP Ver: $NMIS::systemTable{snmpVer}\n"; }
	if ( $NMIS::systemTable{snmpVer} eq "SNMPv2" ) {
		($session) = SNMPv2c_Simple->open($node, $NMIS::nodeTable{$node}{community});
	}
	elsif ( $NMIS::systemTable{snmpVer} eq "SNMPv1" ) {
		($session) = SNMP_Simple->open($node, $NMIS::nodeTable{$node}{community});
	}
	if ( not defined($session) ) { 
		$message = "Session is not Defined Goodly to $node.\n"; 
		logMessage("getNodeInfo, $message");
		if ($debug) { print returnTime." getNodeInfo, $message\n"; }
		goto END_getNodeInfo;
	}
	
	(	$NMIS::systemTable{sysDescr},
		$NMIS::systemTable{sysObjectID},
		$NMIS::systemTable{sysUpTime},
		$NMIS::systemTable{sysContact},
		$NMIS::systemTable{sysName},
		$NMIS::systemTable{sysLocation},
		$NMIS::systemTable{sysServices},
		$NMIS::systemTable{ifNumber}
	)
	= $session->snmpget(
		'sysDescr', 
		'sysObjectID', 
		'sysUpTime', 
		'sysContact', 
		'sysName', 
		'sysLocation', 
		'sysServices', 
		'ifNumber'
	);
	if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
		$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
		$SNMP_Simple::errmsg = "";
		logMessage("getNodeInfo, $message");
		if ($debug) { print returnTime." getNodeInfo, $message\n"; }
		goto END_getNodeInfo;
	}
	elsif ( $NMIS::systemTable{sysObjectID} =~ /unprintable BER type/ 
			or $NMIS::systemTable{sysUpTime} =~ /unprintable BER type/ 
			or $NMIS::systemTable{ifNumber} =~ /unprintable BER type/
	) {
		$snmpresult = 0;
		logMessage("getNodeInfo, $node, SNMP ERROR sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}");
		if ($debug)  { print returnTime." $node SNMP Problem sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}\n"; }
		$writeSystem = "true";
		goto END_getNodeInfo;
	}
	elsif ( $NMIS::systemTable{sysObjectID} eq "" 
			and $NMIS::systemTable{sysUpTime} eq "" 
			and $NMIS::systemTable{ifNumber} eq "" 
	) {
		$snmpresult = 0;
		logMessage("getNodeInfo, $node, SNMP elements empty sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}");
		if ($debug)  { print returnTime." $node elements empty sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}\n"; }
		goto END_getNodeInfo;
	}
	
	## KS 1/4/01 - Added this for to support default contacts and locations.
	if ( $NMIS::systemTable{sysContact} eq "" ) { $NMIS::systemTable{sysContact} = "default"; }
	if ( $NMIS::systemTable{sysLocation} eq "" ) { $NMIS::systemTable{sysLocation} = "default"; }

	# Only continue processing if at least a couple of entries are valid.
	if (    $NMIS::systemTable{sysDescr} ne "" and 
		$NMIS::systemTable{sysObjectID} ne "" and 
		$NMIS::systemTable{sysServices} ne ""
	) {	      
		$writeSystem = "true";
		
		($NMIS::systemTable{sysObjectName}) = SNMP_MIB::oid2name($NMIS::systemTable{sysObjectID});

		# Decide on vendor name.
		@x = split(/\./,$NMIS::systemTable{sysObjectID});
		$OID = $x[6];
		if ( $enterprise{$OID}{Enterprise} ne "" ) {
			$NMIS::systemTable{nodeVendor} = $enterprise{$OID}{Enterprise};
		}
		else { $NMIS::systemTable{nodeVendor} =  "Unknown"; }
		if ($debug) { print returnTime." $node vendor=$NMIS::systemTable{nodeVendor} sysObjectID=$NMIS::systemTable{sysObjectID} VendorID=$OID\n"; }

			# Decide on device Capabilities
			if ( $NMIS::systemTable{sysServices} == 2 ) { 
				$NMIS::systemTable{nodeType} = "switch"; 
				$NMIS::systemTable{netType} = "lan";
			}
			elsif ( $NMIS::systemTable{sysServices} == 4 or 
				$NMIS::systemTable{sysServices} == 6
			) { 
				$NMIS::systemTable{nodeType} = "router"; 
				$NMIS::systemTable{netType} = "lan";
			}				      
			elsif ( $NMIS::systemTable{sysServices} == 78 ) { 
				$NMIS::systemTable{nodeType} = "router"; 
				$NMIS::systemTable{netType} = "wan";
			}
			elsif ( $NMIS::systemTable{sysServices} == 72 or
			$NMIS::systemTable{sysServices} == 76
		) { 
			$NMIS::systemTable{nodeType} = "server"; 
			$NMIS::systemTable{netType} = "lan";
		}
		else { $NMIS::systemTable{nodeType} =  $NMIS::systemTable{sysServices}; }
		if ($debug) { print returnTime." $node sysServices=$NMIS::systemTable{sysServices}\n"; }

		# Checking on the Model Type
		if ( $NMIS::systemTable{sysDescr} =~ /WS-C6/i ) { 
			$NMIS::systemTable{nodeModel} = "Catalyst6000"; 
			$NMIS::systemTable{nodeType} = "switch"; 
			$NMIS::systemTable{netType} = "lan";
			$NMIS::systemTable{supported} = "true";
		}
		elsif ( $NMIS::systemTable{sysDescr} =~ /WS-C5|WS-C2948/i ) { 
			$NMIS::systemTable{nodeModel} = "Catalyst5000"; 
			$NMIS::systemTable{nodeType} = "switch"; 
			$NMIS::systemTable{netType} = "lan";
			$NMIS::systemTable{supported} = "true";
		}
		elsif ( $NMIS::systemTable{sysDescr} =~ /C2900XL|C3500XL|CAT2948G/i ) { 
			$NMIS::systemTable{nodeModel} = "CatalystIOS"; 
			$NMIS::systemTable{nodeType} = "switch"; 
			$NMIS::systemTable{netType} = "lan";
			$NMIS::systemTable{supported} = "true";
		}
		elsif ( $NMIS::systemTable{sysDescr} =~ /2820/i ) { 
			$NMIS::systemTable{nodeModel} = "Catalyst2820"; 
			$NMIS::systemTable{nodeType} = "switch"; 
			$NMIS::systemTable{netType} = "lan";
			$NMIS::systemTable{supported} = "false";
		}
		elsif ( $NMIS::systemTable{sysDescr} =~ /MSM/i ) { 
			$NMIS::systemTable{nodeModel} = "CiscoRouterMSM"; 
			$NMIS::systemTable{nodeType} = "router"; 
			$NMIS::systemTable{netType} = "lan";
			$NMIS::systemTable{supported} = "true";
		}
		elsif ( $NMIS::systemTable{sysDescr} =~ /RSM/i ) { 
			$NMIS::systemTable{nodeModel} = "CiscoRouterRSM"; 
			$NMIS::systemTable{nodeType} = "router"; 
			$NMIS::systemTable{netType} = "lan";
			$NMIS::systemTable{supported} = "true";
		}
		elsif ( $NMIS::systemTable{sysDescr} =~ /LS1010/i ) { 
			$NMIS::systemTable{nodeModel} = "CiscoATM"; 
			$NMIS::systemTable{nodeType} = "switch"; 
			$NMIS::systemTable{netType} = "wan";
			$NMIS::systemTable{supported} = "true";
		}
		elsif ( $NMIS::systemTable{sysDescr} =~ /sun/i ) { 
			$NMIS::systemTable{nodeModel} = "SunSolaris"; 
			$NMIS::systemTable{nodeType} = "server"; 
			$NMIS::systemTable{netType} = "lan";
			$NMIS::systemTable{supported} = "true";
		}
		else { 
			# Might be an IOS Cisco Router
			if (    ( $NMIS::systemTable{nodeVendor} eq "Cisco Systems" ) && 
				( $NMIS::systemTable{nodeType} eq "router" ) 
			) { 
				$NMIS::systemTable{nodeModel} = "CiscoRouter"; 
				$NMIS::systemTable{nodeType} = "router"; 
				$NMIS::systemTable{netType} = "wan";
				$NMIS::systemTable{supported} = "true";
			}
			# Other wise must just be generic
			# Expand this to handle Servers, other model routers etc
			else { 
				$NMIS::systemTable{nodeModel} = "generic"; 
				#$NMIS::systemTable{nodeType} = "generic"; 
				#$NMIS::systemTable{netType} = "lan";
				$NMIS::systemTable{supported} = "true";
			}
		}

		# if the device is a cisco router do collect some extra system info
		if (    ( $NMIS::systemTable{nodeType} eq "router" ) &&
			( $NMIS::systemTable{nodeVendor} eq "Cisco" )
		){
			if ($debug)  { print returnTime." Getting Cisco Router Serial Number\n"; }
			( 
				$NMIS::systemTable{chassisVer},
				$NMIS::systemTable{serialNum},
				$NMIS::systemTable{processorRam}
			) = $session->snmpget( 
				'chassisVersion',
				'chassisId',
				'processorRam'
			);
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("getNodeInfo, $message");
				if ($debug) { print returnTime." getNodeInfo, $message\n"; }
				goto END_getNodeInfo;
			}
		}
		elsif ( ( $NMIS::systemTable{nodeType} eq "switch" ) &&
			( $NMIS::systemTable{nodeVendor} eq "Cisco" )
		){
			if ($debug)  { print returnTime." Getting Cisco Switch Serial Number\n"; }
			( 
				$NMIS::systemTable{serialNum},
				$NMIS::systemTable{sysTrafficPeak},
				$NMIS::systemTable{sysTrafficPeakTime}
			) = $session->snmpget( 
				'chassisSerialNumberString',
				'sysTrafficPeak',
				'sysTrafficPeakTime'
			);
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("getNodeInfo, $message");
				if ($debug) { print returnTime." getNodeInfo, $message\n"; }
				goto END_getNodeInfo;
			}
		}
	} # must have SNMP eq something.
	else {
		$writeSystem = "false";
	}
	
	# Finished with the SNMP
	END_getNodeInfo:
	$session->close();
} # end getNodeInfo

# Put all the good SNMP info into a file for caching and also test SNMP for use.
sub updateUptime {
	my $node = shift;
	my $newUpTime;
	my $oldUpTime;
	my $session;

    if ($debug)  { print returnTime." Updating UPTIME Info\n"; }
	
	# OPEN the SNMP Session 
	if ( $NMIS::systemTable{snmpVer} eq "SNMPv2" ) {
		($session) = SNMPv2c_Simple->open($node, $NMIS::nodeTable{$node}{community});
	}
	else {
		($session) = SNMP_Simple->open($node, $NMIS::nodeTable{$node}{community});
	}
	if ( not defined($session) ) { 
		warn returnTime." updateUptime, Session is not Defined Goodly to $node.\n"; 
		goto END_updateUptime;
	}
	else {
		(	$snmpTable{sysObjectID},
			$snmpTable{sysUpTime},
			$snmpTable{ifNumber}
		) = $session->snmpget (
			'sysObjectID', 
			'sysUpTime', 
			'ifNumber'
		);
		if ($debug)  { print "\tsysObjectID=$snmpTable{sysObjectID}\tifNumber=$snmpTable{ifNumber}\tsysUpTime=$snmpTable{sysUpTime}\n"; }
	
		# Check if SNMP returned anything at all.
		if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
			$snmpresult = 0;
			if ($debug)  { print "\t$node, SNMP ERROR, errmsg=$SNMP_Simple::errmsg\n"; }
			logMessage("updateUptime, $node, SNMP ERROR errmsg=$SNMP_Simple::errmsg");
			notify(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "SNMP Down", details => "");
			$SNMP_Simple::errmsg = "";
			goto END_updateUptime;
		}
		elsif ( $snmpTable{sysObjectID} =~ /unprintable BER type/ 
				or $snmpTable{sysUpTime} =~ /unprintable BER type/ 
				or $snmpTable{ifNumber} =~ /unprintable BER type/
		) {
			$snmpresult = 0;
			logMessage("updateUptime, $node, SNMP ERROR sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}");
			if ($debug)  { print returnTime." $node SNMP Problem sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}\n"; }
			$SNMP_Simple::errmsg = "";
			goto END_updateUptime;
		}
		elsif ( $snmpTable{sysObjectID} eq "" 
				and $snmpTable{sysUpTime} eq "" 
				and $snmpTable{ifNumber} eq "" 
		) {
			$snmpresult = 0;
			logMessage("updateUptime, $node, SNMP elements empty sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}");
			if ($debug)  { print returnTime." $node elements empty sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}\n"; }
			$SNMP_Simple::errmsg = "";
			goto END_updateUptime;
		}
		# check if SNMP matches cached stuff but only if the bugger is up ie did the SNMP return something.
		elsif ( ( $snmpTable{sysObjectID} ne $NMIS::systemTable{sysObjectID}
				or $snmpTable{ifNumber} ne $NMIS::systemTable{ifNumber} 
			) and ( 
				$snmpTable{sysObjectID} ne "" 
				and $snmpTable{ifNumber} ne "" 
		) ) {
			$snmpresult = 100;
			checkEvent(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "SNMP Down", details => "");
		    if ( $snmpTable{ifNumber} ne $NMIS::systemTable{ifNumber} ) {
				logMessage("updateUptime, $node, Number of interfaces changed previously $NMIS::systemTable{ifNumber} now $snmpTable{ifNumber}");
			}
			if ( $snmpTable{sysObjectID} ne $NMIS::systemTable{sysObjectID} ) {
				logMessage("updateUptime, $node, Device type/model changed $NMIS::systemTable{sysObjectID} now $snmpTable{sysObjectID}");
			}
			&createSystemFile($node);
			&createInterfaceFile($node); 
		}
		# SNMP returned some stuff so store and update etc.
		else { 
			$snmpresult = 100;
			checkEvent(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "SNMP Down", details => "");
		    # Read the uptime from the system info file from the last time it was polled
		    $snmpTable{sysUpTime} =~ s/,/ /g;
		    $snmpTable{sysUpTime} =~ s/  / /g;
			$newUpTime = convertUpTime($snmpTable{sysUpTime});
			$oldUpTime = convertUpTime($NMIS::systemTable{sysUpTime});
		
			if ($debug)  { print returnTime." sysUpTime: Old=$NMIS::systemTable{sysUpTime} New=$snmpTable{sysUpTime}\n"; }
		
			if ( $newUpTime < $oldUpTime and $newUpTime ne "" ) {		
				if ($debug)  { print returnTime." NODE RESET: Old sysUpTime=$oldUpTime: New sysUpTime=$newUpTime\n"; }
				notify(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Node Reset", details => "Old_sysUpTime=$NMIS::systemTable{sysUpTime} New_sysUpTime=$snmpTable{sysUpTime}");
			}
		
			$NMIS::systemTable{oldsysUpTime} = $NMIS::systemTable{sysUpTime};
			$NMIS::systemTable{sysUpTime} = $snmpTable{sysUpTime};
			## KS 16/4/2001 - Found that if any details change in nodes.dat not updating system
			$NMIS::systemTable{netType} = $NMIS::nodeTable{$node}{net};
			$NMIS::systemTable{roleType} = $NMIS::nodeTable{$node}{role};
			$NMIS::systemTable{nodeType} = $NMIS::nodeTable{$node}{devicetype};
			
			$writeSystem = "true";
			&writeSystemFile($node);
		}
		# Finished with the SNMP
	}
	END_updateUptime:
	$session->close();
} # end updateUptime

sub writeSystemFile {
	my $node = shift;
	my $key;
	
	if ($debug) { print returnTime." Writing the System File\n"; }
	
	if ( $writeSystem eq "true" ) {
		# Deal with the nasty \n \r im sysdescr just remove the \n to be put back later.
		$NMIS::systemTable{sysDescr} =~ s/\n/ /g;
		
		# get damn comma's out:
		$NMIS::systemTable{sysUpTime} =~ s/,/ /g;
		
		## KS 1/5/01 Made a little more flexible.
		open(OUTFILE, ">$NMIS::config{'<nmis_var>'}/$node.dat") or warn returnTime." writeSystemFile, Couldn't open file $NMIS::config{'<nmis_var>'}/$node.dat for writing. $!\n";
		foreach $key (keys %NMIS::systemTable) {
			print OUTFILE "$key=$NMIS::systemTable{$key}\n";
		}
		close(OUTFILE); 
	}
	else {
	# only print message if supposed to collect and no SNMP available.
		if ( $NMIS::nodeTable{$node}{collect} eq "true" ) {
			logMessage("writeSystemFile, $node, not writing system file writeSystem=$writeSystem as no SNMP to cache");
		}
	}
}

sub updateRRDDB {
	my $type = shift;
	my $node = shift;
	my $string = shift; 

	my $updateDatabase;
	my $time  = 30*int(time/30);
	my @options;
	my $ERROR;
	my $operStatus;
	my $index;

	if ($debug>4) { print returnTime." Starting Update Process $type\n"; }

	if ( $type eq "reach" ) {

		$updateDatabase = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-reach.rrd";

		@options = (
			"-t", "reachability:availability:responsetime:health",
			"N:$reach{reachability}:$reach{availability}:$reach{responsetime}:$reach{health}"
		);
		if ($debug>5) { print "Update data with $reach{reachability}:$reach{availability}:$reach{responsetime}:$reach{health}\n"; } 
	} # if reach is true baby
	# Is it a Cisco Router Health
	elsif ( $type eq "interface" ) {
		$updateDatabase = "$NMIS::config{database_root}/interface/$NMIS::systemTable{nodeType}/$node/$node-".convertIfName($string).".rrd";
		
		# Calculate Operational Status
		if (    ( $ifStats{ifOperStatus} == 1 ) or
			( $ifStats{ifOperStatus} == 5 ) or
			( $string =~ /ser.*[0-9]:.*/i ) or
			( $string =~ /virtual/i ) or
			( $string =~ /BRI[0-9]:[0-9]/i )
		) { $operStatus = 100; }
		else { $operStatus = 0; }

		#print "operStatus=$operStatus\n"; 
		
		# While updating start calculating the total availability of the device
		$reach{operStatus} = $reach{operStatus} + $operStatus;
		$reach{operCount} = $reach{operCount} + 1;

		if ( $ifStats{ifInOctets} eq "" ) { $ifStats{ifInOctets} = 0 }
		if ( $ifStats{ifOutOctets} eq "" ) { $ifStats{ifOutOctets} = 0 }

		@options = (
			"-t", "ifInOctets:ifOutOctets:ifOperStatus",
			"N:$ifStats{ifInOctets}:$ifStats{ifOutOctets}:$operStatus"
		);
		if ($debug>2) { print "    Updating with $updateDatabase with $ifStats{ifInOctets}:$ifStats{ifOutOctets}:$operStatus\n"; }
	}
	elsif ( $type eq "health" and $NMIS::systemTable{nodeModel} =~ /CiscoRouter|CatalystIOS/ ) {
		$updateDatabase = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		@options = (
			"-t", "avgBusy1:avgBusy5:MemoryUsedPROC:MemoryFreePROC:MemoryUsedIO:MemoryFreeIO:bufferElFree:bufferElHit:bufferFail:ipInReceives:ipForwDatagrams:ipInDelivers:ipOutNoRoutes",
			"N:$snmpTable{avgBusy1}:$snmpTable{avgBusy5}:$snmpTable{MemoryUsedPROC}:$snmpTable{MemoryFreePROC}:$snmpTable{MemoryUsedIO}:$snmpTable{MemoryFreeIO}:$snmpTable{bufferElFree}:$snmpTable{bufferElHit}:$snmpTable{bufferFail}:$snmpTable{ipInReceives}:$snmpTable{ipForwDatagrams}:$snmpTable{ipInDelivers}:$snmpTable{ipOutNoRoutes}"
		);
		if ($debug>2) { print "    Updating with $updateDatabase with $snmpTable{avgBusy1}:$snmpTable{avgBusy5}:$snmpTable{MemoryUsedPROC}:$snmpTable{MemoryFreePROC}:\n"; }
	}
	elsif ( $type eq "health" and $NMIS::systemTable{nodeModel} =~ /Catalyst6000/ ) {
		$updateDatabase = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		@options = (
			"-t", "avgBusy1:avgBusy5:MemoryUsedDRAM:MemoryFreeDRAM:MemoryUsedMBUF:MemoryFreeMBUF:MemoryUsedCLUSTER:MemoryFreeCLUSTER:sysTraffic:TopChanges",
			"N:$snmpTable{avgBusy1}:$snmpTable{avgBusy5}:$snmpTable{MemoryUsedDRAM}:$snmpTable{MemoryFreeDRAM}:$snmpTable{MemoryUsedMBUF}:$snmpTable{MemoryFreeMBUF}:$snmpTable{MemoryUsedCLUSTER}:$snmpTable{MemoryFreeCLUSTER}:$snmpTable{sysTraffic}:$snmpTable{TopChanges}"
		);
		if ($debug>2) { print "    Updating with $updateDatabase with $snmpTable{avgBusy1}:$snmpTable{avgBusy5}:$snmpTable{MemoryUsedDRAM}:$snmpTable{MemoryFreeDRAM}\n"; }
	}

	RRDs::update $updateDatabase, @options;

	if ($ERROR = RRDs::error) {
		logMessage("updateRRDDB, $node, update error database=$updateDatabase: $ERROR: options = @options");
	}

	if ($debug>1) { print returnTime." Database = $updateDatabase Model=$NMIS::systemTable{nodeModel}\n"; }
}

sub createRRDDB {
	my $type = shift;
	my $node = shift;

	my $time  = 30*int(time/30);
	my @options;
	my ($STEP, $RRD1);
	my $START = $time;
	
	# Does the database exist already?
	# Check if the RRD Database Exists but is ReadOnly
	# Maybe this should check for valid directory or not.
	if ( -f $database ) { 
		print "Database Exists but is readonly to you!\n";
	}
	# It doesn't so create it
	else { 
		if ( $ifSpeed <= 0 ) { $ifSpeed = 1000000000; }
		# Check what type of database to create
		# Is it a Generic Interface Type
		if ( $type eq "interface" ) {
			@options = (
				"-b", $START, "-s", 300,
				"DS:ifInOctets:COUNTER:900:0:$ifSpeed", 
				"DS:ifOutOctets:COUNTER:900:0:$ifSpeed", 
				"DS:ifOperStatus:GAUGE:900:0:100", 
				"RRA:AVERAGE:0.5:1:2304",
				"RRA:AVERAGE:0.5:6:1536",
				"RRA:AVERAGE:0.5:24:2268",
				"RRA:AVERAGE:0.5:288:1890",
				"RRA:MAX:0.5:1:2304",
				"RRA:MAX:0.5:6:1536",
				"RRA:MAX:0.5:24:2268",
				"RRA:MAX:0.5:288:1890",
				"RRA:MIN:0.5:1:2304",
				"RRA:MIN:0.5:6:1536",
				"RRA:MIN:0.5:24:2268",
				"RRA:MIN:0.5:288:1890"
			);
		}
		# Is it a Reachability Interface Type
		elsif ( $type eq "reach" ) {
			@options = (
				"-b", $START, "-s", 300,
				"DS:reachability:GAUGE:900:0:100", 
				"DS:availability:GAUGE:900:0:100", 
				"DS:responsetime:GAUGE:900:0:U", 
				"DS:health:GAUGE:900:0:100", 
				"RRA:AVERAGE:0.5:1:2304",
				"RRA:AVERAGE:0.5:6:1536",
				"RRA:AVERAGE:0.5:24:2268",
				"RRA:AVERAGE:0.5:288:1890",
				"RRA:MAX:0.5:1:2304",
				"RRA:MAX:0.5:6:1536",
				"RRA:MAX:0.5:24:2268",
				"RRA:MAX:0.5:288:1890",
				"RRA:MIN:0.5:1:2304",
				"RRA:MIN:0.5:6:1536",
				"RRA:MIN:0.5:24:2268",
				"RRA:MIN:0.5:288:1890"
			);
		} # Reachability thingy
		# Is it a Cisco Router Health
		elsif ( $NMIS::systemTable{nodeModel} =~ /CiscoRouter|CatalystIOS/ and $type eq "health" ) {
			@options = (
				"-b", $START, "-s", 300,
				"DS:avgBusy1:GAUGE:900:U:U", 
				"DS:avgBusy5:GAUGE:900:U:U", 
				"DS:MemoryUsedPROC:GAUGE:900:U:U", 
				"DS:MemoryFreePROC:GAUGE:900:U:U", 
				"DS:MemoryUsedIO:GAUGE:900:U:U", 
				"DS:MemoryFreeIO:GAUGE:900:U:U", 
				"DS:bufferElFree:GAUGE:900:U:U", 
				"DS:bufferElHit:COUNTER:900:U:U", 
				"DS:bufferFail:COUNTER:900:U:U", 
				"DS:ipInReceives:COUNTER:900:U:U", 
				"DS:ipForwDatagrams:COUNTER:900:U:U", 
				"DS:ipInDelivers:COUNTER:900:U:U", 
				"DS:ipOutNoRoutes:COUNTER:900:U:U", 
				"RRA:AVERAGE:0.5:1:2304",
				"RRA:AVERAGE:0.5:6:1536",
				"RRA:AVERAGE:0.5:24:2268",
				"RRA:AVERAGE:0.5:288:1890",
				"RRA:MAX:0.5:1:2304",
				"RRA:MAX:0.5:6:1536",
				"RRA:MAX:0.5:24:2268",
				"RRA:MAX:0.5:288:1890",
				"RRA:MIN:0.5:1:2304",
				"RRA:MIN:0.5:6:1536",
				"RRA:MIN:0.5:24:2268",
				"RRA:MIN:0.5:288:1890"
			);
		}
		# Is it a Cisco Catalyst 6000 Switch Health Stats
		elsif ( $NMIS::systemTable{nodeModel} =~ /Catalyst6000/ and $type eq "health" ) {
			@options = (
				"-b", $START, "-s", 300,
				"DS:avgBusy1:GAUGE:900:U:U", 
				"DS:avgBusy5:GAUGE:900:U:U", 
				"DS:MemoryUsedDRAM:GAUGE:900:U:U", 
				"DS:MemoryFreeDRAM:GAUGE:900:U:U", 
				"DS:MemoryUsedMBUF:GAUGE:900:U:U", 
				"DS:MemoryFreeMBUF:GAUGE:900:U:U", 
				"DS:MemoryUsedCLUSTER:GAUGE:900:U:U", 
				"DS:MemoryFreeCLUSTER:GAUGE:900:U:U", 
				"DS:sysTraffic:GAUGE:900:U:U", 
				"DS:TopChanges:COUNTER:900:U:U", 
				"RRA:AVERAGE:0.5:1:2304",
				"RRA:AVERAGE:0.5:6:1536",
				"RRA:AVERAGE:0.5:24:2268",
				"RRA:AVERAGE:0.5:288:1890",
				"RRA:MAX:0.5:1:2304",
				"RRA:MAX:0.5:6:1536",
				"RRA:MAX:0.5:24:2268",
				"RRA:MAX:0.5:288:1890",
				"RRA:MIN:0.5:1:2304",
				"RRA:MIN:0.5:6:1536",
				"RRA:MIN:0.5:24:2268",
				"RRA:MIN:0.5:288:1890"
			);
		}
		# Is it Cisco Router Interface

		if ($debug) { print returnTime." Creating RRD $database starting at $time.\n"; }
		logMessage("createRRDDB, $node, Creating RRD $database starting at $time");
		RRDs::create $database, @options;
		my $ERROR = RRDs::error;
		if ($ERROR) {
			logMessage("createRRDDB, $node, unable to create $database: $ERROR");
		}
	} # else
} # end createRRDDB

# Create the Interface configuration from SNMP Stuff!!!!!
sub createInterfaceFile {
	my $node = shift;
	my $interfacefile = "$NMIS::config{'<nmis_var>'}/$node-interface.dat";
	if ($debug) { print returnTime." Creating the interface file: $interfacefile\n"; }
	if ($debug) { print "\tNote the MIB variable 1.3.6.1.2.1.31.1.1.1.1 is the ifName MIB which is not supported by older SNMP agents.\n"; }
	
	# Create a means to maintain which devices are supported so far.
	my $validModel = "false";
	my @ifNames;
	my $ifIndex;
	my $ourPortName;
	my $ourPortNumber;
	my @intTable;
	my @interfaces;
	my $interface;
	my %interfaceTable;
	my $session;
	my $message;
	my %ifTypeDefs;
	
	my $numInterfaces = $NMIS::systemTable{ifNumber};
	
	#NMIS::loadInterfaceTypes;
	%ifTypeDefs = &loadCSV($NMIS::config{ifTypes_Table},$NMIS::config{ifTypes_Key},"\t");
	
	if ( $NMIS::systemTable{snmpVer} eq "SNMPv2" ) {
		($session) = SNMPv2c_Simple->open($node, $NMIS::nodeTable{$node}{community});
	}
	else {
		($session) = SNMP_Simple->open($node, $NMIS::nodeTable{$node}{community});
	}
	if ( not defined($session) ) { 
		warn returnTime." createInterfaceFile, Session is not Defined Goodly to $node.\n"; 
		goto END_createInterfaceFile;
	}
	
	# An SNMP session exists already and we know the node type
	# Depending on the node type get different information
	if ( $NMIS::systemTable{nodeModel} =~ /CiscoRouter|CiscoATM|CatalystIOS/ ) {
		if ($debug) { print "\tCollecting $NMIS::systemTable{ifNumber} Interfaces \n"; }
		$validModel = "true";
		# Loop to get interface information for Cisco router
		#for ( $index = 1; $index <= $numInterfaces; ++$index ) {
		my %ifIndexTable = $session->snmpgettablean('ifIndex');
		foreach $index (keys %ifIndexTable) {
			# Get Interface Information
			(	$interfaces[$index]{ifDescr},
				$interfaces[$index]{ifType},
				$interfaces[$index]{ifSpeed},
				$interfaces[$index]{ifAdminStatus},
				$interfaces[$index]{ifOperStatus},
				$interfaces[$index]{ifLastChange},
				$interfaces[$index]{Description}
			) = $session->snmpget(  
				'ifDescr'.".$index",
				'ifType'.".$index",
				'ifSpeed'.".$index",
				'ifAdminStatus'.".$index",
				'ifOperStatus'.".$index",
				'ifLastChange'.".$index",
				'ifAlias'.".$index"
			); 
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("createInterfaceFile, $message");
				if ($debug) { print returnTime." createInterfaceFile, $message\n"; }
				goto END_createInterfaceFile;
			}
			if ( $interfaces[$index]{ifDescr} eq "" ) { 
				$interfaces[$index]{ifDescr} = "null";
				++$numInterfaces; 
			}
		 
			if ($debug>3) { print "\tifIndex=$index descr=$interfaces[$index]{ifDescr} admin=$interfaces[$index]{ifAdminStatus} oper=$interfaces[$index]{ifOperStatus} speed=$interfaces[$index]{ifSpeed}\n"; }
			
			# Set the ifType to be something meaningful!!!!
			$interfaces[$index]{ifType} = $ifTypeDefs{$interfaces[$index]{ifType}}{ifType};
			
			# Set AdminStatus and OperStatus to be a word!!
			$interfaces[$index]{ifAdminStatus} = $interfaceStatus[$interfaces[$index]{ifAdminStatus}];
			$interfaces[$index]{ifOperStatus} = $interfaceStatus[$interfaces[$index]{ifOperStatus}];
			
			# Just check if it is an Frame Relay sub-interface
			if ( ( $interfaces[$index]{ifType} eq "frame-relay" and $interfaces[$index]{ifDescr} =~ /\./ ) ) {
				$interfaces[$index]{ifType} = "frame-relay-subinterface";
			}
	} # FOR LOOP
	} # ( $NMIS::systemTable{nodeModel} eq "CiscoRouter" or CiscoATM)
	elsif (	$NMIS::systemTable{nodeModel} =~ /Catalyst6000/ or
			$NMIS::systemTable{nodeModel} =~ /Catalyst5000/
	) {
		$validModel = "true";
		my @ENUMportType;
		$ENUMportType[18] = "e10a100BaseTX";
		$ENUMportType[27] = "e1000BaseLX";
		$ENUMportType[28] = "e1000BaseSX";
		$ENUMportType[31] = "empty";
		$ENUMportType[32] = "e1000BaseLH";
		my @ENUMportSpantreeFastStart = ("null","enabled","disabled");
		my @ENUMportOperStatus = ("null","other","ok","minorFault","majorFault");
		my @ENUMportDuplex = ("null","half","full","disagree","auto");
		
		# Get a list of interface ifIndexes and ifName mappings
		@intTable = $session->snmpgettablek('ifIndex', 'ifName', 'ifAdminStatus', 'ifLastChange', 'ifSpeed');
		if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
			$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
			$SNMP_Simple::errmsg = "";
			logMessage("createInterfaceFile, $message");
			if ($debug) { print returnTime." createInterfaceFile, $message\n"; }
			goto END_createInterfaceFile;
		}
		if ($debug) { print "\tCatalyst Found $#intTable interfaces\n"; }
		
		foreach $ifIndex (1 .. $#intTable) {
			if ($intTable[$ifIndex]->{'ifName'} ne "" ) {
				if ($debug>3) { print "\t$index,$ifIndex,$intTable[$ifIndex]->{'ifIndex'},$intTable[$ifIndex]->{'ifName'}\n"; }

				$intTable[$ifIndex]->{'ifName'} =~ s/,/:/g;
				# Assign table into normalised interfaceTable 
				$interfaces[$ifIndex]{ifIndex} = $intTable[$ifIndex]->{'ifIndex'};
				$interfaces[$ifIndex]{ifDescr} = $intTable[$ifIndex]->{'ifName'};
				$interfaces[$ifIndex]{ifAdminStatus} = $intTable[$ifIndex]->{'ifAdminStatus'};
				$interfaces[$ifIndex]{ifSpeed} = $intTable[$ifIndex]->{'ifSpeed'};
				$interfaces[$ifIndex]{ifLastChange} = $intTable[$ifIndex]->{'ifLastChange'};
	
			# if the Interface is a num/num ie a catalyst port then get additional stats
				if ($intTable[$ifIndex]->{ifName} =~ /^\d{1,2}\/\d{1,2}/ ) {
			    if ($debug>3) { print "\tget more details: index=$ifIndex\tifName=$intTable[$ifIndex]->{'ifName'}\tifDescr=$interfaces[$ifIndex]{ifDescr}\n"; }
					$ourPortNumber =  $intTable[$ifIndex]->{ifName};
					$ourPortNumber =~ s/\//\./;
					
					(       
						$interfaces[$ifIndex]{Description}, 
						$interfaces[$ifIndex]{ifType}, 
						$interfaces[$ifIndex]{ifOperStatus}, 
						$interfaces[$ifIndex]{portDuplex},
						$interfaces[$ifIndex]{portSpantreeFastStart},
						$interfaces[$ifIndex]{vlanPortVlan},
						$interfaces[$ifIndex]{portAdminSpeed}
					) = $session->snmpget(
						'portName'.".$ourPortNumber", 
						'portType'.".$ourPortNumber", 
						'portOperStatus'.".$ourPortNumber", 
						'portDuplex'.".$ourPortNumber",
						'portSpantreeFastStart'.".$ourPortNumber",
						'vlanPortVlan'.".$ourPortNumber",
						'portAdminSpeed'.".$ourPortNumber"
					);
					if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
						$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
						$SNMP_Simple::errmsg = "";
						logMessage("createInterfaceFile, $message");
						if ($debug) { print returnTime." createInterfaceFile, $message\n"; }
						goto END_createInterfaceFile;
					}
					
					# Turn Enumerated types into words for each value
					# portOperStatus ENUMs
					$interfaces[$ifIndex]{ifOperStatus} = $ENUMportOperStatus[$interfaces[$ifIndex]{ifOperStatus}];
					
					# portDuplex ENUMs
					$interfaces[$ifIndex]{portDuplex} = $ENUMportDuplex[$interfaces[$ifIndex]{portDuplex}];
					
					# portSpantreeFastStart ENUMs
					$interfaces[$ifIndex]{portSpantreeFastStart} = $ENUMportSpantreeFastStart[$interfaces[$ifIndex]{portSpantreeFastStart}]; 
					
					# portType ENUMs
					$interfaces[$ifIndex]{ifType} = $ENUMportType[$interfaces[$ifIndex]{ifType}];
					# Because auto is shown as a speed of 1 make it change.
					# portAdminSpeed ENUMs
					if ( $interfaces[$ifIndex]{portAdminSpeed} == 1 ) {
						$interfaces[$ifIndex]{portAdminSpeed} = "auto"; 
					}
				} # if interface is num/num
				# must be a vlan or GEC or something else.
				else {
					if ( 	$interfaces[$ifIndex]{ifDescr} =~ /vlan/i ) {
						$interfaces[$ifIndex]{ifType} = "VLAN";
						$interfaces[$ifIndex]{ifSpeed} = "1000000000";
					}
					elsif ( $interfaces[$ifIndex]{ifDescr} =~ /gec/i ) {
						$interfaces[$ifIndex]{ifType} = "GIG EtherChannel";
					}
					elsif ( $interfaces[$ifIndex]{ifDescr} =~ /sl0/i ) {
						$interfaces[$ifIndex]{ifType} = "SLIP";
					}
					elsif ( $interfaces[$ifIndex]{ifDescr} =~ /sc0/i ) {
						$interfaces[$ifIndex]{ifType} = "Management Ethernet";
					}
					$interfaces[$ifIndex]{ifSpeed} = $intTable[$ifIndex]->{'ifSpeed'};
					$interfaces[$ifIndex]{ifOperStatus} = $intTable[$ifIndex]->{'ifOperStatus'};
					
					$interfaces[$ifIndex]{ifOperStatus} = $interfaceStatus[$interfaces[$ifIndex]{ifOperStatus}];
				} # else other
				$interfaces[$ifIndex]{ifAdminStatus} = $interfaceStatus[$interfaces[$ifIndex]{ifAdminStatus}];
		    } # if name is not ""
		} # Foreach loop
	}
	elsif ( $NMIS::systemTable{nodeModel} =~ /Sun|generic/ ) {
		if ($debug) { print "Collecting $NMIS::systemTable{ifNumber} Interfaces \n"; }
		$validModel = "true";
		
		# Loop to get interface information from SNMP
		my %ifIndexTable = $session->snmpgettablean('ifIndex');
		foreach $index (keys %ifIndexTable) {
			# Get Interface Information
			(       $interfaces[$index]{ifDescr},
				$interfaces[$index]{ifType},
				$interfaces[$index]{ifSpeed},
				$interfaces[$index]{ifAdminStatus},
				$interfaces[$index]{ifOperStatus},
				$interfaces[$index]{ifLastChange}
			) = $session->snmpget(  
				'ifDescr'.".$index",
				'ifType'.".$index",
				'ifSpeed'.".$index",
				'ifAdminStatus'.".$index",
				'ifOperStatus'.".$index",
				'ifLastChange'.".$index"
			); 
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("createInterfaceFile, $message");
				if ($debug) { print returnTime." createInterfaceFile, $message\n"; }
				goto END_createInterfaceFile;
			}
			if ( $interfaces[$index]{ifDescr} eq "" ) { 
				$interfaces[$index]{ifDescr} = "null";
				++$numInterfaces; 
			}
			 
			if ($debug) { print "\tifIndex=$index descr=$interfaces[$index]{ifDescr} admin=$interfaces[$index]{ifAdminStatus} oper=$interfaces[$index]{ifOperStatus} speed=$interfaces[$index]{ifSpeed}\n"; }
			
			# Set the ifType to be something meaningful!!!!
			$interfaces[$index]{ifType} = $ifTypeDefs{$interfaces[$index]{ifType}}{ifType};
	
			# Interfaces with N/A in them are bad!
			if ( $interfaces[$index]{ifSpeed} eq "0" and $interfaces[$index]{ifType} =~ /ethernet/i ) {
				$interfaces[$index]{ifSpeed} = 10000000;
			}
			# Set AdminStatus and OperStatus to be a word!!
			$interfaces[$index]{ifAdminStatus} = $interfaceStatus[$interfaces[$index]{ifAdminStatus}];
			$interfaces[$index]{ifOperStatus} = $interfaceStatus[$interfaces[$index]{ifOperStatus}];
		} # FOR LOOP
	} # Sun or generic
	else {
		$validModel eq "false"; 
	}
	# IP Address Table
	#ip.ipAddrTable.ipAddrEntry.ipAdEntAddr.10.64.100.145 = ipAdEntAddr: 10.64.100.145
	#ip.ipAddrTable.ipAddrEntry.ipAdEntIfIndex.10.64.100.145 = 4
	#ip.ipAddrTable.ipAddrEntry.ipAdEntNetMask.10.64.100.145 = ipAdEntAddr: 255.255.255.248
	#get the IP addressing stuff
	if ( $validModel eq "true") {
		my %ifIndexTable = $session->snmpgettablean('ipAdEntIfIndex');
		my %ifMaskTable = $session->snmpgettablean('ipAdEntNetMask');
		if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
			$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
			$SNMP_Simple::errmsg = "";
			logMessage("createInterfaceFile, $message");
			if ($debug) { print returnTime." createInterfaceFile, $message\n"; }
			goto END_createInterfaceFile;
		}
		foreach $index (keys %ifIndexTable) {
			if ($debug) { 	
				print "\tip addresses: index=$index ifIndex=$ifIndexTable{$index} mask=$ifMaskTable{$index}\n";	
			}
			$interfaces[$ifIndexTable{$index}]{ipAdEntAddr} = $index;
			$interfaces[$ifIndexTable{$index}]{ipAdEntNetMask} = $ifMaskTable{$index};
			($interfaces[$ifIndexTable{$index}]{ipSubnet},$interfaces[$ifIndexTable{$index}]{ipSubnetBits}) = ipSubnet(address => $index, mask => $ifMaskTable{$index});
		}
	}
	
	# Process the table and make any changes necessary
	if ( $validModel eq "true") {
		# Now we have a nicely populated Interface Table lets write it out to a file
		if ($debug) { print returnTime." Interface List for $node\n"; }
	
	    foreach $index (1 .. $#interfaces) {
	
			if ( $interfaces[$index]{ifDescr} ne "" ) {
				#Decide if the interface is one that we can do stats on or not based on Description and ifType and AdminStatus
				# If the interface is admin down no stats
				if (	$interfaces[$index]{ifAdminStatus} eq "down" or
						$interfaces[$index]{ifAdminStatus} eq "testing" or
						$interfaces[$index]{ifAdminStatus} eq "null" or
						$interfaces[$index]{ifDescr} eq "null" or
						$collect eq "false" 
				) {
					$interfaces[$index]{collect} = "false";
				} 
				# if it is a router and the interface name has any of these strings
				elsif ( $NMIS::systemTable{nodeModel} =~ /router|ciscoatm/i ) {
					if (
						$interfaces[$index]{ifDescr} =~ /sublayer|null|tun|controller|e0|e1|async|bri[0-9]$|virtual|Bearer Channel/i or
						#I would like to have t0, t1 and t3 but this stops ethernet0, etc.
						#$interfaces[$index]{ifDescr} =~ /sublayer|null|tun|controller|e0|e1|t0|t1|t3|async|bri[0-9]$|virtual|Bearer Channel/i or
						# Found problems with Cisco ATM interfaces on new IOS
						$interfaces[$index]{ifDescr} =~ /atm layer|atm subif/i or
						# Nothing to collect on Voice interfaces
						$interfaces[$index]{ifDescr} =~ /FXS|FXO|Voice Encapsulation|Voice Over IP/ or
						# or if the interface is of this type
						#$interfaces[$index]{ifType} =~ /lapd|ds0|other|softwareloopback||isdn(.*)x\.25|isdn s\/t/i
						$interfaces[$index]{ifType} =~ /lapd|ds0|ds1|other|softwareloopback|isdn/i
						# I thought this would be a better way to do this but need regex to pass in.
						#$NMIS::config{no_collect_ifDescr} =~ $interfaces[$index]{ifDescr} or
						#$NMIS::config{no_collect_ifType} =~ $interfaces[$index]{ifType}
					) { $interfaces[$index]{collect} = "false"; }
					else { $interfaces[$index]{collect} = "true"; }
				}
				elsif ( $NMIS::systemTable{nodeModel} =~ /CatalystIOS/ ) {
					if (	$interfaces[$index]{Description} eq "" or
							$interfaces[$index]{ifType} !~ /ethernet/i
					) { $interfaces[$index]{collect} = "false"; }
					else { $interfaces[$index]{collect} = "true"; }
				}
				# if it is a catalyst and the interface name has any of these strings
				elsif ( $NMIS::systemTable{nodeModel} =~ /Catalyst6000|Catalyst5000/ and $interfaces[$index]{ifDescr} !~ /gec/i ) {
					if (    
						# has to have the port up at the time, this will get rid of PC's and servers that have moved
						$interfaces[$index]{ifOperStatus} eq "other" or
						# has to have one of these words in its description
						## KS 8/4/01 Uncomment for more specific matching of interfaces
						#( $interfaces[$index]{Description} !~ /trunk|channel|data|link|backbone/i ) ||
						# blank decription and not VLAN or GEC
						$interfaces[$index]{Description} eq ""
					) { $interfaces[$index]{collect} = "false"; }
					else { $interfaces[$index]{collect} = "true"; }
				}
				elsif ( $NMIS::systemTable{nodeModel} =~ /sun|generic/i ) {
					if ( 	$interfaces[$index]{ifType} =~ /loopback/i 
						#or $interfaces[$index]{Description} eq "" 
					) { $interfaces[$index]{collect} = "false"; }
					else { $interfaces[$index]{collect} = "true"; }
				} 
				else { $interfaces[$index]{collect} = "true"; }
				
				# Remove Nasty comma's from Interface Last Change!
				$interfaces[$index]{ifLastChange} =~ s/,/ /g;
				
				if ($debug) { print "\t$index\t$interfaces[$index]{'collect'}\t$interfaces[$index]{'ifDescr'}\t$interfaces[$index]{'ifType'}\t\"$interfaces[$index]{'Description'}\"\n"; }
				if ($debug>3) { print "\t$index,$interfaces[$index]{ifDescr},$interfaces[$index]{'collect'},$interfaces[$index]{'ifType'},$interfaces[$index]{'ifSpeed'},$interfaces[$index]{'ifAdminStatus'},$interfaces[$index]{'ifOperStatus'},$interfaces[$index]{'ifLastChange'},$interfaces[$index]{'Description'}\n"; }
	
				$interface = convertIfName($interfaces[$index]{ifDescr});
				$interfaceTable{$interface}{interface} = $interface;
				$interfaceTable{$interface}{ifIndex} = $index;
				$interfaceTable{$interface}{ifDescr} = $interfaces[$index]{ifDescr};
				$interfaceTable{$interface}{collect} = $interfaces[$index]{collect};
				$interfaceTable{$interface}{ifType} = $interfaces[$index]{ifType};
				$interfaceTable{$interface}{ifSpeed} = $interfaces[$index]{ifSpeed};
				$interfaceTable{$interface}{ifAdminStatus} = $interfaces[$index]{ifAdminStatus};
				$interfaceTable{$interface}{ifOperStatus} = $interfaces[$index]{ifOperStatus};
				$interfaceTable{$interface}{ifLastChange} = $interfaces[$index]{ifLastChange};
				$interfaceTable{$interface}{Description} = $interfaces[$index]{Description};
				$interfaceTable{$interface}{portDuplex} = $interfaces[$index]{portDuplex};
				$interfaceTable{$interface}{portSpantreeFastStart} = $interfaces[$index]{portSpantreeFastStart};
				$interfaceTable{$interface}{vlanPortVlan} = $interfaces[$index]{vlanPortVlan};
				$interfaceTable{$interface}{portAdminSpeed} = $interfaces[$index]{portAdminSpeed};
				$interfaceTable{$interface}{ipAdEntAddr} = $interfaces[$index]{ipAdEntAddr};
				$interfaceTable{$interface}{ipAdEntNetMask} = $interfaces[$index]{ipAdEntNetMask};
				$interfaceTable{$interface}{ipSubnet} = $interfaces[$index]{ipSubnet};
				$interfaceTable{$interface}{ipSubnetBits} = $interfaces[$index]{ipSubnetBits};
			} # if ifdescr not ""
		} # FOR LOOP

		# Write the interface table out.
		&writeCSV(%interfaceTable,$interfacefile,"\t");
	    #foreach $interface (keys %interfaceTable ) {
		#	print "$interface: $interfaceTable{$interface}{ifDescr} $interfaceTable{$interface}{ifType} $interfaceTable{$interface}{ifAdminStatus}\n";
		#}
	} # if validModel
	else {
		if ($debug) { print returnTime." createInterfaceFile - Node is not a valid model $node\n"; }
		logMessage("createInterfaceFile, $node, createInterfaceFile - Node is not a valid model $node");
	}
	# Finished with the SNMP
	END_createInterfaceFile:
	$session->close();
} # end createInterfaceFile

sub checkConfig {
	my $check = shift;
	my $change = 0;
	my $conf_key;
	my $file;
	my $conf_count = 0;
	
	if ($debug) { print returnTime." Config Checking - this will validate the NMIS config file=$conf\n"; }
	if ( $check eq "check" ) { $change = 0;  }	
	else { $change = 1; }
	
	foreach $conf_key (sort(keys %NMIS::config)) {
		++$conf_count;
		if ($debug > 1) { print "  config item: $conf_key = $NMIS::config{$conf_key}\n"; }
		# Check files in htdocs stuff
		## KS 23/6/01 Removed map for deprecated map icon check.
		#if ( $conf_key =~ /icon|arrow_|map|styles|help_file/ ) {
		if ( $conf_key =~ /icon|arrow_|styles|help_file/ ) {
			# check if file exists
			$file = $NMIS::config{$conf_key};
			$file =~ s/$NMIS::config{'<url_base>'}/$NMIS::config{web_root}/;
			if ( not -f $file ) {
				warn returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is a file expanded is $file.  The file doesn't exist, check config items \"web_root=$NMIS::config{web_root}\" and \"<url_base>=$NMIS::config{'<url_base>'}\".\n"; }	
			else { 
				if ($type eq "config") { print "  $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
				if ($debug > 1) { print "  $conf_key = $NMIS::config{$conf_key} - expanded $file - this is a file and the file exists.\n"; } 
			}
		}
		# Check files in htdocs stuff
		elsif ( $conf_key =~ /web_.*_root/ ) {
			# check if file exists
			$file = $NMIS::config{$conf_key};
			$file =~ s/$NMIS::config{web_report_root}/$NMIS::config{web_root}/;
			if ( not -d $file ) {
				if ($type eq "config") { createDir($file); }
				else { warn returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is a directory expanded is $file.  The directory doesn't exist, check config items \"web_root=$NMIS::config{web_root}\" and \"web_reports_root=$NMIS::config{web_report_root}\".\n"; }
			}	
			else { 
				if ($type eq "config") { print "  $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
				if ($debug > 1) { print "  $conf_key = $NMIS::config{$conf_key} - expanded $file - this is a directory and the file exists.\n"; } 
			}
		}
		# Check base directories for <> config variables
		elsif ( $conf_key =~ /_root|nmis_|<.*>/ and $conf_key !~ /url|_log$|_host/ ) {
			# check if directory exists
			if ( not -d $NMIS::config{$conf_key} ) {
				if ($type eq "config") { createDir($NMIS::config{$conf_key}); }
				else { warn returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is a directory and the directory doesn't exist\n"; }
			}	
			else { 
				if ($type eq "config") { print "  $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
				if ($debug > 1) { print "  $conf_key = $NMIS::config{$conf_key} - this is a directory and the directory exists.\n"; } 
			}
		}
		# Check files in cgi-bin stuff
		elsif ( $conf_key =~ /^nmis$|^logs$|^admin$|^ip$|^map$|^view$/ ) {
			# check if file exists
			$file = $NMIS::config{$conf_key};
			$file =~ s/$NMIS::config{'<cgi_url_base>'}/$NMIS::config{'<nmis_cgi>'}/;
			if ( not -f $file ) {
				warn returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is a file expanded is $file.  The file doesn't exist, check config items \"cgi_url_base=$NMIS::config{'<cgi_url_base>'}\" and \"<nmis_cgi>=$NMIS::config{'<nmis_cgi>'}\".\n"; }	
			else { 
				if ($type eq "config") { print "  $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
				if ($debug > 1) { print "  $conf_key = $NMIS::config{$conf_key} - expanded $file - this is a file and the file exists.\n"; } 
			}
		}
		# Check straight files 
		elsif ( $conf_key =~ /Table|_file|_log|_conf|AuthUserFile/ or $conf_key eq "file" ) {
			# check if file exists
			if ( not -f $NMIS::config{$conf_key} ) {
				warn returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is a file and the file doesn't exist\n";
			}	
			else { 
				if ($type eq "config") { print "  $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
				if ($debug > 1) { print "  $conf_key = $NMIS::config{$conf_key} - this is a file and the file exists.\n"; } 
			}
		}
		elsif ( $conf_key =~ /_mib$/ ) {
			# check if file exists
			if ( not -f "$NMIS::config{mib_root}/$NMIS::config{$conf_key}" ) {
				warn returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is a OID file and the file doesn't exist in directory $NMIS::config{mib_root}\n";
			}	
			else { 
				if ($type eq "config") { print "  $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
				if ($debug > 1) { print "  $conf_key = $NMIS::config{$conf_key} - this is a file and the file exists.\n"; } 
			}
		}
		elsif ( $NMIS::config{$conf_key} =~ /true|false/ or $conf_key =~ /_Title|_Key|domain_name|url_base/ ) { 
				if ($type eq "config") { print "  $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
		}
		else {
			if ($type eq "config") { print "  $conf_key=$NMIS::config{$conf_key} - NO TEST AVAILABLE\n"; } 
			if ($debug == 9) { print "    no test for config item $conf_key\n"; }
		}
	}
	
	if ( $conf_count ne $NMIS::config{conf_count} ) {
		print "ERROR: NMIS Config incomplete found $conf_count items should be $NMIS::config{conf_count}. Check config against sample config.\n";
	}
	
	# Do the database directories exist if not make them?
	my $dir;
	if ($debug) { print returnTime." Config Checking - Checking database directories\n"; }
	if ( -d "$NMIS::config{database_root}" ) {
		if ($type eq "config") { 
			createDir("$NMIS::config{database_root}/health");
			createDir("$NMIS::config{database_root}/health/generic");
			createDir("$NMIS::config{database_root}/health/router");
			createDir("$NMIS::config{database_root}/health/switch");
			createDir("$NMIS::config{database_root}/health/server");
			createDir("$NMIS::config{database_root}/interface");
			createDir("$NMIS::config{database_root}/interface/generic");
			createDir("$NMIS::config{database_root}/interface/router");
			createDir("$NMIS::config{database_root}/interface/switch");
			createDir("$NMIS::config{database_root}/interface/server"); 
		}

	}
}

sub createDir {
	my $dir = shift;
	if ( not -d $dir ) { 
		print "  Creating directory $dir\n"; 
		mkdir($dir,0775) or warn "ERROR: cannot mkdir $dir: $!\n"; 
	}
}

sub printApache {
	my $check = shift;
	my $change = 0;
	my $conf_key;
	my $file;
	my @cgi = (
		"nmis",
		"logs",
		"admin",
		"view"
	);
	
	if ($debug) { print returnTime." Apache HTTPD Config for NMIS for config file=$conf\n"; }
	if ( $check eq "check" ) { $change = 0;  }	
	else { $change = 1; }

	print <<EO_TEXT;
## For more information on the listed Apache features read:
## Alias directive:        http://httpd.apache.org/docs/mod/mod_alias.html#alias
## ScriptAlias directive:  http://httpd.apache.org/docs/mod/mod_alias.html#scriptalias
## Order directive:        http://httpd.apache.org/docs/mod/mod_access.html#order
## Allow directive:        http://httpd.apache.org/docs/mod/mod_access.html#allow
## Deny directive:         http://httpd.apache.org/docs/mod/mod_access.html#deny
## AuthType directive:     http://httpd.apache.org/docs/mod/core.html#authtype
## AuthName directive:     http://httpd.apache.org/docs/mod/core.html#authname
## AuthUserFile directive: http://httpd.apache.org/docs/mod/mod_auth.html#authuserfile
## Require directive:      http://httpd.apache.org/docs/mod/core.html#require

# Usual Apache Config File!
#<apache_root>/conf/httpd.conf

# add a password to the users.dat file!
#<apache_root>/bin/htpasswd /usr/local/nmis/conf/users.dat nmis

# restart the daemon!
#<apache_root>/bin/apachectl restart 
#
# NOTE:
# <apache_root> is normally /usr/local/apache
# the "bin" directory might be "sbin"
# the "conf" directory might be "etc"
# the httpd.conf might be split across httpd.conf, access.conf and srm.conf

# NMIS Aliases

Alias $NMIS::config{'<url_base>'}/ "$NMIS::config{web_root}/"
ScriptAlias $NMIS::config{'<cgi_url_base>'}/ "$NMIS::config{'<nmis_cgi>'}/"

EO_TEXT

	foreach $conf_key (@cgi) {
		print <<EO_TEXT;
<Location "$NMIS::config{$conf_key}">
	## For IP address based permissions
	## this will allow either any one on:
	## Network 10.0.0.0, network 172.16.0.0, host 192.168.1.1
	## or anyone with the domain .sins.com.au
	## UNCOMMENT THE FOLLOWING 3 LINES TO ENABLE IP ADDRESS AUTHENTICATION
	## ENSURE YOU UNDERSTAND PRIOR TO ENABLING.
	#Order deny,allow
	#deny from all
	#allow from 10.0.0.0/8 172.16.0.0/16 192.168.1.1 .sins.com.au
	#
	## For Username based authentication
	AuthType Basic
	AuthName "NMIS"
	AuthUserFile /usr/local/nmis/conf/users.dat
	Require valid-user
</Location>

EO_TEXT
	}
}	
