#
#    csv.pm - csv NMIS Perl Package - Network Mangement Information System
#    Copyright (C) 2000,2001 Sinclair InterNetworking Services Pty Ltd
#    Keith Sinclair <keith@sinclair.org.au>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
package csv;

use strict;

use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;

$VERSION = 1.00;

@ISA = qw(Exporter);

@EXPORT = qw(	
		loadCSV
		writeCSV
		loadCSVHeaders
	);

@EXPORT_OK = qw( );

sub loadCSV {
	my $file = shift;
	my $key = shift;
	my $seperator = shift;
	my $reckey;
	my $line = 1;
	
	if ( $seperator eq "" ) { $seperator = "\t"; }

	my $passCounter = 0;

	my $i;
	my @rowElements;
	my @headers;
	my %headersHash;
	my @keylist;
	my $row;
	my $head;
	
	my %data;
	
	open (DATAFILE, "$file") or warn "Cannot open $file";

	while (<DATAFILE>) {
		#$_ =~ s/\n$//g;
		# If it is the first pass load the column headers into an array and a hash.
		if ( $_ !~ /^#|^;|^ / and $_ ne "" and $passCounter == 0 ) {
			++$passCounter;
			$_ =~ s/\"//g;
			@headers = split(/$seperator|\n/, $_);
			for ( $i = 0; $i <= $#headers; ++$i ) {
				$headersHash{$headers[$i]} = $i;
			}
		}
		elsif ( $_ !~ /^#|^;|^ / and $_ ne "" and $passCounter > 0 ) {
			$_ =~ s/\"//g;
			@rowElements = split(/$seperator|\n/, $_);
			if ( $key =~ /:/ ) {
				$reckey = "";
				@keylist = split(":",$key);
				for ($i = 0; $i <= $#keylist; ++$i) {
					$reckey = $reckey.lc("$rowElements[$headersHash{$keylist[$i]}]");
					if ( $i < $#keylist )  { $reckey = $reckey."_" }
				}
			}
			else {
				$reckey = lc("$rowElements[$headersHash{$key}]");
			}
			if ( $#headers > 0 and $#headers != $#rowElements ) {
				$head = $#headers + 1;
				$row = $#rowElements + 1;
				print STDERR "ERROR: $0 in csv.pm: Invalid CSV data file $file; line $line; record \"$reckey\"; $head elements in header; $row elements in data.\n";
			}
			#What if $reckey is blank could form an alternate key?
			if ( $reckey eq "" or $key eq "" ) {
				$reckey = join("-", @rowElements);
			}
			
			for ($i = 0; $i <= $#rowElements; ++$i) {
				if ( $rowElements[$i] eq "null" ) { $rowElements[$i] = ""; }
				$data{$reckey}{$headers[$i]} = $rowElements[$i];
			}
		}
		++$line;
	}
	close (DATAFILE);
	
	return (%data);
}

sub writeCSV (\%@) {
	# MUST BE CALLED WITH &writeCSV not writeCSV!!!!!!!!!
	my $seperator = pop;
	my $file = pop;
	my %csv = @_;
	my $key;
	my $head;
	my $gotheader = 0;
	my @line;
	my $string;
	my $i;

	if ( $seperator eq "" ) { $seperator = "\t"; }
	
	open(OUT, ">$file") or warn "Couldn't open file $file for writing. $!";
	foreach $key (sort (keys %csv) ) {
		if ( not $gotheader) {
			$gotheader = 1;
			$i = 0;
			foreach $head (sort (keys %{$csv{$key}})) {
				$line[$i] = $head;
				++$i;
			}
			$string = join($seperator,@line);
			#Handy for debug
			#print "HEAD: $string\n";
			print OUT "$string\n";
		}
		
		$i = 0;
		foreach $head (sort (keys %{$csv{$key}})) {
			if ( $csv{$key}{$head} eq "" ) { $csv{$key}{$head} = "null"; }
			$line[$i] = $csv{$key}{$head};
			++$i;
		}
		$string = join($seperator,@line);
		#Handy for debug
		#print "DATA: $string\n";
		print OUT "$string\n";
	}
	close(OUT);
}

sub loadCSVHeaders {
	my $file = shift;
	my $seperator = shift;
	
	if ( $seperator eq "" ) { $seperator = "\t"; }

	my $passCounter = 0;

	my $i;
	my @headers;
	my %headersHash;

	open (DATAFILE, "$file") or warn "Cannot open $file";
        while (<DATAFILE>) {
		#chomp;
		# If it is the first pass load the column headers into an array and a hash.
		if ( $_ !~ /^#|^;|^\n/ and $passCounter == 0 ) {
			++$passCounter;
			$_ =~ s/\"//g;
			@headers = split(/$seperator/, $_);
			#for ( $i = 0; $i <= $#headers; ++$i ) {
			#	$headersHash{$headers[$i]} = $i;
			#}
			return (@headers);
		}
	}
	close (DATAFILE);
}

1;
