#!/usr/bin/perl
#
#    nmiscgi.pl - NMIS CGI Program - Network Mangement Information System
#    Copyright (C) 2000,2001 Sinclair InterNetworking Services Pty Ltd
#    Keith Sinclair <keith@sinclair.org.au>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#*****************************************************************************
use lib "/usr/share/nmis/lib";

# 
#****** Shouldn't be anything else to customise below here *******************

require 5;

use Time::ParseDate; 
use RRDs; 
use strict;
use web;
use csv;
use NMIS;
use func;
 
# This reads in the information sent when the user pressed Submit
my %FORM = getCGIForm($ENV{'QUERY_STRING'});

# Break the queary up for the names
my $type = $FORM{type};
my $node = $FORM{node};
my $link = $FORM{link};
my $debug = $FORM{debug};
my $graphtype = $FORM{graphtype};
my $graphlength = $FORM{graphlength};
my $graphstart = $FORM{graphstart};
my $report = $FORM{report};
my $find = $FORM{find};
my $interface = $FORM{interface};
my $health = $FORM{health};
my $length = $FORM{length};
my $sort = $FORM{sort};
my $outage = $FORM{outage};
my $start = $FORM{start};
my $end = $FORM{end};
my $change = $FORM{change};
my $group = $FORM{group};
my $menu = $FORM{menu};
my $title = $FORM{title};

# Allow program to use other configuration files
my $conf;
if ( $FORM{file} ne "" ) { $conf = $FORM{file}; }
else { $conf = "nmis.conf"; }
my $configfile = "/etc/nmis/$conf";
if ( -f $configfile ) { loadConfiguration($configfile); }
else { die "Can't access configuration file $configfile.\n"; }

my ($imageDir, $imageFile, $graphret, $xs, $ys, $ERROR);

my %summaryHash;

# Set the debug to be on or off from the command line argument
if ( $debug eq "true" ) { 
        $debug = "true"; 
        $NMIS::debug = "true";
}
elsif ( $debug eq "verbose" ) { 
        $debug = "verbose"; 
        $NMIS::debug = "verbose";
}
else { 
        $debug = "false"; 
        $NMIS::debug = "false";
}

# default graph length is 2days
if ( $graphlength eq "" ) { $graphlength = "2days"; }

#Global Time Stamp from when set date is.
my ($time,$datestamp,$datestamp_2, $database, $adminCellColor, $operCellColor, $lastCellColor, $availCellColor, $operAvailability, $totalUtil, $totalCellColor, $intString);

# A couple of regular use variables
my ($sysUpTime,$line,$tmpifDescr,$graph);

# Global Interpreted Variables ie stuff we figured out. 
my ($nodeType, $nodeVendor, $nodeModel, $index, $out);

# Global SNMP Variables
my ($sysDescr, $sysObjectID, $sysUpTime, $sysContact, $sysName, $sysLocation, $sysServices, $ifNumber, $oldsysUpTime);

# Select function to be performed

# make the default action to be info
if ( $find ne "" ) { &typeFind; }
elsif ( $outage ne "" ) { &typeOutage; }
elsif ( $type eq "" and $node ne "" ) { &typeInfo; }
#elsif ( $type eq "" ) { &nmisSummary; }
elsif (	$type eq "health" and $node ne "" ) { &typeHealth; }
elsif (	$type eq "summary" and $node ne "" ) { &typeInfo; } 
elsif ( $type eq "link" ) { &typeLink; } 
elsif ( $type eq "dns" ) { &typeDNS; } 
elsif ( $type eq "event" ) { &typeEvent($node); } 
elsif ( $type eq "graph" ) { &typeGraph; } 
elsif ( $type eq "reports" ) { &typeReports; } 
elsif ( $type eq "config" ) { &typeConfig; } 
elsif ( $type eq "nodes" ) { &typeNodes; } 
elsif ( $type eq "find" ) { &typeFind; }
elsif ( $type eq "outage" ) { &typeOutage; }
elsif ( $type eq "ack" ) { 
	&eventAck(ack => $FORM{ack}, node => $FORM{node}, event => $FORM{event}, details => $FORM{details}); 
	&typeEvent();
}
elsif ( $type eq "drawgraph" ) { &drawrrd( node => $FORM{node}, type => $FORM{graph}, length => $FORM{length}, start => $FORM{start}, 
						end => $FORM{end}, width => $FORM{width}, height => $FORM{height}, interface => $FORM{interface} ); } 
else { &typeNMISMenu; }

exit 0;

sub typeNMISMenu {
	pageStart("$NMIS::config{dash_title}","true");
	cssTableStart;
	print "<tr><td class=\"dash\" colspan=\"1\">$NMIS::config{dash_title}</tr></td>\n";
	if ( $group ne "" ) { &nmisGroupSummary($group); }
	elsif ( $menu eq "large" or $NMIS::config{show_large_menu} eq "true" ) { &nmisMenuLarge; }
	elsif ( $menu eq "small" or $NMIS::config{show_large_menu} eq "false" ) { 
		&nmisMenuSmall;
		&nmisMenuBar;
		&nmisSummary; 
	}
	else { 
		&nmisMenuSmall;
		&nmisMenuBar;
		&nmisSummary; 
	}
	rowStart;
	cssPrintCell("grey","<a href=\"http://ee-staff.ethz.ch/~oetiker/webtools/rrdtool/\"><img border=\"0\" src=\"/nmis/rrdtool.gif\"></a>",1);
	rowEnd;
	tableEnd;
	pageEnd;
}

sub typeConfig {

	my $out;

	pageStart("View Configuration","false");
	cssTableStart;
	&nmisMenuSmall; 
	&nmisMenuBar;
	cssHeaderBar("View Configuration $configfile","grey");

	rowStart;
		cssCellStart("white",6);
		cssTableStart("white");
		rowStart;
			cellStart;
			preStart;

			if ( $ENV{REMOTE_USER} ne "" ) {
				$out = `/usr/bin/cat $configfile`;
			}
			else {
				# If you want authorised users then comment out this line
				# after configuring Apache authentication.		
				$out = `/usr/bin/cat $configfile`;
				# and uncomment this one
			 	#$out = "Not allowed by this user";
			}

			print "$out";

			preEnd;
			cellEnd;
		rowEnd;
		tableEnd;
		cellEnd;
	rowEnd;
	tableEnd;
	pageEnd;
}

sub typeGraph {

	my $graph_time;
	my $start_time;
	my $end_time;
	my $datestamp_start;
	my $datestamp_end;
	my $graphLink;
	my $random = rand 10000;
	my $heading;

	# if graphstart is blank then just doing last graphlength
	if ( $graphstart !~ /^[0-9]/ ) {
		$end_time = time;
		$graph_time = convertTimeLength($graphlength);
		$start_time = $end_time - $graph_time;
	}
	# Else the graph is graphlength long starting graphstart
	else {
		$graph_time = convertTimeLength($graphstart);
		$start_time = time - $graph_time;
		$end_time = $start_time + convertTimeLength($graphlength);
	}

	$datestamp_start = returnDateStamp($start_time);
	$datestamp_end = returnDateStamp($end_time);

	$heading = "";

	if ( $graphtype eq "util" ) { $heading = "$node Interface $interface Utilisation Including Availability"; }
	elsif ( $graphtype eq "bits" ) { $heading = "$node Interface $interface Bits/Second Utilisation"; }
	elsif ( $graphtype eq "health" ) { $heading = "$node Overall Reachability, Availability and Health"; }
	elsif ( $graphtype eq "response" ) { $heading = "$node Response Time in milliseconds"; }
	elsif ( $graphtype eq "cpu" ) { $heading = "$node CPU Utilisation"; }
	elsif ( $graphtype eq "ip" ) { $heading = "$node IP Routing Utilisation"; }
	elsif ( $graphtype eq "buffer" ) { $heading = "$node Buffer Utilisation"; }
	elsif ( $graphtype eq "mem-proc" ) { $heading = "$node Processor Memory Utilisation"; }
	elsif ( $graphtype eq "mem-io" ) { $heading = "$node IO Memory Utilisation"; }
	elsif ( $graphtype eq "mem-router" ) { $heading = "$node Rotuer Memory Utilisation"; }
	elsif ( $graphtype eq "mem-switch" ) { $heading = "$node Switch Memory Utilisation"; }
	elsif ( $graphtype eq "traffic" ) { $heading = "$node System Traffic"; }
	elsif ( $graphtype eq "topo" ) { $heading = "$node Topology Changes"; }

	pageStart("Graph Drill In for $heading @ ".returnDateStamp,"true");
	cssTableStart;
	#&nmisMenuSmall; 

	cssHeaderBar("$heading for $graphlength","grey");

	# Get the System info from the file and whack it into the hash
	loadSystemFile($node);
	loadNodeDetails;

	if ( $NMIS::nodeTable{$node}{collect} eq "true" ) {
		loadInterfaceFile($node);
	}

	rowStart;
	cssCellStart("white");
	cssTableStart("white");
	&graphMenu;

	# If the heading isn't blank then there must be a graph type for it.
	if ( $heading ne "" ) {
		#KS 11 Mar 2001 NEW Embedded graphics, none of this dump in a temp directory anymore
		$graphLink="<img border=\"0\" src=\"\/cgi-nmis\/nmiscgi.pl?type=drawgraph&node=$node&graph=$graphtype&length=$graphlength&start=$start_time&end=$end_time&width=660&height=150&interface=$interface\">";	
	}
	else {
	 	$graphLink="Other graph types not yet supported\n";
	}

	if ( $interface eq "" ) {

		if ( $NMIS::systemTable{nodeModel} =~ /router/i ) {
			cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=health&node=$node&graphlength=$graphlength&interface=$interface\">Health</a>",1);
			cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=response&node=$node&graphlength=$graphlength&interface=$interface\">Response</a>",1);
			cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=cpu&node=$node&graphlength=$graphlength&interface=$interface\">CPU</a>",2);
			cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=ip&node=$node&graphlength=$graphlength&interface=$interface\">IP</a>",1);
			rowEnd;
		}
		elsif ( $NMIS::systemTable{nodeModel} eq "Catalyst6000" ) {
			cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=health&node=$node&graphlength=$graphlength&interface=$interface\">Health</a>",1);
			cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=response&node=$node&graphlength=$graphlength&interface=$interface\">Response</a>",1);
			cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=cpu&node=$node&graphlength=$graphlength&interface=$interface\">CPU</a>",2);
			cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=traffic&node=$node&graphlength=$graphlength&interface=$interface\">Traffic</a>",1);
			rowEnd;
		}

		rowStart;
		print "<th colspan=\"11\" width=\"600\">$graphLink</th>\n";
		rowEnd;
		
	}
	else {
		$tmpifDescr = convertIfName($interface);

		$time = RRDs::last "$NMIS::config{database_root}/interface/$NMIS::systemTable{nodeType}/$node/$node-$tmpifDescr.rrd";
		my $lastUpdate = returnDateStamp($time);

		cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=bits&node=$node&interface=$interface&graphlength=$graphlength\">Bits</a>",1);
		cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=util&node=$node&interface=$interface&graphlength=$graphlength\">Util</a>",1);
		cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=response&node=$node&graphlength=$graphlength&interface=$interface\">Response</a>",1);
		cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=cpu&node=$node&graphlength=$graphlength&interface=$interface\">CPU</a>",1);
		cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=ip&node=$node&graphlength=$graphlength&interface=$interface\">IP</a>",1);
		rowEnd;

		rowStart;
		cssPrintCell("menubar","Type",1);
		cssPrintCell("menubar","$NMIS::interfaceTable{$tmpifDescr}{ifType}",3);
		cssPrintCell("menubar","Speed",1);
		cssPrintCell("menubar","$NMIS::interfaceTable{$tmpifDescr}{ifSpeed}",4);
		rowEnd;
		rowStart;
		cssPrintCell("menubar","Last Updated",1);
		cssPrintCell("menubar","$lastUpdate",3);
		cssPrintCell("menubar","Description",1);
		cssPrintCell("menubar","$NMIS::interfaceTable{$tmpifDescr}{Description}",4);
		rowEnd;
		cssPrintCell("menubar","$heading",11);
		rowStart;
		print "<th colspan=\"11\" width=\"600\">$graphLink</th>\n";
		rowEnd;
		
	}

	tableEnd;
	cellEnd;
	rowEnd;

	rowEnd;
	
	tableEnd;
	pageEnd;

    	sub graphMenu {
		print <<EO_HTML;
<form ACTION="$ENV{SCRIPT_NAME}?file=$conf&type=find">
<tr>
	<td class="grey">Graph Length</td>
	<td class="grey">
		<select name=graphlength SIZE=1>
	            <option value="$graphlength">$graphlength</OPTION>
	            <option value="2hours">2hours</OPTION>
	            <option value="4hours">4hours</OPTION>
	            <option value="6hours">6hours</OPTION>
	            <option value="8hours">8hours</OPTION>
	            <option value="10hours">10hours</OPTION>
	            <option value="12hours">12hours</OPTION>
	            <option value="16hours">16hours</OPTION>
	            <option value="20hours">20hours</OPTION>
	            <option value="1day">1day</OPTION>
	            <option value="36hours">36hours</OPTION>
	            <option value="2days">2days</OPTION>
	            <option value="3days">3days</OPTION>
	            <option value="4days">4days</OPTION>
	            <option value="5days">5days</OPTION>
	            <option value="6days">6days</OPTION>
	            <option value="1week">1week</OPTION>
	            <option value="2weeks">2weeks</OPTION>
	            <option value="3weeks">3weeks</OPTION>
	            <option value="4weeks">4weeks</OPTION>
	            <option value="5weeks">5weeks</OPTION>
	            <option value="6weeks">6weeks</OPTION>
	            <option value="7weeks">7weeks</OPTION>
	            <option value="8weeks">8weeks</OPTION>
	            <option value="1month">1month</OPTION>
	            <option value="2months">2months</OPTION>
	            <option value="4months">4months</OPTION>
	            <option value="6months">6months</OPTION>
	            <option value="1year">1year</OPTION>
		</select>
	</td>
	<td class="grey">Starting</td>
	<td class="grey">
		<select name=graphstart SIZE=1>
	            <option value="$graphstart">$graphstart</OPTION>
	            <option value=""></OPTION>
	            <option value="6hours">6hours</OPTION>
	            <option value="12hours">12hours</OPTION>
	            <option value="18hours">18hours</OPTION>
	            <option value="24hours">24hours</OPTION>
	            <option value="36hours">36hours</OPTION>
	            <option value="1day">1day</OPTION>
	            <option value="2days">2days</OPTION>
	            <option value="3days">3days</OPTION>
	            <option value="4days">4days</OPTION>
	            <option value="5days">5days</OPTION>
	            <option value="6days">6days</OPTION>
	            <option value="1week">1week</OPTION>
	            <option value="2weeks">2weeks</OPTION>
	            <option value="3weeks">3weeks</OPTION>
	            <option value="4weeks">4weeks</OPTION>
	            <option value="5weeks">5weeks</OPTION>
	            <option value="6weeks">6weeks</OPTION>
	            <option value="7weeks">7weeks</OPTION>
	            <option value="8weeks">8weeks</OPTION>
	            <option value="1month">1month</OPTION>
	            <option value="2months">2months</OPTION>
	            <option value="4months">4months</OPTION>
	            <option value="6months">6months</OPTION>
	            <option value="1year">1year</OPTION>
		</select>
	</td>
	<td class="grey">Graph Type</td>
	<td class="grey">
		<input TYPE=text SIZE=10 name=graphtype value=$graphtype>
	</td>
	<td class="grey">Type</td>
	<td class="grey">
		<input TYPE=text SIZE=5 name=type value=$type>
	</td>
	<td class="grey"><input TYPE=submit value="GRAPH"></td>
</tr>
<tr>
	<td class="grey">Node</td>
	<td class="grey">
		<input TYPE=text SIZE=5 name=node value=$node>
	</td>
	<td class="grey">Interface</td>
	<td class="grey">
		<input TYPE=text SIZE=8 name=interface value=$interface>
	</td>
</form> 
EO_HTML
	} # end graphMenu

} # end typeGraph

sub typeEvent {
	my $node = shift;
	my $match = "false";
	my $start;
	my $last;
	my $button;
	my $style;
	
	pageStart("Event List","true");
	cssTableStart;
	&nmisMenuSmall; 
	&nmisMenuBar;
	$datestamp = returnDateStamp;
    	cssHeaderBar("Event List at $datestamp","grey");


	# Load the event table into the hash
	&loadEventStateTime;

	rowStart;
	cssCellStart("white",12);
	cssTableStart("white");

	# only display the table if there are any events.
	if ( $NMIS::eventCount == 0 ) {
		printHeadRow("No Events Current @ $datestamp","#FFFFFF");
	}
	else {
		&displayEvents;
	}

	# Finish the page off!!!
	tableEnd;
	cellEnd;
	rowEnd;
	if ( $match eq "false" ) {
		printHeadRow("No Events Current for $node @ $datestamp","#FFFFFF");
	}
	tableEnd;

	cellEnd;
	rowEnd;
	tableEnd;
	pageEnd;

	sub displayEvents {
		my $event_hash;
		my $color;
		my $outage;
		printHeadRow(",Node,Event Time,Event Start,Last Update,Event,Event Level,Details,Acknowledge,Escalation","#FFFFFF");

	        foreach $event_hash ( sort ( {
			$NMIS::eventTable{$a}{ack} cmp $NMIS::eventTable{$b}{ack} or
			$NMIS::eventTable{$b}{startdate} cmp $NMIS::eventTable{$a}{startdate} or
			$NMIS::eventTable{$a}{escalate} cmp $NMIS::eventTable{$b}{escalate}
	        } keys(%NMIS::eventTable)) )  {
			# Only display valid nodes!!!!!
			if ( 	$node eq ""
				or $node eq $NMIS::eventTable{$event_hash}{node}
			) {
				if ( $match eq "false" ) { $match = "true"; }
				if ( $NMIS::eventTable{$event_hash}{ack} eq "false" ) {
					$color = eventColor($NMIS::eventTable{$event_hash}{event_level});
				}
				else {
					$color = "white";
				}
				$start = returnDateStamp($NMIS::eventTable{$event_hash}{startdate});
				$last = returnDateStamp($NMIS::eventTable{$event_hash}{lastchange});
				# User logic, hmmmm how will users interpret this!
				if ( $NMIS::eventTable{$event_hash}{ack} eq "false" ) {
					$button = "true";
					$outage = convertSecsHours(time - $NMIS::eventTable{$event_hash}{startdate});
				}
				else {
					$button = "false";
					$outage = convertSecsHours($NMIS::eventTable{$event_hash}{lastchange} - $NMIS::eventTable{$event_hash}{startdate});
				}
				if ( $color eq "white" ) { $style = "small"; }
				else { $style = ""; }

				print <<EO_HTML;
		<tr>
		<td class="$style" bgcolor="$color">
		<a href="$NMIS::config{admin}?tool=ping&node=$NMIS::eventTable{$event_hash}{node}"><img alt="ping $NMIS::eventTable{$event_hash}{node}" src="$NMIS::config{ping_icon}" border="0"></a><a href="$NMIS::config{admin}?tool=trace&node=$NMIS::eventTable{$event_hash}{node}"><img alt="traceroute $NMIS::eventTable{$event_hash}{node}" src="$NMIS::config{trace_icon}" border="0"></a><a href="telnet://$NMIS::eventTable{$event_hash}{node}"><img alt="telnet to $NMIS::eventTable{$event_hash}{node}" src="$NMIS::config{telnet_icon}" border="0"></a><a href="$ENV{SCRIPT_NAME}?file=$conf&node=$NMIS::eventTable{$event_hash}{node}"><img alt="NMIS Page for $NMIS::eventTable{$event_hash}{node}" src="$NMIS::config{nmis_icon}" border="0"></a>
		</td>
		<td class="$style" bgcolor="$color"><a href="$NMIS::config{logs}?log=Event_Log&search=$NMIS::eventTable{$event_hash}{node}&sort=descending">$NMIS::eventTable{$event_hash}{node}</a></td>
		<td class="small" bgcolor="$color">$outage hh:mm:ss</td>
		<td class="small" bgcolor="$color">$start</td>
		<td class="small" bgcolor="$color">$last</td>
		<td class="$style" bgcolor="$color">$NMIS::eventTable{$event_hash}{event}</td>
		<td class="$style" bgcolor="$color">$NMIS::eventTable{$event_hash}{event_level}</td>
		<td class="$style" bgcolor="$color">$NMIS::eventTable{$event_hash}{details}</td>
		<form action="$ENV{SCRIPT_NAME}" method="get">
		<td class="$style" bgcolor="$color" align="center">
		<input type="hidden" name="type" value="ack">
		<input type="hidden" name="node" value="$NMIS::eventTable{$event_hash}{node}">
		<input type="hidden" name="event" value="$NMIS::eventTable{$event_hash}{event}">
		<input type="hidden" name="details" value="$NMIS::eventTable{$event_hash}{details}">
		<input type="hidden" name="ack" value="$button">
		<input type="submit" value="$button">
		</td>
		</form>
		<td class="$style" bgcolor="$color" align="center">$NMIS::eventTable{$event_hash}{escalate}</td>
		</tr>
EO_HTML
			}
	        } # foreach $event_hash
	} # sub displayEvents
} # sub typeEvent

sub typeInfo {

	my %interfaceUtilTable;
	my %interfaceSwitch;
	my $port;
	my $utilHash;
	my $reportStats;
	my @tmparray;
	my $ifLastChange;
	my $tmpurl;
	my $interface;
	my %interfaceTable;

	pageStart("Information about $node","true");
	cssTableStart;
	&nmisMenuSmall; 
	&nmisMenuBar;
	$datestamp = returnDateStamp;
	loadEventState;

    cssHeaderBar("Information for $node at $datestamp","grey");
	# Compare the Up time to the last uptime? and check if unhealthy?

	# Get the System info from the file and whack it into the hash
	loadNodeDetails;
	loadSystemFile($node);
	
	if ( $NMIS::nodeTable{$node}{collect} eq "true" ) {
		%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","interface","\t");

	}

	displaySystemHeader($node);

	print <<EO_HTML;
</tr>
<tr>	
	<td class="white">
      		<table class="white">
		<tr>
			<td class="white" align="center" colspan="11">
				<a href="$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=health&graphlength=2days&node=$node&interface=$interfaceTable{$interface}{ifDescr}" 
				target=ViewWindow onMouseOver="window.status='Drill in to Device Health.';return true" onClick="viewdoc('$tmpurl',850,455)">
				<img border="0" alt="Device Health" src="$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=health&length=2days&start=0&end=0&width=500&height=100">
				</a>

			</td>
		</tr>
EO_HTML

	if ( $NMIS::systemTable{nodeModel} =~ /router|atm|catalyst|sun|generic/i ) {

		# Extract the interface statics and summaries for display in a second.
		foreach $interface (keys %interfaceTable ) {
			#if ( $interfaceTable{$interface}{ifDescr} ne "" ) {		
				# Set the standard interface name
	 			$tmpifDescr = convertIfName($interfaceTable{$interface}{ifDescr});
	
				# Get the ifOperStatus from the last poll by producing the graph and catching the output!!
				$imageDir="$NMIS::config{interface_root}/$NMIS::systemTable{nodeType}/$node";
	
				# Don't do any stats cause the interface is not one we collect
				if ( $interfaceTable{$interface}{collect} ne "true" ) { 
					$operAvailability = "N/A";  
					$totalUtil = "-1";
					$adminCellColor="#ffffff"; 			
					$availCellColor="#ffffff";
					$totalCellColor="#ffffff";
					$intString="$interfaceTable{$interface}{ifDescr}";
	
					if ( $interfaceTable{$interface}{ifOperStatus} eq "other" ) { $operCellColor="#ffff00"; }
					elsif ( $interfaceTable{$interface}{ifOperStatus} eq "ok" ) { $operCellColor="#00ff00"; }
					else { 	$operCellColor="#ffffff"; }
	
					$ifLastChange = $interfaceTable{$interface}{ifLastChange};
					$ifLastChange =~ s/(.*)days.*/$1/;
					if ( $ifLastChange >= 35 and $interfaceTable{$interface}{ifOperStatus} eq "other" ) { $lastCellColor = "#999900"; }
					elsif ( $ifLastChange >= 5 and $interfaceTable{$interface}{ifOperStatus} eq "other" ) { $lastCellColor = "#ffff00"; }
					else { $lastCellColor = "#ffffff"; }
				}
				else {
					# Reset the number!!!
					$operAvailability = 0;
					$totalUtil = 0;
	
					# Set the cell color to reflect the interface status
					# so if admin = down then oper irrelevent
					if ( 	( $interfaceTable{$interface}{ifAdminStatus} eq "down" ) ||
						( $interfaceTable{$interface}{ifAdminStatus} eq "testing" ) ||
						( $interfaceTable{$interface}{ifAdminStatus} eq "null" )
					) {
						$adminCellColor="#ffffff"; 
						$operCellColor="#ffffff";
						$operAvailability = "N/A"; 
						$availCellColor="#ffffff";
						$totalUtil = "N/A";
						$totalCellColor="#ffffff";
						$lastCellColor = "#ffffff";
						$intString="$interfaceTable{$interface}{ifDescr}";
					}
					# it is supposed to be up what the heck is going on!!!!
					# Set the opercolor to reflect the real status, according to the average.
					else { 
						#$intString="<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$node&type=interface#$interfaceTable{$interface}{ifDescr}\">$interfaceTable{$interface}{ifDescr}</a>";
						$tmpurl = "$ENV{SCRIPT_NAME}?file=$conf&node=$node&type=graph&graphtype=util&graphlength=2days&interface=$tmpifDescr";
						$intString="<a href=\"$tmpurl\" target=ViewWindow onMouseOver=\"window.status='Drill into $interfaceTable{$interface}{ifDescr}.';return true\" onClick=\"viewdoc('$tmpurl',850,455)\">$interfaceTable{$interface}{ifDescr}</a>";
	
						$adminCellColor="#00ff00"; 
						my $event_status = eventExist($node,"Interface Down","interface=$interfaceTable{$interface}{ifDescr}"); 
						if (	( $event_status eq "true" ) && 
							( $interfaceTable{$interface}{ifOperStatus} =~ /up|ok/ )
						) {
							# Red for down
							$operCellColor = "#ff0000";
							$interfaceTable{$interface}{ifOperStatus} = "down";
						}
						elsif (	( $event_status eq "false" ) &&
							( $interfaceTable{$interface}{ifOperStatus} =~ /up|ok/ )
						) {
							# Green for up
							$operCellColor = "#00ff00";
							$interfaceTable{$interface}{ifOperStatus} = "up";
						}
						elsif ( $interfaceTable{$interface}{ifOperStatus} eq "down" ) {
							# Red for down
							$operCellColor = "#ff0000";
						}
						elsif ( $interfaceTable{$interface}{ifOperStatus} eq "dormant" ) {
							# Red for down
							$operCellColor = "#ffff00";
						}
						else { $operCellColor = "#ffffff"; }
	
						$lastCellColor = "#ffffff";
	
						# Get the link availability from the local node!!!
			    		$reportStats = summaryStats($node,"util","-8 hours",time,$interfaceTable{$interface}{ifDescr},$interfaceTable{$interface}{ifSpeed});
			    		#print STDERR "reportStats=$reportStats\n";
						if ( $reportStats != 1 and $reportStats ne "" ) {
							@tmparray = split " ","@$reportStats";
							$operAvailability = $tmparray[0];
							$totalUtil = $tmparray[3];
							$operAvailability =~ s/(.*)=(.*)/$2/;
							$totalUtil =~ s/(.*)=(.*)/$2/;
	
							$availCellColor = colorHighGood($operAvailability);
						       	$totalCellColor = colorLowGood($totalUtil);
						}
	
						if ( $interfaceTable{$interface}{ifOperStatus} eq "dormant" )  {
							$operAvailability = "N/A" ;
							$availCellColor = "#FFFFFF";
							$operCellColor = "#ffff00";
						}
	
					}  # ELSE
				} # if interface is loop contr etc.
	
				if ( $interfaceTable{$interface}{ifDescr} ne "null" ) {
					$interfaceUtilTable{$tmpifDescr}{ifDescr}=$interfaceTable{$interface}{ifDescr};
				       	$interfaceUtilTable{$tmpifDescr}{totalUtil}=$totalUtil;
				       	$interfaceUtilTable{$tmpifDescr}{operAvailability}=$operAvailability;
				       	$interfaceUtilTable{$tmpifDescr}{totalCellColor}=$totalCellColor;
				       	$interfaceUtilTable{$tmpifDescr}{availCellColor}=$availCellColor;
				       	$interfaceUtilTable{$tmpifDescr}{operCellColor}=$operCellColor;
				       	$interfaceUtilTable{$tmpifDescr}{lastCellColor}=$lastCellColor;
				       	$interfaceUtilTable{$tmpifDescr}{adminCellColor}=$adminCellColor;
				       	$interfaceUtilTable{$tmpifDescr}{intString}=$intString;
				}
			#}
		} # FOR LOOP

		if ( $NMIS::systemTable{nodeModel} =~ /router|catalystios|atm|generic|sun/i ) {
			cssPrintHeadRow("Interface Table","menubar",11);
			cssPrintHeadRow("Name,IP,Description,Admin Status,Oper Status,Int. Avail.,Util. 6hrs,Collect,Type,Speed,ifIndex","menubar");

			# So now I have a hash of interface utilsation!!!!
			foreach $utilHash ( sort { 
							$interfaceUtilTable{$b}{totalUtil} <=> $interfaceUtilTable{$a}{totalUtil} ||
							$interfaceUtilTable{$a}{intString} cmp $interfaceUtilTable{$b}{intString} 
						} keys %interfaceUtilTable ) {
				# controls weather or not we see non collected interfaces
				$tmpifDescr = convertIfName($interfaceUtilTable{$utilHash}{ifDescr});
				if (  	( 	$NMIS::config{show_non_collected_interfaces} eq "true" 
						and $interfaceTable{$tmpifDescr}{ifDescr} ne "" 
					)
					or 
					( 	$NMIS::config{show_non_collected_interfaces} eq "false"
						and $interfaceTable{$tmpifDescr}{collect} ne "false" 
						and $interfaceTable{$tmpifDescr}{ifDescr} ne "" 
					)
				) {
					$interfaceTable{$tmpifDescr}{ifSpeed} = convertIfSpeed($interfaceTable{$tmpifDescr}{ifSpeed});
	
					if ( $interfaceUtilTable{$utilHash}{totalUtil} == -1 ) { $interfaceUtilTable{$utilHash}{totalUtil} = "N/A"; }
					rowStart;
					printCell("$interfaceUtilTable{$utilHash}{intString}");
					if ( $interfaceTable{$tmpifDescr}{ipAdEntAddr} ne "" and $interfaceTable{$tmpifDescr}{ipAdEntNetMask} ne "" ) {
						printCell("$interfaceTable{$tmpifDescr}{ipAdEntAddr}<br>$interfaceTable{$tmpifDescr}{ipAdEntNetMask}<br><a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=find&find=$interfaceTable{$tmpifDescr}{ipSubnet}\">$interfaceTable{$tmpifDescr}{ipSubnet}</a>");
					}
					else {
						printCell("");
					}
					printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=find&find=$interfaceTable{$tmpifDescr}{Description}\">$interfaceTable{$tmpifDescr}{Description}</a>");
					printCell("$interfaceTable{$tmpifDescr}{ifAdminStatus}",$interfaceUtilTable{$utilHash}{adminCellColor},1,"center");
					printCell("$interfaceTable{$tmpifDescr}{ifOperStatus}",$interfaceUtilTable{$utilHash}{operCellColor},1,"center");
					printCell("$interfaceUtilTable{$utilHash}{operAvailability}",$interfaceUtilTable{$utilHash}{availCellColor},1,"center");
					printCell("$interfaceUtilTable{$utilHash}{totalUtil}",$interfaceUtilTable{$utilHash}{totalCellColor},1,"center");
					printCell("$interfaceTable{$tmpifDescr}{collect}","#FFFFFF",1,"center");
					printCell("$interfaceTable{$tmpifDescr}{ifType}");
					printCell("$interfaceTable{$tmpifDescr}{ifSpeed}","#FFFFFF",1,"center");
					printCell("$interfaceTable{$tmpifDescr}{ifIndex}","#FFFFFF",1,"center");
					rowEnd;
				}
			} # foreach loop
		}
		elsif ( $NMIS::systemTable{nodeModel} =~ /Catalyst6000|Catalyst5000/i ) {
			printHeadRow("Interface Table","#FFFFFF",15);
			printHeadRow("Name,Description,Admin Status,Oper Status,Avail.,Util. 6hrs,Collection,Type,Speed,Admin Speed,ifIndex,Last Change,Duplex,PortFast,Vlan","#FFFFFF");

			# So now I have a hash of interface utilsation!!!!
			foreach $utilHash ( sort { 
							$interfaceUtilTable{$b}{totalUtil} <=> $interfaceUtilTable{$a}{totalUtil} ||
							$interfaceUtilTable{$a}{intString} cmp $interfaceUtilTable{$b}{intString} 
						} keys %interfaceUtilTable ) {
				$tmpifDescr = convertIfName($interfaceUtilTable{$utilHash}{ifDescr});
				if (  	( 	$NMIS::config{show_non_collected_interfaces} eq "true" 
						and $interfaceTable{$tmpifDescr}{ifDescr} ne "" 
					)
					or 
					( 	$NMIS::config{show_non_collected_interfaces} eq "false"
						and $interfaceTable{$tmpifDescr}{collect} ne "false" 
						and $interfaceTable{$tmpifDescr}{ifDescr} ne "" 
					)
				) {
					$interfaceTable{$tmpifDescr}{ifSpeed} = convertIfSpeed($interfaceTable{$tmpifDescr}{ifSpeed});
					$interfaceTable{$tmpifDescr}{portAdminSpeed} = convertIfSpeed($interfaceTable{$tmpifDescr}{portAdminSpeed});
					if ( $interfaceUtilTable{$utilHash}{totalUtil} == -1 ) { $interfaceUtilTable{$utilHash}{totalUtil} = "N/A"; }
					rowStart;
					printCell("$interfaceUtilTable{$utilHash}{intString}");
					printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=find&find=$interfaceTable{$tmpifDescr}{Description}\">$interfaceTable{$tmpifDescr}{Description}</a>");
					printCell("$interfaceTable{$tmpifDescr}{ifAdminStatus}",$interfaceUtilTable{$utilHash}{adminCellColor},1,"center");
					printCell("$interfaceTable{$tmpifDescr}{ifOperStatus}",$interfaceUtilTable{$utilHash}{operCellColor},1,"center");
					printCell("$interfaceUtilTable{$utilHash}{operAvailability}",$interfaceUtilTable{$utilHash}{availCellColor},1,"center");
					printCell("$interfaceUtilTable{$utilHash}{totalUtil}",$interfaceUtilTable{$utilHash}{totalCellColor},1,"center");
					printCell("$interfaceTable{$tmpifDescr}{collect}","#FFFFFF",1,"center");
					printCell("$interfaceTable{$tmpifDescr}{ifType}");
					printCell("$interfaceTable{$tmpifDescr}{ifSpeed}","#FFFFFF",1,"center");
					printCell("$interfaceTable{$tmpifDescr}{portAdminSpeed}","#FFFFFF",1,"center");
					printCell("$interfaceTable{$tmpifDescr}{ifIndex}","#FFFFFF",1,"center");
					printCell("$interfaceTable{$tmpifDescr}{ifLastChange}",$interfaceUtilTable{$utilHash}{lastCellColor},1,"center");
					printCell("$interfaceTable{$tmpifDescr}{portDuplex}","#FFFFFF",1,"center");
					printCell("$interfaceTable{$tmpifDescr}{portSpantreeFastStart}","#FFFFFF",1,"center");
					printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=find&find=$interfaceTable{$tmpifDescr}{vlanPortVlan}\">$interfaceTable{$tmpifDescr}{vlanPortVlan}</a>","#FFFFFF",1,"center");
					rowEnd;
				}
			} # foreach loop

	 	}	
 	}
	else  {
		printHeadRow("No Interface Table Information Available","#FFFFFF",20);
	}
			
	tableEnd;
	cellEnd;
	rowEnd;

	rowEnd;
	tableEnd;
	pageEnd;
}

sub typeLink {

	my $index;
	my %reportTable;
	my %summaryTable;
	my $summaryhash;
	my @tmparray;
	my @tmpsplit;
	my $cellColor;

	# Fill the data hashes with groovy information!!!!
	loadLinkDetails;
	loadNodeDetails;

	if ( $link eq "" and $node eq "" ) { &linkMenu; }
	elsif ( $node ne "" ) { &linkNode; }
	else { &linkDetails; }

	sub linkMenu{
		my $linkname;
		my $reportStats;

		pageStart("Link Menu","false");
		cssTableStart;
		&nmisMenuSmall; 
		&nmisMenuBar;
		cssHeaderBar("Link List (stats based on last 2 days)","grey");

		# Get each of the links and get some summary info!!!
	    foreach $linkname ( keys (%NMIS::linkTable) )   {
			# Get the response time from the remote node!!!
			if ( $NMIS::linkTable{$linkname}{node2} ne "" ) {
				loadSystemFile($NMIS::linkTable{$linkname}{node2});
	    		$reportStats = summaryStats($NMIS::linkTable{$linkname}{node2},"health","-2 days",time);
				if ( $reportStats != 1 and $reportStats ne "" ) {
					@tmparray = split " ","@$reportStats";
					for ( $index = 0; $index <= $#tmparray; ++$index ) {
						@tmpsplit = split "=",@tmparray[$index];
						$reportTable{$linkname}{$tmpsplit[0]} = $tmpsplit[1];
					}
				}
			}

			loadSystemFile($NMIS::linkTable{$linkname}{node1});
			loadInterfaceFile($NMIS::linkTable{$linkname}{node1});

			# Get the link availability from the local node!!!
			$tmpifDescr = convertIfName($NMIS::linkTable{$linkname}{interface1});
			if ( $NMIS::linkTable{$linkname}{node1} ne "" 
				 and $NMIS::interfaceTable{$tmpifDescr}{collect} eq "true"
			) {
	    		$reportStats = summaryStats($NMIS::linkTable{$linkname}{node1},"util","-2 days",time,$tmpifDescr,$NMIS::interfaceTable{$tmpifDescr}{ifSpeed});
				if ( $reportStats != 1 and $reportStats ne "" ) {
					@tmparray = split " ","@$reportStats";
					for ( $index = 0; $index <= $#tmparray; ++$index ) {
						@tmpsplit = split "=",@tmparray[$index];
						$reportTable{$linkname}{$tmpsplit[0]} = $tmpsplit[1] ;
					}
				}
			}
		} # foreach $linkname

		rowStart;
		cellStart ("white",12);

		# Show the International Links
		cssTableStart("white");
		printHeadRow("International Links","#FFFFFF",13);
		&displayLinks("international");
	 	tableEnd;
		paragraph;

		# Show the Domestic Links
	 	cssTableStart("white");
		printHeadRow("Domestic Links","#FFFFFF",13);
		&displayLinks("domestic");
		tableEnd;
		paragraph;

		# Show the Access Links
		cssTableStart("white");
		printHeadRow("Frame Relay Access Links","#FFFFFF",13);
		&displayLinks("access");
	 	tableEnd;

		cellEnd;
		rowEnd;
		tableEnd;

		cellEnd;
		rowEnd;
		tableEnd;
		pageEnd;
	} # Link Menu

	sub linkNode{
		my $linkname;
		my $reportStats;

		pageStart("Link List for $node","false");
		cssTableStart;
		&nmisMenuSmall; 
		&nmisMenuBar;
		cssHeaderBar("Link List for $node (stats based on last 2 days)","grey");

		# Get each of the links and get some summary info!!!
		foreach $linkname ( keys (%NMIS::linkTable) )   {
			if ( $NMIS::linkTable{$linkname}{node1} eq $node
				 or $NMIS::linkTable{$linkname}{node2} eq $node
			) {
				# Get the response time from the remote node!!!
				if ( $NMIS::linkTable{$linkname}{node2} ne "" ) {
					loadSystemFile($NMIS::linkTable{$linkname}{node2});
		    		$reportStats = summaryStats($NMIS::linkTable{$linkname}{node2},"health","-2 days",time);
					if ( $reportStats != 1 and $reportStats ne "" ) {
						@tmparray = split " ","@$reportStats";
						for ( $index = 0; $index <= $#tmparray; ++$index ) {
							@tmpsplit = split "=",@tmparray[$index];
							$reportTable{$linkname}{$tmpsplit[0]} = $tmpsplit[1] ;
						}
					}
				}

				loadSystemFile($NMIS::linkTable{$linkname}{node1});
				loadInterfaceFile($NMIS::linkTable{$linkname}{node1});

				# Get the link availability from the local node!!!
				$tmpifDescr = convertIfName($NMIS::linkTable{$linkname}{interface1});
				if ( $NMIS::linkTable{$linkname}{node1} ne ""
					and $NMIS::interfaceTable{$tmpifDescr}{collect} eq "true"
				) {
		    		$reportStats = summaryStats($NMIS::linkTable{$linkname}{node1},"util","-2 days",time,$tmpifDescr,$NMIS::interfaceTable{$tmpifDescr}{ifSpeed});
					if ( $reportStats != 1 and $reportStats ne "" ) {
						@tmparray = split " ","@$reportStats";
						for ( $index = 0; $index <= $#tmparray; ++$index ) {
							@tmpsplit = split "=",@tmparray[$index];
							$reportTable{$linkname}{$tmpsplit[0]} = $tmpsplit[1] ;
						}
					}
				}
			} # if $node = Node1
		} # foreach $linkname

		rowStart;
		cellStart ("white",12);

		# Show the International Links
		cssTableStart("white");
		printHeadRow("International Links","#FFFFFF",13);
		&displayLinks("international");
	 	tableEnd;
		paragraph;

		# Show the Domestic Links
	 	cssTableStart("white");
		printHeadRow("Domestic Links","#FFFFFF",13);
		&displayLinks("domestic");
		tableEnd;
		paragraph;

		# Show the Access Links
		cssTableStart("white");
		printHeadRow("Frame Relay Access Links","#FFFFFF",13);
		&displayLinks("access");
	 	tableEnd;

		cellEnd;
		rowEnd;
		tableEnd;

		cellEnd;
		rowEnd;
		tableEnd;
		pageEnd;
	} # Link Node

	sub linkDetails{
		my @tmparray;
		my $index;

		my ($graphLinkUtil,$graphLinkBits);

		pageStart("$NMIS::linkTable{$link}{link} Link Details","true");
		cssTableStart;
		&nmisMenuSmall; 
		&nmisMenuBar;
	    	cssHeaderBar("$link Link Details","grey");

		rowStart;
		cellStart ("white",12);
		cssTableStart("white");

		rowStart;
		printHeadCell("Net");
		printCell("$NMIS::linkTable{$link}{net}");
		printHeadCell("Role");
		printCell("$NMIS::linkTable{$link}{role}");
		printHeadCell("Location");
		printCell("$NMIS::linkTable{$link}{location}");
		rowEnd;

		rowStart;
		printHeadCell("Primary Node","#FFFFFF",2);
		printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$NMIS::linkTable{$link}{node1}\">$NMIS::linkTable{$link}{node1}</a>","#FFFFFF",1);
		printHeadCell("Primary Interface","#FFFFFF",2);
		printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$NMIS::linkTable{$link}{node1}&type=interface#$NMIS::linkTable{$link}{interface1}\">$NMIS::linkTable{$link}{interface1}</a>","#FFFFFF",1);
		rowEnd;

		if ( $NMIS::linkTable{$link}{location} ne "access" ) {
			rowStart;
			printHeadCell("Secondary Node","#FFFFFF",2);
			printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$NMIS::linkTable{$link}{node2}\">$NMIS::linkTable{$link}{node2}</a>","#FFFFFF",1);
			printHeadCell("Secondary Interface","#FFFFFF",2);
			printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$NMIS::linkTable{$link}{node2}&type=interface#$NMIS::linkTable{$link}{interface2}\">$NMIS::linkTable{$link}{interface2}</a>","#FFFFFF",1);
			rowEnd;
		}

		if ( $NMIS::linkTable{$link}{depend} ne "none" ) {
			# Check if its daisy chain of links
			rowStart;
			if ( $NMIS::linkTable{$link}{depend} =~ /;/ ) {
				@tmparray = split ";", $NMIS::linkTable{$link}{depend};
				printHeadCell("Link Dependancies","#FFFFFF",3);
				cellStart("#FFFFFF",3);
				for ($index = 0; $index <= $#tmparray; ++$index) {
					print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=link&link=$tmparray[$index]\">$tmparray[$index]</a>";
					if ( $index != $#tmparray ) { print "; "; }
				}
				cellEnd;
			}
			else {
				printHeadCell("Link Dependancies","#FFFFFF",3);
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=link&link=$NMIS::linkTable{$link}{depend}\">$NMIS::linkTable{$link}{depend}</a>","#FFFFFF",3);
			}
			rowEnd;
		}

		rowStart;
 		cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=response&graphlength=2days&node=$NMIS::linkTable{$link}{node2}&interface=$NMIS::interfaceTable{$interface}{ifDescr}\">Response Time</a>",2);
 		cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=util&graphlength=2days&node=$NMIS::linkTable{$link}{node1}&interface=$NMIS::linkTable{$link}{interface1}\">Util Graph</a>",2);
 		cssPrintCell("menubar","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=bits&graphlength=2days&node=$NMIS::linkTable{$link}{node1}&interface=$NMIS::linkTable{$link}{interface1}\">Bits Per Second Graph</a>",2);
		rowEnd;

		if ( $NMIS::linkTable{$link}{node2} ne "" ) {
			$node = $NMIS::linkTable{$link}{node2};
			loadSystemFile($node);
			my $graphResponseUtil =
				"<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=response&graphlength=2days&node=$node&interface=$NMIS::interfaceTable{$interface}{ifDescr}\">"
				."<img border=\"0\" alt=\"Device Response Time\" src=\"$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=response&length=2days&start=0&end=0&width=500&height=100\"></a>"
				."";

			rowStart;
			cssPrintCell("grey","Response Time for Secondary Node $node</STRONG>",6);
			rowEnd;

			rowStart;
 	 		cssPrintCell("center","$graphResponseUtil",6);
			rowEnd;
				
		}
		# Display the Primary Node Details
		$node = $NMIS::linkTable{$link}{node1};
		$tmpifDescr = convertIfName($NMIS::linkTable{$link}{interface1});
		loadSystemFile($node);

		rowStart;
		cssPrintCell("grey","Primary Node Interface Stats for $node Interface: $NMIS::linkTable{$link}{interface1}",6);
		rowEnd;

		$graphLinkUtil =
			"<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=util&graphlength=2days&node=$node&interface=$NMIS::linkTable{$link}{interface1}\">"
			."<img alt=\"Interface Availability and Utilisation\" border=\"0\" src=\"$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=bits&length=2days&start=0&end=0&width=500&height=100&interface=$tmpifDescr\"></a>"
			."";

		rowStart;

  		cssPrintCell("center","$graphLinkUtil",6);
		rowEnd;

		tableEnd;
		cellEnd;
		rowEnd;
		tableEnd;

		cellEnd;
		rowEnd;
		tableEnd;
		pageEnd;
	} # Link Details

	sub displayLinks {
	  	my $string = shift;

		my $linkname;
		my $cellColor;
		my @tmparray;
		my $index;

		if ( $string eq "access" ) {
			printHeadRow("Link Name,Net,Role,Node,Interface,Link Availability,% Total Util,% In Util,% Out Util","#FFFFFF");
		}
		else {
			printHeadRow("Link Name,Net,Role,Primary Node,Interface,Secondary Node,Interface,Link Availability,% Total Util,% In Util,% Out Util,Response Time,Dependancies","#FFFFFF");
		}
		#print each of the display groups in turn
	        foreach $linkname (sort ( keys (%NMIS::linkTable) ) )  {
			if ( 
				( 	$node eq "" 
					and $NMIS::linkTable{$linkname}{location} eq "$string" 
				) or (	
					$NMIS::linkTable{$linkname}{node1} eq $node 
					or $NMIS::linkTable{$linkname}{node2} eq $node
				) and
					$NMIS::linkTable{$linkname}{location} eq "$string" 
			) {
				$tmpifDescr = convertIfName($NMIS::linkTable{$link}{interface1});
				rowStart;
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=link&link=$linkname\">$NMIS::linkTable{$linkname}{link}</a>");
				printCell("$NMIS::linkTable{$linkname}{net}");
				printCell("$NMIS::linkTable{$linkname}{role}");
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$NMIS::linkTable{$linkname}{node1}\">$NMIS::linkTable{$linkname}{node1}</a>");
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$NMIS::linkTable{$linkname}{node1}&type=interface#$NMIS::linkTable{$linkname}{interface1}\">$NMIS::linkTable{$linkname}{interface1}</a>");
				if ( $string ne "access" ) {
					printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$NMIS::linkTable{$linkname}{node2}\">$NMIS::linkTable{$linkname}{node2}</a>");
					printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$NMIS::linkTable{$linkname}{node2}&type=interface#$NMIS::linkTable{$linkname}{interface2}\">$NMIS::linkTable{$linkname}{interface2}</a>");
				}
				$cellColor = colorHighGood($reportTable{$linkname}{availability});
				printCellRight("$reportTable{$linkname}{availability}",$cellColor);

				$cellColor = colorLowGood($reportTable{$linkname}{totalUtil});
				printCellRight("$reportTable{$linkname}{totalUtil}",$cellColor);
			
				$cellColor = colorLowGood($reportTable{$linkname}{inputUtil});
				printCellRight("$reportTable{$linkname}{inputUtil}",$cellColor);

				$cellColor = colorLowGood($reportTable{$linkname}{outputUtil});
				printCellRight("$reportTable{$linkname}{outputUtil}",$cellColor);

				if ( $string ne "access" ) {
					if ( $reportTable{$linkname}{response} ne "" ) {
						$cellColor = colorResponseTime($reportTable{$linkname}{response});
						printCellRight("$reportTable{$linkname}{response}",$cellColor);
					}
					else { printCell(""); }
				}

				if ( $string ne "access" ) {
					if ( $NMIS::linkTable{$linkname}{depend} ne "none" ) {
						# Check if its daisy chain of links
						if ( $NMIS::linkTable{$linkname}{depend} =~ /;/ ) {
							@tmparray = split ";", $NMIS::linkTable{$linkname}{depend};
							cellStart("#FFFFFF");
							for ($index = 0; $index <= $#tmparray; ++$index) {
								print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=link&link=$tmparray[$index]\">$tmparray[$index]</a>";
								if ( $index != $#tmparray ) { print "<BR>"; }
							}
							cellEnd;
						}
						else {
							printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=link&link=$NMIS::linkTable{$linkname}{depend}\">$NMIS::linkTable{$linkname}{depend}</a>","#FFFFFF");
						}
					}
					else { printCell(""); }
				}

				rowEnd;
			} # if $location eq $string

	        } # foreach $linkname

	} # sub displayLinks

} # sub typeLink

sub typeHealth {
	
	my $tmpurl;

	pageStart("Health Info","true");
	cssTableStart;
	&nmisMenuSmall; 
	&nmisMenuBar;
	my $time = time;
	$datestamp = returnDateStamp;
	$datestamp_2 = returnDateStamp($time - 2 * 24 * 60 * 60);
    	cssHeaderBar("Health statistics for $node from $datestamp_2 to $datestamp","grey");

	# Get the System info from the file and whack it into the hash
	loadSystemFile($node);
	displaySystemHeader($node);

	if ( $NMIS::nodeTable{$node}{collect} eq "true" and $NMIS::systemTable{nodeModel} =~ /CiscoRouter|CatalystIOS/ ) {
		rowStart;
		cssCellStart("white");
		cssTableStart("white");
		rowStart;
			cssPrintCell("menubar","<a href=\"#INFO\">Info</a>");
			cssPrintCell("menubar","<a href=\"#REACH\">Reachability</a>");
			cssPrintCell("menubar","<a href=\"#RESPONSE\">Response Time</a>");
			if ( $NMIS::nodeTable{$node}{collect} eq "true" ) {
				cssPrintCell("menubar","<a href=\"#CPU\">CPU</a>");
				cssPrintCell("menubar","<a href=\"#IP\">IP Routing</a>");
				cssPrintCell("menubar","<a href=\"#MEM\">Memory</a>");
				cssPrintCell("menubar","<a href=\"#BUFFER\">Buffer</a>");
			}
		rowEnd;
		tableEnd;
		cellEnd;
		rowEnd;
	}
	if ( $NMIS::nodeTable{$node}{collect} eq "true" and $NMIS::systemTable{nodeModel} eq "Catalyst6000" ) {
		rowStart;
		cssCellStart("white");
		cssTableStart("white");
		rowStart;
			cssPrintCell("grey","<a href=\"#INFO\">Info</a>");
			cssPrintCell("grey","<a href=\"#REACH\">Reachability</a>");
			cssPrintCell("grey","<a href=\"#RESPONSE\">Response Time</a>");
			if ( $NMIS::nodeTable{$node}{collect} eq "true" ) {
				cssPrintCell("grey","<a href=\"#CPU\">CPU</a>");
				cssPrintCell("grey","<a href=\"#TRAFFIC\">System Traffic</a>");
				cssPrintCell("grey","<a href=\"#TOPO\">Topology Changes</a>");
				cssPrintCell("grey","<a href=\"#MEM\">Memory</a>");
			}
		rowEnd;
		tableEnd;
		cellEnd;
		rowEnd;
	}

	print <<EO_HTML;
<tr>
	<td align="center" bgcolor="white"><A name="REACH"><b>
		<a href="#TOP">Reachability, Availability and Health</a><BR>
		<a href="$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=health&graphlength=2days&node=$node" 
		target=ViewWindow onMouseOver="window.status='Drill into Device Reachability.';return true" onClick="viewdoc('$tmpurl',850,455)">
		<img border=\"0\" alt="Device Reachability" src="$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=health&length=2days&start=0&end=0&width=500&height=100">
		</a>
	</b>
	</td>
</tr>
<tr>
	<td align="center" bgcolor="white"><A name="RESPONSE"><b>
		<a href="#TOP">Response Time</a><BR>
		<a href="$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=response&graphlength=2days&node=$node"
		target=ViewWindow onMouseOver="window.status='Drill into Device Response Time.';return true" onClick="viewdoc('$tmpurl',850,455)">
		<img border=\"0\" alt="Response Time" src="$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=response&length=2days&start=0&end=0&width=500&height=100">
</a>
	</b>
	</td>
</tr>
EO_HTML

	if ( $NMIS::nodeTable{$node}{collect} eq "true" and $NMIS::systemTable{nodeModel} =~ /CiscoRouter|CatalystIOS/ ) {
		print <<EO_HTML;
<tr>
	<td align="center" bgcolor="white"><A name="CPU"><b>
		<a href="#TOP">CPU</a><BR>
		<a href="$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=cpu&graphlength=2days&node=$node"
		target=ViewWindow onMouseOver="window.status='Drill into CPU Utilisation.';return true" onClick="viewdoc('$tmpurl',850,455)">
		<img border=\"0\" alt="CPU Utilisation" src="$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=cpu&length=2days&start=0&end=0&width=500&height=100">
		</a>
	</b>
	</td>
</tr>
<tr>
	<td align="center" bgcolor="white"><A name="IP"><b>
		<a href="#TOP">IP Routing</a><BR>
		<a href="$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=ip&graphlength=2days&node=$node"
		target=ViewWindow onMouseOver="window.status='Drill into IP Routing Stats.';return true" onClick="viewdoc('$tmpurl',850,455)">
		<img border=\"0\" alt="IP Routing Stats" src="$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=ip&length=2days&start=0&end=0&width=500&height=100">
		</a>
	</b>
	</td>
</tr>
<tr>
	<td align="CENTER" bgcolor="#FFFFFF"><A name="BUFFER"><b>
		<a href="#TOP">Buffer</a><BR>
		<a href="$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=buffer&graphlength=2days&node=$node"
		target=ViewWindow onMouseOver="window.status='Drill into Buffer Stats.';return true" onClick="viewdoc('$tmpurl',850,455)">
		<img border=\"0\" alt="Buffer Stats" src="$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=buffer&length=2days&start=0&end=0&width=500&height=100">
		</a>
	</b>
	</td>
</tr>
<tr>
	<td align="center" bgcolor="white"><A name="MEM"><b>
		<a href="#TOP">Memory</a><BR>
		<a href="$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=mem-io&graphlength=2days&node=$node"
		target=ViewWindow onMouseOver="window.status='Drill into Router Memory Utilisation.';return true" onClick="viewdoc('$tmpurl',850,455)">
		<img border=\"0\" alt="Router Memory Utilisation" src="$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=mem-router&length=2days&start=0&end=0&width=500&height=100">
		</a>
	</b>
	</td>
</tr>
EO_HTML
	}
####################################
	elsif ( $NMIS::nodeTable{$node}{collect} eq "true" and $NMIS::systemTable{nodeModel} eq "Catalyst6000" ) {
		print <<EO_HTML;
<tr>
	<td align="center" bgcolor="white"><A name="CPU"><b>
		<a href="#TOP">CPU</a><BR>
		<a href="$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=switch&graphlength=2days&node=$node">
		<img border=\"0\" alt="CPU Utilisation" src="$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=cpu&length=2days&start=0&end=0&width=500&height=100">
		</a>
	</b>
	</td>
</tr>
<tr>
	<td align="center" bgcolor="white"><A name="TRAFFIC"><b>
		<a href="#TOP">Traffic</a><BR>
		<a href="$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=traffic&graphlength=2days&node=$node">
		<img border=\"0\" alt="System Traffic" src="$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=traffic&length=2days&start=0&end=0&width=500&height=100">
		</a>
	</b>
	</td>
</tr>
<tr>
	<td align="center" bgcolor="white"><A name="TOPO"><b>
		<a href="#TOP">Topology Changes</a><BR>
		<a href="$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=topo&graphlength=2days&node=$node">
		<img border=\"0\" alt="Topology Change" src="$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=topo&length=2days&start=0&end=0&width=500&height=100">
		</a>
	</b>
	</td>
</tr>
<tr>
	<td align="center" bgcolor="white"><A name="MEM"><b>
		<a href="#TOP">Memory</a><BR>
		<a href="$ENV{SCRIPT_NAME}?file=$conf&type=graph&graphtype=mem-switch&graphlength=2days&node=$node">
		<img border=\"0\" alt="Switch Memory Utilisation" src="$ENV{SCRIPT_NAME}?file=$conf&type=drawgraph&node=$node&graph=mem-switch&length=2days&start=0&end=0&width=500&height=100">
		</a>
	</b>
	</td>
</tr>
EO_HTML
	}

	tableEnd;
	pageEnd;
} # typeHealth

sub typeReports {

	my $net;
	my $role;
	my $header;
	my $start_time;
	my $datestamp_start;
	my $datestamp_end;
	my $time = time;
	my $graphLink;
	my $random = rand 10000;
	my $reportnode;
	my $index;
	my $reportFile;
	my $reportLink;
	my $reportStats;
	my $reportLength;
	my %reportTable;
	my %summaryTable;
	my $summaryhash;
	my @tmparray;
	my @tmpsplit;
	my $cellColor;
	my $stamp = returnDateStamp;
	my $group;

	loadNodeDetails;

	pageStart("Reports Stuff","false");

	$start_time = convertTime($graphlength);
	$datestamp_start = returnDateStamp($start_time);
	$datestamp_end = returnDateStamp($time);

 	cssTableStart;

	#&nmisMenuSmall;

	if ( $report eq "health" ) {

		if ( $length eq "month" ) { 
			$reportLength = "-1 months";
			$header = "Monthly Dynamic Summary Health Metrics @ $datestamp_end"; 
		}
		elsif ( $length eq "day" ) { 
			$reportLength = "-1 days";
			$header = "Daily Dynamic Summary Health Metrics @ $datestamp_end"; 
		}
		else { 	
			$reportLength = "-1 weeks";
			$header = "Weekly Dynamic Summary Health Metrics @ $datestamp_end"; 
		}

	    	cssHeaderBar("$header","grey");
	
		# Get each of the nodes info in a HASH for playing with
		foreach $reportnode (sort(keys(%NMIS::nodeTable))) {
			loadSystemFile($reportnode);
    		$reportStats = summaryStats($reportnode,"health",$reportLength,time);
			if ( $reportStats != 1 and $reportStats ne "" ) {
				@tmparray = split " ","@$reportStats";
				for ( $index = 0; $index <= $#tmparray; ++$index ) {
					@tmpsplit = split "=",@tmparray[$index];
					$reportTable{$reportnode}{$tmpsplit[0]} = $tmpsplit[1] ;
				}
			}

			## Small exception trap for devices which don't have valid results.
			if ( 	$reportTable{$reportnode}{reachable} eq "NaN" and 
				$reportTable{$reportnode}{available} eq "NaN" and 
				$reportTable{$reportnode}{health} eq "NaN" and 
				$reportTable{$reportnode}{response} eq "NaN"
			) {
				$reportTable{$reportnode}{reachable} = 0;    
				$reportTable{$reportnode}{available} = 0;    
				$reportTable{$reportnode}{health} = 0;             
				$reportTable{$reportnode}{response} = 0;       
			}
			$reportTable{$reportnode}{net} = $NMIS::nodeTable{$reportnode}{net};
			$reportTable{$reportnode}{role} = $NMIS::nodeTable{$reportnode}{role};
			$reportTable{$reportnode}{devicetype} = $NMIS::nodeTable{$reportnode}{devicetype};
			$reportTable{$reportnode}{group} = $NMIS::nodeTable{$reportnode}{group};

			# Calculate the summaries!!!!
			$summaryhash = "$reportTable{$reportnode}{net}-$reportTable{$reportnode}{role}";
			$summaryTable{$summaryhash}{net} = $reportTable{$reportnode}{net};
			$summaryTable{$summaryhash}{role} = $reportTable{$reportnode}{role};
			$summaryTable{$summaryhash}{reachable} = $summaryTable{$summaryhash}{reachable} + $reportTable{$reportnode}{reachable};
			$summaryTable{$summaryhash}{available} = $summaryTable{$summaryhash}{available} + $reportTable{$reportnode}{available};
			$summaryTable{$summaryhash}{health} = $summaryTable{$summaryhash}{health} + $reportTable{$reportnode}{health};
			$summaryTable{$summaryhash}{response} = $summaryTable{$summaryhash}{response} + $reportTable{$reportnode}{response};
			++$summaryTable{$summaryhash}{count};

			$summaryhash = $reportTable{$reportnode}{group};
			$summaryTable{$summaryhash}{net} = $reportTable{$reportnode}{group};
			$summaryTable{$summaryhash}{role} = "";
			$summaryTable{$summaryhash}{group} = $reportTable{$reportnode}{group};
			$summaryTable{$summaryhash}{reachable} = $summaryTable{$summaryhash}{reachable} + $reportTable{$reportnode}{reachable};
			$summaryTable{$summaryhash}{available} = $summaryTable{$summaryhash}{available} + $reportTable{$reportnode}{available};
			$summaryTable{$summaryhash}{health} = $summaryTable{$summaryhash}{health} + $reportTable{$reportnode}{health};
			$summaryTable{$summaryhash}{response} = $summaryTable{$summaryhash}{response} + $reportTable{$reportnode}{response};
			++$summaryTable{$summaryhash}{count};
		}

	 	rowStart;
	 	cellStart("#FFFFFF",20);

	 	cssTableStart("white");

	 	printHeadRow("Average Health for Device Groups","#FFFFFF",10);							
	 	printHeadRow("Group,Reachability,Interface Avail.,Health,Response Time","#FFFFFF");
															
		# print a summary table for the health
		foreach $group ( sort ( keys %summaryTable ) ) {
			if ( $summaryTable{$group}{net} =~ /lan|wan/i ) {
				$summaryTable{$group}{net} = uc($summaryTable{$group}{net});
			}
			rowStart;
			printHeadCell("<a href=\"#$group\">$summaryTable{$group}{net} $summaryTable{$group}{role}</a>");
			if ( $summaryTable{$group}{reachable} > 0 ) {
				$summaryTable{$group}{avgreachable} = sprintf("%.3f",$summaryTable{$group}{reachable} / $summaryTable{$group}{count}); 
				$cellColor = colorHighGood($summaryTable{$group}{avgreachable});
				printHeadCell($summaryTable{$group}{avgreachable},$cellColor);
			}
			if ( $summaryTable{$group}{available} > 0 ) { 
				$summaryTable{$group}{avgavailable} = sprintf("%.3f",$summaryTable{$group}{available} / $summaryTable{$group}{count}); 
				$cellColor = colorHighGood($summaryTable{$group}{avgavailable});
				printHeadCell($summaryTable{$group}{avgavailable},$cellColor);
			}
			if ( $summaryTable{$group}{health} > 0 ) { 
				$summaryTable{$group}{avghealth} = sprintf("%.3f",$summaryTable{$group}{health} / $summaryTable{$group}{count}); 
				$cellColor = colorHighGood($summaryTable{$group}{avghealth});
				printHeadCell($summaryTable{$group}{avghealth},$cellColor);
			}
			if ( $summaryTable{$group}{response} > 0 ) { 
				$summaryTable{$group}{avgresponse} = sprintf("%.3f",$summaryTable{$group}{response} / $summaryTable{$group}{count}); 
				$cellColor = colorResponseTime($summaryTable{$group}{avgresponse});
				printHeadCell($summaryTable{$group}{avgresponse},$cellColor);
			}
			rowEnd;
		}

	 	tableEnd;
	 	paragraph;

		foreach $group ( sort ( keys %NMIS::groupTable ) ) {
		 	cssTableStart("white");
			 	printHeadRow("<A name=$group><a href=#TOP>$group</a>","#FFFFFF",10);
			 	rowStart;
				printHeadCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=$type&report=$report&length=$length&sort=node\">Node</a>","#FFFFFF");
			 	printHeadCell("Device Type","#FFFFFF");
			 	printHeadCell("Role","#FFFFFF");
			 	printHeadCell("Net","#FFFFFF");
			 	printHeadCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=$type&report=$report&length=$length&sort=reachability\">Reachability</a>","#FFFFFF");
			 	printHeadCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=$type&report=$report&length=$length&sort=availability\">Interface Avail.</a>","#FFFFFF");
			 	printHeadCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=$type&report=$report&length=$length&sort=health\">Health</a>","#FFFFFF");
			 	printHeadCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=$type&report=$report&length=$length&sort=response\">Response Time</a>","#FFFFFF");
			 	rowEnd;
				&reportNodeType($group);
		 	tableEnd;		 	
			paragraph;
	 	}


	 	cellEnd;
	 	rowEnd;
	} # report eq health
	elsif ( $report eq "outage" ) {
		$header = "Outage Report";
		my $index;
		my %logreport;
		my @logline;
		my $outageLength;
		my $outageColor;
		my $outageDetails;
		my $outageType;
		my $hash;
		my $logfile = $NMIS::config{event_log};
		my @spacesplit;
			   
	 	# Open the file and store in table
		open (DATA, $logfile) || warn returnTime." typeReports, Cannot open the file $logfile. $!\n";

		# find the line with the entry in and store in the array
		while (<DATA>) {
			chomp;
			if ( $_ =~ /time=/i ) {
				@logline = split ( "," , $_ );
				$hash = "hash$index";
				$logreport{$hash}{time} = $logline[0];		      
				$logreport{$hash}{node} = $logline[1];   
				$logreport{$hash}{event} = $logline[2];   
				$logreport{$hash}{event_level} = $logline[3];
				$logreport{$hash}{details} = $logline[4];   
				$logreport{$hash}{outage} = $logreport{$hash}{details};
				$logreport{$hash}{outage} =~ s/interface=.* secs|Time=.*secs |Time=.*secs//gi; 
				$logreport{$hash}{outage} =~ s/change=(.*)/$1/i; 
				if ( $logreport{$hash}{outage} ne "" ) {
					$logreport{$hash}{outage} = "Change#=$logreport{$hash}{outage}";
				}
			  	if ($logreport{$hash}{event} =~ /node/i ){ 
					$logreport{$hash}{outageType} = "Node Outage";
					$logreport{$hash}{outageLength} = $logreport{$hash}{details};
					$logreport{$hash}{outageLength} =~ s/Time=(.*) secs/$1/; 
					$logreport{$hash}{outageLength} = convertSecsHours($logreport{$hash}{outageLength});
					$logreport{$hash}{details} = "";
				}# End of if node.
				else {
					$logreport{$hash}{outageType} = "Interface Outage";
					@spacesplit = split ( " " , $logreport{$hash}{details});
					$logreport{$hash}{details} = $spacesplit[0];
					$logreport{$hash}{details} =~ s/Interface=//i;
					$logreport{$hash}{outageLength} = $spacesplit[1];
					$logreport{$hash}{outageLength} =~ s/Time=(.*)/$1/; 
					$logreport{$hash}{outageLength} = convertSecsHours($logreport{$hash}{outageLength});

					#=~ s/Time=(.*)/$1/; 
				}
				++$index;
			}
		}
		close (DATA);
	    cssHeaderBar("$header @ $stamp","grey");
	
	 	rowStart;
	 	cellStart("#FFFFFF",20);

	 	cssTableStart("white");
       		
		rowStart;
		printHeadCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=$type&report=$report&length=$length&sort=time\">Time</a>","#FFFFFF");
		printHeadCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=$type&report=$report&length=$length&sort=node\">Node</a>","#FFFFFF");
		printHeadCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=$type&report=$report&length=$length&sort=type\">Outage Type</a>","#FFFFFF");
		printHeadCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=$type&report=$report&length=$length&sort=length\">Outage Time</a>","#FFFFFF");
		printHeadCell("Interface","#FFFFFF");
		printHeadCell("Planned Outage","#FFFFFF");
		rowEnd;

		if ( $sort eq "time" or $sort eq "" ) {
			foreach $index ( sort 
				{ 
					$logreport{$a}{time} <=> $logreport{$b}{time} ||
					$logreport{$a}{node} cmp $logreport{$b}{node} 
				} 
				keys %logreport ) 
			{
				$outageColor = colorTime($logreport{$index}{outageLength});
				rowStart;
				printCell(returnDateStamp($logreport{$index}{time}),$outageColor);
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=summary&node=$logreport{$index}{node}\">$logreport{$index}{node}</a>",$outageColor);
				printCell($logreport{$index}{outageType},$outageColor);
				print "<td bgcolor=$outageColor align=right>$logreport{$index}{outageLength}</td>";
				printCell($logreport{$index}{details},$outageColor);
				printCell($logreport{$index}{outage},$outageColor,1,"center");
				rowEnd;
			} # End for loop.
		}
		elsif ( $sort eq "node" ) {
			foreach $index ( sort 
				{ 
					$logreport{$a}{node} cmp $logreport{$b}{node} ||
					$logreport{$a}{time} <=> $logreport{$b}{time}
				} 
				keys %logreport ) 
			{
				$outageColor = colorTime($logreport{$index}{outageLength});
				rowStart;
				printCell(returnDateStamp($logreport{$index}{time}),$outageColor);
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=summary&node=$logreport{$index}{node}\">$logreport{$index}{node}</a>",$outageColor);
				printCell($logreport{$index}{outageType},$outageColor);
				print "<td bgcolor=$outageColor align=right>$logreport{$index}{outageLength}</td>";
				printCell($logreport{$index}{details},$outageColor);
				printCell($logreport{$index}{change},$outageColor,1,"center");
				rowEnd;
			} # End for loop.
		}
		elsif ( $sort eq "type" ) {
			foreach $index ( sort 
				{ 
					$logreport{$a}{outageType} cmp $logreport{$b}{outageType} ||
					$logreport{$a}{time} <=> $logreport{$b}{time} 
				} 
				keys %logreport ) 
			{
				$outageColor = colorTime($logreport{$index}{outageLength});
				rowStart;
				printCell(returnDateStamp($logreport{$index}{time}),$outageColor);
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=summary&node=$logreport{$index}{node}\">$logreport{$index}{node}</a>",$outageColor);
				printCell($logreport{$index}{outageType},$outageColor);
				print "<td bgcolor=$outageColor align=right>$logreport{$index}{outageLength}</td>";
				printCell($logreport{$index}{details},$outageColor);
				printCell($logreport{$index}{outage},$outageColor,1,"center");
				rowEnd;
			} # End for loop.
		}
		elsif ( $sort eq "length" ) {
			foreach $index ( sort 
				{ 
					$logreport{$b}{outageLength} cmp $logreport{$a}{outageLength} ||
					$logreport{$a}{time} <=> $logreport{$b}{time} 
				} 
				keys %logreport ) 
			{
				$outageColor = colorTime($logreport{$index}{outageLength});
				rowStart;
				printCell(returnDateStamp($logreport{$index}{time}),$outageColor);
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=summary&node=$logreport{$index}{node}\">$logreport{$index}{node}</a>",$outageColor);
				printCell($logreport{$index}{outageType},$outageColor);
				print "<td bgcolor=$outageColor align=right>$logreport{$index}{outageLength}</td>";
				printCell($logreport{$index}{details},$outageColor);
				printCell($logreport{$index}{change},$outageColor,1,"center");
				rowEnd;
			} # End for loop.
		}
						   
		tableEnd;															
	 	cellEnd;
	 	rowEnd;
	} # end of report = outage
	elsif ( $report eq "response" ) {
		$reportLength = "-6 hours";
		$header = "6 Hours Dynamic Reponse Time Summary @ $datestamp_end"; 

		cssHeaderBar("$header @ $stamp","grey");
	
		# Get each of the nodes info in a HASH for playing with
		foreach $reportnode (sort(keys(%NMIS::nodeTable))) {
			loadSystemFile($reportnode);
    		$reportStats = summaryStats($reportnode,"health",$reportLength,time);
			if ( $reportStats != 1 and $reportStats ne "" ) {
				@tmparray = split " ","@$reportStats";
				for ( $index = 0; $index <= $#tmparray; ++$index ) {
					@tmpsplit = split "=",@tmparray[$index];
					$reportTable{$reportnode}{$tmpsplit[0]} = $tmpsplit[1] ;
				}
				$reportTable{$reportnode}{net} = $NMIS::nodeTable{$reportnode}{net};
				$reportTable{$reportnode}{role} = $NMIS::nodeTable{$reportnode}{role};
				$reportTable{$reportnode}{devicetype} = $NMIS::nodeTable{$reportnode}{devicetype};
			}
		}

	 	rowStart;
	 	cellStart("#FFFFFF",20);

	 	cssTableStart("white");
	 	printHeadRow("Average Response Time for Routers","#FFFFFF",10);							
		 	rowStart;
			printHeadCell("Node","#FFFFFF");
		 	printHeadCell("Node Type","#FFFFFF");
		 	printHeadCell("Response Time","#FFFFFF");
		 	rowEnd;
			foreach $reportnode ( sort { $reportTable{$b}{response} <=> $reportTable{$a}{response} } keys %reportTable ) 
			{
				loadSystemFile($reportnode);
				if ( $reportTable{$reportnode}{devicetype} eq "router" ) {
					rowStart;
					printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=summary&node=$reportnode\">$reportnode</a>");
					printCell("$reportTable{$reportnode}{devicetype}");
					$cellColor = colorResponseTime($reportTable{$reportnode}{response});
					printCellRight("$reportTable{$reportnode}{response}",$cellColor);
					rowEnd;
				}
			}
	 	tableEnd;

	 	cellEnd;
	 	rowEnd;
	} # report eq response
	elsif ( $report eq "port" ) {
		my $intHash;
		my %portCount;
		my $countHash;
		my $countHead;
		my $floor;
		my $print;
		my $percentage;
		my $color;

		loadInterfaceInfo;

		# Get each of the nodes info in a HASH for playing with
		foreach $intHash (sort(keys(%NMIS::interfaceInfo))) {

			if ( $NMIS::interfaceInfo{$intHash}{ifDescr} != /vlan|gec|sc|sl/i ) {
				if ( $NMIS::interfaceInfo{$intHash}{node} =~ /^ssg/i ) {
					$countHash = "GPO";
				}
				elsif ( $NMIS::interfaceInfo{$intHash}{node} =~ /^ssb|^ssa/i ) {
					$countHash = "20 Bond";
				}
				elsif ( $NMIS::interfaceInfo{$intHash}{node} =~ /^sma/i ) {
					$countHash = "Melbourne";
				}
				elsif ( $NMIS::interfaceInfo{$intHash}{node} =~ /^sba/i ) {
					$countHash = "Brisbane";
				}
				elsif ( $NMIS::interfaceInfo{$intHash}{node} =~ /^sla/i ) {
					$countHash = "London";
				}
				elsif ( $NMIS::interfaceInfo{$intHash}{node} =~ /^sp/i ) {
					$countHash = "Perth";
				}
				elsif ( $NMIS::interfaceInfo{$intHash}{node} =~ /^sa/i ) {
					$countHash = "Adelaide";
				}
				elsif ( $NMIS::interfaceInfo{$intHash}{node} =~ /^snyc/i ) {
					$countHash = "New York";
				}
				elsif ( $NMIS::interfaceInfo{$intHash}{node} =~ /^shkg/i ) {
					$countHash = "Hong Kong";
				}
				else {
					$countHash = "Other";
				}
				$floor = substr("$NMIS::interfaceInfo{$intHash}{node}",3,2);
				if ( $floor == "30" or $floor == "31" ) { $floor = "B3"; }
				elsif ( $floor == "60" or $floor == "61" ) { $floor = "06"; }

				++$portCount{Total}{totalportcount};
				++$portCount{Total}{"admin-$NMIS::interfaceInfo{$intHash}{ifAdminStatus}"};
				++$portCount{Total}{"oper-$NMIS::interfaceInfo{$intHash}{ifOperStatus}"};
				++$portCount{Total}{"speed-$NMIS::interfaceInfo{$intHash}{ifSpeed}"};
				++$portCount{Total}{"duplex-$NMIS::interfaceInfo{$intHash}{portDuplex}"};

				++$portCount{$countHash}{totalportcount};
				++$portCount{$countHash}{"admin-$NMIS::interfaceInfo{$intHash}{ifAdminStatus}"};
				++$portCount{$countHash}{"oper-$NMIS::interfaceInfo{$intHash}{ifOperStatus}"};
				++$portCount{$countHash}{"speed-$NMIS::interfaceInfo{$intHash}{ifSpeed}"};
				++$portCount{$countHash}{"duplex-$NMIS::interfaceInfo{$intHash}{portDuplex}"};

				++$portCount{"$countHash Level $floor"}{totalportcount};
				++$portCount{"$countHash Level $floor"}{"admin-$NMIS::interfaceInfo{$intHash}{ifAdminStatus}"};
				++$portCount{"$countHash Level $floor"}{"oper-$NMIS::interfaceInfo{$intHash}{ifOperStatus}"};
				++$portCount{"$countHash Level $floor"}{"speed-$NMIS::interfaceInfo{$intHash}{ifSpeed}"};
				++$portCount{"$countHash Level $floor"}{"duplex-$NMIS::interfaceInfo{$intHash}{portDuplex}"};
			}
		}

		$header = "Port Count Summary Report @ $datestamp_end"; 
	    	cssHeaderBar("$header","grey");

	 	rowStart;
	 	cellStart("#FFFFFF",20);
	 	cssTableStart("white");

	       	printHeadRow("<BIG>Summary Port Counts</BIG>","#FFFFFF",3);
	       	printRow("The port count summary is indicitive.  Consideration should be given to weight the port counts<BR>according to day of week port types etc.","#FFFF00",3);

		$intHash = "Total";
	       	printHeadRow("$intHash Port Totals","#FFFFFF",3);

		rowStart;
		printCell("Port Count");
		printCell("$portCount{$intHash}{totalportcount}","#FFFFFF",1,"right");
		printCell("");
		rowEnd;

		$color = "#FFFFFF";
		rowStart;
		printCell("Admin Up Port Count");
		printCell("$portCount{$intHash}{'admin-up'}","#FFFFFF",1,"right");
		$percentage = sprintf("%.0f",$portCount{$intHash}{'admin-up'} / $portCount{$intHash}{totalportcount} * 100);
		printCell("$percentage%",$color,1,"right");
		rowEnd;

		rowStart;
		printCell("Admin Down Port Count");
		printCell("$portCount{$intHash}{'admin-down'}","#FFFFFF",1,"right");
		$percentage = sprintf("%.0f",$portCount{$intHash}{'admin-down'} / $portCount{$intHash}{totalportcount} * 100);
		printCell("$percentage%",$color,1,"right");
		rowEnd;

		rowStart;
		printCell("Oper Up Port Count");
		printCell("$portCount{$intHash}{'oper-ok'}","#FFFFFF",1,"right");
		$percentage = sprintf("%.0f",$portCount{$intHash}{'oper-ok'} / $portCount{$intHash}{totalportcount} * 100);
		$color = colorPort($percentage);
		printCell("$percentage%",$color,1,"right");
		rowEnd;

		rowStart;
		printCell("Oper Down Port Count");
		printCell("$portCount{$intHash}{'oper-other'}","#FFFFFF",1,"right");
		$percentage = sprintf("%.0f",$portCount{$intHash}{'oper-other'} / $portCount{$intHash}{totalportcount} * 100);
		printCell("$percentage%",$color,1,"right");
		rowEnd;

		rowStart;
		printCell("Oper Minor Fault Port Count");
		printCell("$portCount{$intHash}{'oper-minorFault'}","#FFFFFF",1,"right");
		$percentage = sprintf("%.0f",$portCount{$intHash}{'oper-minorFault'} / $portCount{$intHash}{totalportcount} * 100);
		if ( $portCount{$intHash}{'oper-minorFault'} > 0 ) { $color = "#FFFF00"; } else { $color = "#00FF00"; }
		printCell("$percentage%",$color,1,"right");
		rowEnd;

		$color = "#FFFFFF";
		rowStart;
		printCell("Full Duplex Port Count");
		printCell("$portCount{$intHash}{'duplex-full'}","#FFFFFF",1,"right");
		$percentage = sprintf("%.0f",$portCount{$intHash}{'duplex-full'} / $portCount{$intHash}{totalportcount} * 100);
		printCell("$percentage%",$color,1,"right");
		rowEnd;

		rowStart;
		printCell("Half Duplex Port Count");
		printCell("$portCount{$intHash}{'duplex-half'}","#FFFFFF",1,"right");
		$percentage = sprintf("%.0f",$portCount{$intHash}{'duplex-half'} / $portCount{$intHash}{totalportcount} * 100);
		printCell("$percentage%",$color,1,"right");
		rowEnd;

		rowStart;
		printCell("10 megabit Port Count");
		printCell("$portCount{$intHash}{'speed-10000000'}","#FFFFFF",1,"right");
		$percentage = sprintf("%.0f",$portCount{$intHash}{'speed-10000000'} / $portCount{$intHash}{totalportcount} * 100);
		printCell("$percentage%",$color,1,"right");
		rowEnd;

		rowStart;
		printCell("100 megabit Port Count");
		printCell("$portCount{$intHash}{'speed-100000000'}","#FFFFFF",1,"right");
		$percentage = sprintf("%.0f",$portCount{$intHash}{'speed-100000000'} / $portCount{$intHash}{totalportcount} * 100);
		printCell("$percentage%",$color,1,"right");
		rowEnd;

		rowStart;
		printCell("1 gigabit Port Count");
		printCell("$portCount{$intHash}{'speed-1000000000'}","#FFFFFF",1,"right");
		$percentage = sprintf("%.0f",$portCount{$intHash}{'speed-1000000000'} / $portCount{$intHash}{totalportcount} * 100);
		printCell("$percentage%",$color,1,"right");
		rowEnd;
		printRow;

		# Do the summaries for everthing but Total and Level summaries
		foreach $intHash (keys(%portCount)) {
			if ( $intHash =~ /Level|Total/ ) { $print = "false"; }
			else { $print = "true"; }

			if ( $print eq "true" ) {
				printHeadRow("$intHash Port Totals","#FFFFFF",3);

				rowStart;
				printCell("Port Count");
				printCell("$portCount{$intHash}{totalportcount}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{totalportcount} / $portCount{Total}{totalportcount} * 100);
				$color = colorLowGood($percentage);
				printCell("$percentage% of Total",$color,1,"right");
				rowEnd;

				$color = "#FFFFFF";
				rowStart;
				printCell("Admin Up Port Count");
				printCell("$portCount{$intHash}{'admin-up'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'admin-up'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("Admin Down Port Count");
				printCell("$portCount{$intHash}{'admin-down'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'admin-down'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("Oper Up Port Count");
				printCell("$portCount{$intHash}{'oper-ok'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'oper-ok'} / $portCount{$intHash}{totalportcount} * 100);
				$color = colorPort($percentage);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("Oper Down Port Count");
				printCell("$portCount{$intHash}{'oper-other'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'oper-other'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("Oper Minor Fault Port Count");
				printCell("$portCount{$intHash}{'oper-minorFault'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'oper-minorFault'} / $portCount{$intHash}{totalportcount} * 100);
				if ( $portCount{$intHash}{'oper-minorFault'} > 0 ) { $color = "#FFFF00"; } else { $color = "#00FF00"; }
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				$color = "#FFFFFF";
				rowStart;
				printCell("Full Duplex Port Count");
				printCell("$portCount{$intHash}{'duplex-full'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'duplex-full'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("Half Duplex Port Count");
				printCell("$portCount{$intHash}{'duplex-half'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'duplex-half'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("10 megabit Port Count");
				printCell("$portCount{$intHash}{'speed-10000000'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'speed-10000000'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("100 megabit Port Count");
				printCell("$portCount{$intHash}{'speed-100000000'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'speed-100000000'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("1 gigabit Port Count");
				printCell("$portCount{$intHash}{'speed-1000000000'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'speed-1000000000'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;
				printRow;
			}
		}

	 	tableEnd;
		paragraph;
	 	cssTableStart("white");
		# Do the summaries for Floor Levels summaries
		printHeadRow("<BIG>Floor Port Counts</BIG>","#FFFFFF",3);
		foreach $intHash (sort (keys(%portCount))) {
			if ( $intHash =~ /Level/ ) {
				$print = "true";
			}
			else {
				$print = "false";
			}
			if ( $print eq "true" ) {
				printRow;
			       	printHeadRow("$intHash Port Totals","#FFFFFF",3);

				rowStart;
				printCell("Port Count");
				printCell("$portCount{$intHash}{totalportcount}","#FFFFFF",1,"right");
				$countHash = $intHash;
				$countHash =~ s/(.*) Level.*/$1/g;
				$percentage = sprintf("%.0f",$portCount{$intHash}{totalportcount} / $portCount{$countHash}{totalportcount} * 100);
				$color = colorLowGood($percentage);
				printCell("$percentage% of $countHash",$color,1,"right");
				rowEnd;

				$color = "#FFFFFF";
				rowStart;
				printCell("Admin Up Port Count");
				printCell("$portCount{$intHash}{'admin-up'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'admin-up'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("Admin Down Port Count");
				printCell("$portCount{$intHash}{'admin-down'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'admin-down'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("Oper Up Port Count");
				printCell("$portCount{$intHash}{'oper-ok'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'oper-ok'} / $portCount{$intHash}{totalportcount} * 100);
				$color = colorPort($percentage);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("Oper Down Port Count");
				printCell("$portCount{$intHash}{'oper-other'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'oper-other'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("Oper Minor Fault Port Count");
				printCell("$portCount{$intHash}{'oper-minorFault'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'oper-minorFault'} / $portCount{$intHash}{totalportcount} * 100);
				if ( $portCount{$intHash}{'oper-minorFault'} > 0 ) { $color = "#FFFF00"; } else { $color = "#00FF00"; }
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				$color = "#FFFFFF";
				rowStart;
				printCell("Full Duplex Port Count");
				printCell("$portCount{$intHash}{'duplex-full'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'duplex-full'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("Half Duplex Port Count");
				printCell("$portCount{$intHash}{'duplex-half'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'duplex-half'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("10 megabit Port Count");
				printCell("$portCount{$intHash}{'speed-10000000'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'speed-10000000'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("100 megabit Port Count");
				printCell("$portCount{$intHash}{'speed-100000000'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'speed-100000000'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;

				rowStart;
				printCell("1 gigabit Port Count");
				printCell("$portCount{$intHash}{'speed-1000000000'}","#FFFFFF",1,"right");
				$percentage = sprintf("%.0f",$portCount{$intHash}{'speed-1000000000'} / $portCount{$intHash}{totalportcount} * 100);
				printCell("$percentage%",$color,1,"right");
				rowEnd;
				printRow;
			}
		}
	 	tableEnd;
	 	cellEnd;
	 	rowEnd;
	}
	else {
		# Dynamically list the directory contents
		if ( $report ne "" ) {
	        opendir (DIR, "$NMIS::config{report_root}");
	        my @dirlist = readdir DIR;
	        closedir DIR;
	        foreach my $dir (@dirlist) {
	        	if ( $dir =~ /^$report/ ) { $reportTable{$dir} = $dir; }
	        }

			if ($report eq "month") { $header = "Monthly Stored Reports"; }
			elsif ($report eq "week") { $header = "Weekly Stored Reports"; }
			elsif ($report eq "day") { $header = "Daily Stored Reports"; }
			elsif ($report eq "outage-week") { $header = "Weekly Stored Outage Reports"; }
			elsif ($report eq "response-day") { $header = "Daily Stored Response Time Reports"; }

			&nmisMenuSmall;
			&nmisMenuBar;
		    cssHeaderBar("$header","grey");

		 	rowStart;
		    cellStart("#FFFFFF",20);

		 	cssTableStart("white");

			foreach $reportFile (sort (keys %reportTable)) { 
				$reportFile =~ s/$NMIS::config{report_root}\///gi;
				$reportLink = $reportFile;
				$reportLink =~ s/.html//;
				$reportLink =~ s/response-day-|outage-week-|week-|day-|month-//;

				if ($report eq "month") {
					@tmparray = split("-",$reportLink);
					$tmparray[0] = convertMonth($tmparray[0]);
					$reportLink = "$tmparray[0] $tmparray[1]";
				}
				else {
					@tmparray = split("-",$reportLink);
					$tmparray[1] = convertMonth($tmparray[1]);
					$reportLink = "$tmparray[3] $tmparray[0] $tmparray[1] $tmparray[2]";
				}

				printRow ("<a href=$NMIS::config{web_report_root}/$reportFile>$reportLink</a></font>");

			}
			tableEnd;
			cellEnd;
			rowEnd;

		} # if report is blank
		# Show a menu of Daily, weekly, monthly if no options
		else {
			&nmisMenuSmall;
			&nmisMenuBar;
			cssHeaderBar("Report Menu","grey");

		 	rowStart;
			cellStart("#FFFFFF",20);

		 	cssTableStart("white");

			printHeadRow("Dynamic Reports");
			rowStart;
			cellStart;
			print "<STRONG>Network Health Reports</STRONG><BR>";
			print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=reports&report=health&length=day\">Daily Dynamic Reports</a><BR>";
			print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=reports&report=health&length=week\">Weekly Dynamic Reports</a><BR>";
			print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=reports&report=health&length=month\">Monthly Dynamic Reports</a><BR>";
			print "<BR>";
			print "<STRONG>Outage Reports</STRONG><BR>";
			print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=reports&report=outage\">Dynamic Outage Reports</a><BR>";
			print "<BR>";
			print "<STRONG>Response Time Reports</STRONG><BR>";
			print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=reports&report=response\">Dynamic Response Time Report (Last 6 Hours)</a><BR>";
			print "<BR>";
			print "<STRONG>Port Report</STRONG><BR>";
			print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=reports&report=port\">Dynamic Port Count Summary Report</a><BR>";
			print "<BR>";
			cellEnd;
			rowEnd;
			printHeadRow("Snapshot Reports");
			rowStart;
			cellStart;
			print "<STRONG>Network Health Reports</STRONG><BR>";
			print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=reports&report=day\">Daily Stored Reports</a><BR>";
			print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=reports&report=week\">Weekly Stored Reports</a><BR>";
			print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=reports&report=month\">Monthly Stored Reports</a><BR>";
			print "<BR>";
			print "<STRONG>Outage Reports</STRONG><BR>";
			print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=reports&report=outage-week\">Weekly Outage Reports</a><BR>";
			print "<BR>";
			print "<STRONG>Response Time Reports</STRONG><BR>";
			print "<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=reports&report=response-day\">Daily Response Time Reports</a><BR>";
			print "<BR>";
			cellEnd;
			rowEnd;

			tableEnd;
			cellEnd;
			rowEnd;
		}
	} # else if $report 

	tableEnd;
	pageEnd;

	sub reportNodeType {
		my $netType = shift;
		my $roleType = shift;
		# Report sorting based on match the first comparison of elements, if the same uses next
		# defined elements.
		if ( $sort eq "node" or $sort eq "" ) {
			foreach $reportnode ( sort { $a cmp $b } keys %reportTable ) 
			{
				loadSystemFile($reportnode);
				if (	( $reportTable{$reportnode}{net} eq $netType and
					  $reportTable{$reportnode}{role} eq $roleType 
					) or 
					( $reportTable{$reportnode}{group} eq $netType and 
					  $roleType eq ""
					) 
				) {
					rowStart;
					printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=summary&node=$reportnode\">$reportnode</a>");
					printCell("$reportTable{$reportnode}{devicetype}");
					printCell("$reportTable{$reportnode}{role}");
					printCell("$reportTable{$reportnode}{net}");
					$cellColor = colorHighGood($reportTable{$reportnode}{reachable});
					printCellRight("$reportTable{$reportnode}{reachable}",$cellColor);
					$cellColor = colorHighGood($reportTable{$reportnode}{available});
					printCellRight("$reportTable{$reportnode}{available}",$cellColor);
					$cellColor = colorHighGood($reportTable{$reportnode}{health});
					printCellRight("$reportTable{$reportnode}{health}",$cellColor);
					$cellColor = colorResponseTime($reportTable{$reportnode}{response});
					printCellRight("$reportTable{$reportnode}{response}",$cellColor);
					rowEnd;
				}
			}
		} # end sort eq reachability
		elsif ( $sort eq "reachability" ) {
			foreach $reportnode ( sort 
				{ 
					$reportTable{$a}{reachable} <=> $reportTable{$b}{reachable} ||
					$reportTable{$a}{health} <=> $reportTable{$b}{health} ||
					$a cmp $b
				} 
				keys %reportTable ) 
			{
				loadSystemFile($reportnode);
				if (	( $reportTable{$reportnode}{net} eq $netType and
					  $reportTable{$reportnode}{role} eq $roleType 
					) or 
					( $reportTable{$reportnode}{group} eq $netType and 
					  $roleType eq ""
					) 
				) {
					rowStart;
					printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=summary&node=$reportnode\">$reportnode</a>");
					printCell("$reportTable{$reportnode}{devicetype}");
					printCell("$reportTable{$reportnode}{role}");
					printCell("$reportTable{$reportnode}{net}");
					$cellColor = colorHighGood($reportTable{$reportnode}{reachable});
					printCellRight("$reportTable{$reportnode}{reachable}",$cellColor);
					$cellColor = colorHighGood($reportTable{$reportnode}{available});
					printCellRight("$reportTable{$reportnode}{available}",$cellColor);
					$cellColor = colorHighGood($reportTable{$reportnode}{health});
					printCellRight("$reportTable{$reportnode}{health}",$cellColor);
					$cellColor = colorResponseTime($reportTable{$reportnode}{response});
					printCellRight("$reportTable{$reportnode}{response}",$cellColor);
					rowEnd;
				}
			}
		} # end sort eq reachability
		elsif ( $sort eq "availability" ) {
			foreach $reportnode ( sort 
				{ 
					$reportTable{$a}{available} <=> $reportTable{$b}{available} ||
					$reportTable{$a}{health} <=> $reportTable{$b}{health} ||
					$a cmp $b
				} 
				keys %reportTable ) 
			{
				loadSystemFile($reportnode);
				if (	( $reportTable{$reportnode}{net} eq $netType and
					  $reportTable{$reportnode}{role} eq $roleType 
					) or 
					( $reportTable{$reportnode}{group} eq $netType and 
					  $roleType eq ""
					) 
				) {
					rowStart;
					printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=summary&node=$reportnode\">$reportnode</a>");
					printCell("$reportTable{$reportnode}{devicetype}");
					printCell("$reportTable{$reportnode}{role}");
					printCell("$reportTable{$reportnode}{net}");
					$cellColor = colorHighGood($reportTable{$reportnode}{reachable});
					printCellRight("$reportTable{$reportnode}{reachable}",$cellColor);
					$cellColor = colorHighGood($reportTable{$reportnode}{available});
					printCellRight("$reportTable{$reportnode}{available}",$cellColor);
					$cellColor = colorHighGood($reportTable{$reportnode}{health});
					printCellRight("$reportTable{$reportnode}{health}",$cellColor);
					$cellColor = colorResponseTime($reportTable{$reportnode}{response});
					printCellRight("$reportTable{$reportnode}{response}",$cellColor);
					rowEnd;
				}
			}
		} # end sort eq reachability
		elsif ( $sort eq "response" ) {
			foreach $reportnode ( sort 
				{ 
					$reportTable{$b}{response} <=> $reportTable{$a}{response} ||
					$reportTable{$a}{health} <=> $reportTable{$b}{health} ||
					$a cmp $b
				} 
				keys %reportTable ) 
			{
				loadSystemFile($reportnode);
				if (	( $reportTable{$reportnode}{net} eq $netType and
					  $reportTable{$reportnode}{role} eq $roleType 
					) or 
					( $reportTable{$reportnode}{group} eq $netType and 
					  $roleType eq ""
					) 
				) {
					rowStart;
					printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=summary&node=$reportnode\">$reportnode</a>");
					printCell("$reportTable{$reportnode}{devicetype}");
					printCell("$reportTable{$reportnode}{role}");
					printCell("$reportTable{$reportnode}{net}");
					$cellColor = colorHighGood($reportTable{$reportnode}{reachable});
					printCellRight("$reportTable{$reportnode}{reachable}",$cellColor);
					$cellColor = colorHighGood($reportTable{$reportnode}{available});
					printCellRight("$reportTable{$reportnode}{available}",$cellColor);
					$cellColor = colorHighGood($reportTable{$reportnode}{health});
					printCellRight("$reportTable{$reportnode}{health}",$cellColor);
					$cellColor = colorResponseTime($reportTable{$reportnode}{response});
					printCellRight("$reportTable{$reportnode}{response}",$cellColor);
					rowEnd;
				}
			}
		} # end sort eq reachability
		elsif ( $sort eq "health" ) {
			foreach $reportnode ( sort 
				{ 
					$reportTable{$a}{health} <=> $reportTable{$b}{health} ||
					$a cmp $b
				} 
				keys %reportTable ) 
			{
				loadSystemFile($reportnode);
				if (	( $reportTable{$reportnode}{net} eq $netType and
					  $reportTable{$reportnode}{role} eq $roleType 
					) or 
					( $reportTable{$reportnode}{group} eq $netType and 
					  $roleType eq ""
					) 
				) {
					rowStart;
					printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=summary&node=$reportnode\">$reportnode</a>");
					printCell("$reportTable{$reportnode}{devicetype}");
					printCell("$reportTable{$reportnode}{role}");
					printCell("$reportTable{$reportnode}{net}");
					$cellColor = colorHighGood($reportTable{$reportnode}{reachable});
					printCellRight("$reportTable{$reportnode}{reachable}",$cellColor);
					$cellColor = colorHighGood($reportTable{$reportnode}{available});
					printCellRight("$reportTable{$reportnode}{available}",$cellColor);
					$cellColor = colorHighGood($reportTable{$reportnode}{health});
					printCellRight("$reportTable{$reportnode}{health}",$cellColor);
					$cellColor = colorResponseTime($reportTable{$reportnode}{response});
					printCellRight("$reportTable{$reportnode}{response}",$cellColor);
					rowEnd;
				}
			}
		} # end elsif
		# default value for sort is reachability

		# Print the summaries!!!!
		if ( $netType ne "" and $roleType eq "" ) { $summaryhash = $netType; }
		else { $summaryhash = "$netType-$roleType"; }
		
		rowStart;
		printHeadCell("Group Averages","#FFFFFF",4);
			$cellColor = colorHighGood($summaryTable{$summaryhash}{avgreachable});
			printHeadCell($summaryTable{$summaryhash}{avgreachable},$cellColor);
			$cellColor = colorHighGood($summaryTable{$summaryhash}{avgavailable});
			printHeadCell($summaryTable{$summaryhash}{avgavailable},$cellColor);
			$cellColor = colorHighGood($summaryTable{$summaryhash}{avghealth});
			printHeadCell($summaryTable{$summaryhash}{avghealth},$cellColor);
			$cellColor = colorResponseTime($summaryTable{$summaryhash}{avgresponse});
			printHeadCell($summaryTable{$summaryhash}{avgresponse},$cellColor);
		rowEnd;
	}
} # end typeReports

sub typeFind {
	my $intHash;
	my $counter;
	my $tmpurl;

	pageStart("Find an Interface","false");
	cssTableStart;
	&nmisMenuSmall; 
	&nmisMenuBar;

	if ( $find eq "" ) { 
		cssHeaderBar("Find an Interface","grey");
		&findMenu; 
	}
	else { 
		cssHeaderBar("Finding Interfaces with \"$find\"","grey");
		&findMenu;
	 	&findNodes; 
	}
	rowEnd;
	tableEnd;
	pageEnd;

	sub findMenu {
		rowStart;
		cellStart("white",1);
		cssTableStart("menu");
			rowStart;
			print "<form ACTION=\"$ENV{SCRIPT_NAME}\">\n";
			printCell("<input TYPE=submit value=\"GO\">");
			printCell("Find String <input name=find value=\"$find\">","#FFFFFF",6);
			print "</form>\n";
			rowEnd;
		tableEnd;
		cellEnd;
		rowEnd;
	}	

	sub findNodes {

		# Remove nasty bad characters from $find
		$find =~ s/\(.*\)|\(|\)|#|\*//g;

		loadInterfaceInfo;

		rowStart;

		cellStart("white",1);
		cssTableStart("white");
		rowStart;
		printHeadCell("Node");
		printHeadCell("Name");
		printHeadCell("IP Address");
		printHeadCell("Subnet Mask");
		printHeadCell("Subnet");
		printHeadCell("Description");
		printHeadCell("ifIndex");
		printHeadCell("Collect");
		printHeadCell("Type");
		printHeadCell("Speed");
		printHeadCell("Admin Status");
		printHeadCell("Oper Status");
		printHeadCell("portDuplex");
		printHeadCell("PortFast");
		printHeadCell("VLAN");
		rowEnd;
		
		$counter = 0;		 
		# Get each of the nodes info in a HASH for playing with
		foreach $intHash (sort(keys(%NMIS::interfaceInfo))) {

			#$NMIS::interfaceInfo{$intHash}{Description} =~ s/\(.*\)|\(|\)|#|\*//g;

			if ( 	$NMIS::interfaceInfo{$intHash}{node} =~ /$find/i or
					$NMIS::interfaceInfo{$intHash}{Description} =~ /$find/i or
					$NMIS::interfaceInfo{$intHash}{ifDescr} =~ /$find/i or
					$NMIS::interfaceInfo{$intHash}{ifType} =~ /$find/i or
					$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} =~ /$find/i or
					$NMIS::interfaceInfo{$intHash}{ipAdEntNetMask} =~ /$find/i or
					$NMIS::interfaceInfo{$intHash}{ipSubnet} =~ /$find/i or
					$NMIS::interfaceInfo{$intHash}{vlanPortVlan} =~ /$find/
			) {
				++$counter;

				$NMIS::interfaceInfo{$intHash}{ifSpeed} = convertIfSpeed($NMIS::interfaceInfo{$intHash}{ifSpeed});

				rowStart;
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=summary&node=$NMIS::interfaceInfo{$intHash}{node}\">$NMIS::interfaceInfo{$intHash}{node}</a>");

				if ( $NMIS::interfaceInfo{$intHash}{collect} eq "true" ) {
					$tmpurl="$ENV{SCRIPT_NAME}?file=$conf&node=$NMIS::interfaceInfo{$intHash}{node}&type=graph&graphtype=util&graphlength=2days&interface=$NMIS::interfaceInfo{$intHash}{ifDescr}";
					#printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=interface&node=$NMIS::interfaceInfo{$intHash}{node}&interface=$NMIS::interfaceInfo{$intHash}{ifDescr}\"></a>","#FFFFFF",1,"center");
					printCell("<a href=\"$tmpurl\" target=ViewWindow onMouseOver=\"window.status='Drill into $NMIS::interfaceInfo{$intHash}{ifDescr}.';return true\" onClick=\"viewdoc('$tmpurl',850,455)\">$NMIS::interfaceInfo{$intHash}{ifDescr}</a>","#FFFFFF",1,"center");
				}
				else { 	printCell("$NMIS::interfaceInfo{$intHash}{ifDescr}","#FFFFFF",1,"center"); }

				printCell("$NMIS::interfaceInfo{$intHash}{ipAdEntAddr}");
				printCell("$NMIS::interfaceInfo{$intHash}{ipAdEntNetMask}");
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=find&find=$NMIS::interfaceInfo{$intHash}{ipSubnet}\">$NMIS::interfaceInfo{$intHash}{ipSubnet}</a>");
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=find&find=$NMIS::interfaceInfo{$intHash}{Description}\">$NMIS::interfaceInfo{$intHash}{Description}</a>");
				printCell("$NMIS::interfaceInfo{$intHash}{ifIndex}");
				printCell("$NMIS::interfaceInfo{$intHash}{collect}");
				printCell("$NMIS::interfaceInfo{$intHash}{ifType}");
				printCell("$NMIS::interfaceInfo{$intHash}{ifSpeed}","#FFFFFF",1,"right");
				if ( $NMIS::interfaceInfo{$intHash}{ifAdminStatus} eq "up" ) {
					printCell("$NMIS::interfaceInfo{$intHash}{ifAdminStatus}","#00FF00",1,"center");
				}
				else {	printCell("$NMIS::interfaceInfo{$intHash}{ifAdminStatus}","#FFFFFF",1,"center"); }
				if ( $NMIS::interfaceInfo{$intHash}{ifOperStatus} =~ /up|ok/ ) {
					printCell("$NMIS::interfaceInfo{$intHash}{ifOperStatus}","#00FF00",1,"center");
				}
				else {	printCell("$NMIS::interfaceInfo{$intHash}{ifOperStatus}","#FFFFFF",1,"center"); }
				printCell("$NMIS::interfaceInfo{$intHash}{portDuplex}","#FFFFFF",1,"center");
				printCell("$NMIS::interfaceInfo{$intHash}{portSpantreeFastStart}");
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=find&find=$NMIS::interfaceInfo{$intHash}{vlanPortVlan}\">$NMIS::interfaceInfo{$intHash}{vlanPortVlan}</a>","#FFFFFF",1,"center");
				rowEnd;
			}
		}

		tableEnd;
		cellEnd;

		paragraph;
		printHeadRow("$counter matches of \"$find\" found in interface list","#FFFFFF",1);
	}
}

sub typeDNS {
	my $intHash;
	my $shortInt;
	my $tmpurl;
	my @in_addr_arpa;
	my $node;
	my $location;
	my %location_data;

	pageStart("Host and DNS Records","false");
	cssTableStart;
	&nmisMenuSmall; 
	&nmisMenuBar;

	#Load the Interface Information file
	loadInterfaceInfo;
	#Load the location data.
	%location_data = loadCSV($NMIS::config{Locations_Table},$NMIS::config{Locations_Key},"\t");

		rowStart;
		cellStart("white",1);
		cssTableStart("white");

		# Host Records
		rowStart;
		printHeadCell("Host Records");
		rowEnd;
		rowStart;
		print	"<td><pre>\n";
	       	foreach $intHash (sort(keys(%NMIS::interfaceInfo))) {
	       		if ( 	$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} ne "" and
	       			$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} ne "0.0.0.0" and
	       			$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} !~ /^127/  
	       		) {
				$NMIS::interfaceInfo{$intHash}{ifSpeed} = convertIfSpeed($NMIS::interfaceInfo{$intHash}{ifSpeed});
				$shortInt = shortInterface($NMIS::interfaceInfo{$intHash}{ifDescr});
				print	
				"$NMIS::interfaceInfo{$intHash}{ipAdEntAddr}\t\t".
				"$NMIS::interfaceInfo{$intHash}{node}--$shortInt\t".
				"#\"".
				"$NMIS::interfaceInfo{$intHash}{Description}\t".
				"$NMIS::interfaceInfo{$intHash}{ifDescr}\t".
				"$NMIS::interfaceInfo{$intHash}{ipSubnet}".
				"$NMIS::interfaceInfo{$intHash}{ipAdEntNetMask}\t".
				"$NMIS::interfaceInfo{$intHash}{ifSpeed}\t".
				"$NMIS::interfaceInfo{$intHash}{ifType}".
				"\"\n";
			}
		} # FOR
		print "</pre></td>\n";
		rowEnd;

		# DNS Records
		rowStart;
		printHeadCell("DNS Records");
		rowEnd;
		rowStart;
		print	"<td><pre>\n";
	       	foreach $intHash (sort(keys(%NMIS::interfaceInfo))) {
	       		if ( 	$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} ne "" and
	       			$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} ne "0.0.0.0" and
	       			$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} !~ /^127/  
	       		) {
				$shortInt = shortInterface($NMIS::interfaceInfo{$intHash}{ifDescr});
				print	
				"$NMIS::interfaceInfo{$intHash}{node}--$shortInt\t\t".
				"IN\tA\t".
				"$NMIS::interfaceInfo{$intHash}{ipAdEntAddr}\n".
				"$NMIS::interfaceInfo{$intHash}{node}--$shortInt\t\t".
				"IN\tTXT\t".
				"\"$NMIS::interfaceInfo{$intHash}{Description} ".
				"$NMIS::interfaceInfo{$intHash}{ifDescr} ".
				"$NMIS::interfaceInfo{$intHash}{ipSubnet} ".
				"$NMIS::interfaceInfo{$intHash}{ipAdEntNetMask} ".
				"$NMIS::interfaceInfo{$intHash}{ifSpeed} ".
				"$NMIS::interfaceInfo{$intHash}{ifType}".
				"\"\n";
			}
		} # FOR
		print "</pre></td>\n";
		rowEnd;
		#0.19.64.10.in-addr.arpa.       IN      PTR     network.mosp.cisco.com.
		#1.19.64.10.in-addr.arpa.       IN      PTR     gw.mosp.cisco.com.

		# in-addr.arpa. Records
		rowStart;
		printHeadCell("in-addr.arpa. DNS Records");
		rowEnd;
		rowStart;
		print	"<td><pre>\n";
	       	foreach $intHash (sort(keys(%NMIS::interfaceInfo))) {
	       		if ( 	$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} ne "" and
	       			$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} ne "0.0.0.0" and
	       			$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} !~ /^127/  
	       		) {
				$shortInt = shortInterface($NMIS::interfaceInfo{$intHash}{ifDescr});
				@in_addr_arpa = split (/\./,$NMIS::interfaceInfo{$intHash}{ipAdEntAddr});
				print		
				"$in_addr_arpa[3].$in_addr_arpa[2].$in_addr_arpa[1].$in_addr_arpa[0].in-addr.arpa.\t".
				"IN\tPTR\t".
				"$NMIS::interfaceInfo{$intHash}{node}--$shortInt.$NMIS::config{domain_name}".
				"\n".
				"$in_addr_arpa[3].$in_addr_arpa[2].$in_addr_arpa[1].$in_addr_arpa[0].in-addr.arpa.\t".
				"IN\tA\t".
				"$NMIS::interfaceInfo{$intHash}{ipAdEntNetMask}".
				"\n";
			}
		} # FOR
		print "</pre></td>\n";
		rowEnd;
# Extract from RFC1876 A Means for Expressing Location Information in the Domain Name System
# This RFC specifies creates DNS LOC (location) records for visual traceroutes
#--snip--
#3. Master File Format                                                                     
#   The LOC record is expressed in a master file in the following format:
#   <owner> <TTL> <class> LOC ( d1 [m1 [s1]] {"N"|"S"} d2 [m2 [s2]]
#                               {"E"|"W"} alt["m"] [siz["m"] [hp["m"]
#                               [vp["m"]]]] )
#   (The parentheses are used for multi-line data as specified in [RFC1035] section 5.1.)
#   where:
#       d1:     [0 .. 90]            (degrees latitude)
#       d2:     [0 .. 180]           (degrees longitude)
#       m1, m2: [0 .. 59]            (minutes latitude/longitude)
#       s1, s2: [0 .. 59.999]        (seconds latitude/longitude)
#       alt:    [-100000.00 .. 42849672.95] BY .01 (altitude in meters)
#       siz, hp, vp: [0 .. 90000000.00] (size/precision in meters)
#
#   If omitted, minutes and seconds default to zero, size defaults to 1m,
#   horizontal precision defaults to 10000m, and vertical precision
#   defaults to 10m.  These defaults are chosen to represent typical
#   ZIP/postal code area sizes, since it is often easy to find
#   approximate geographical location by ZIP/postal code.
#
#4. Example Data
#;;;
#;;; note that these data would not all appear in one zone file
#;;;
#;; network LOC RR derived from ZIP data.  note use of precision defaults
#cambridge-net.kei.com.        LOC   42 21 54 N 71 06 18 W -24m 30m
#;; higher-precision host LOC RR.  note use of vertical precision default
#loiosh.kei.com.               LOC   42 21 43.952 N 71 5 6.344 W -24m 1m 200m
#pipex.net.                    LOC   52 14 05 N 00 08 50 E 10m
#curtin.edu.au.                LOC   32 7 19 S 116 2 25 E 10m
#rwy04L.logan-airport.boston.  LOC   42 21 28.764 N 71 00 51.617 W -44m 2000m
#--end snip--
		# DNS LOC Records
		rowStart;
		printHeadCell("DNS LOC Records");
		rowEnd;
		rowStart;
		print	"<td><pre>\n";
	       	foreach $intHash (sort(keys(%NMIS::interfaceInfo))) {
	       		if ( $NMIS::interfaceInfo{$intHash}{ipAdEntAddr} ne "" ) {
	       			if ( $node ne $NMIS::interfaceInfo{$intHash}{node} ) {
		       			$node = $NMIS::interfaceInfo{$intHash}{node};
		       			loadSystemFile($node);
		       			$location = lc($NMIS::systemTable{sysLocation});
					if ( 	$location_data{$location}{Latitude} ne "" and
						$location_data{$location}{Longitude} ne "" and
						$location_data{$location}{Altitude} ne ""
					) {
						print	
						"$NMIS::interfaceInfo{$intHash}{node}\t\t\t".
						"IN\tLOC\t".
						# Latitude from locations table
						"$location_data{$location}{Latitude} ".
						# Longitude from locations table
						"$location_data{$location}{Longitude} ".
						# Altidude from locations table
						"$location_data{$location}{Altitude} ".
						# Standard precision setting
						"1.00m 10000m 100m".
						"\n";
					}
		       		}
				if ( 	$location_data{$location}{Latitude} ne "" and
					$location_data{$location}{Longitude} ne "" and
					$location_data{$location}{Altitude} ne ""
				) {
					$shortInt = shortInterface($NMIS::interfaceInfo{$intHash}{ifDescr});
					print	
					"$NMIS::interfaceInfo{$intHash}{node}--$shortInt\t\t".
					"IN\tLOC\t".
					# Latitude from locations table
					"$location_data{$location}{Latitude} ".
					# Longitude from locations table
					"$location_data{$location}{Longitude} ".
					# Altidude from locations table
					"$location_data{$location}{Altitude} ".
					# Standard precision setting
					"1.00m 10000m 100m".
					"\n";
				}
			}
		} # FOR
		print "</pre></td>\n";
		rowEnd;

		tableEnd;
		cellEnd;
		rowEnd;
	tableEnd;
	pageEnd;
} #typeDNS

sub typeOutage {
	my $intHash;
	my $counter;
	my $timeNow = time;
	my $start_seconds;
	my $error;
	my $stamp = returnDateStamp;

	$start_seconds = parsedate("$start");

	pageStart("Scheduled Outages","false");
	cssTableStart;
	&nmisMenuSmall; 
	&nmisMenuBar;

	if ( $outage eq "" or $outage eq "list" ) { 
	    	cssHeaderBar("List the Outages @ $stamp","grey");
		&outageMenu; 
		&outageList;
	}
	elsif ( $outage eq "add" ) { 
	    	cssHeaderBar("Adding an outage @ $stamp","grey");
		$error = outageAdd($node,$start,$end,$change);
		$outage = "list";
		&outageMenu;
		if ( $error ne "false" ) {
			errorMessage($error,"#FFFF00",6);
		}
		&outageList;
	}
	elsif ( $outage eq "delete" ) { 
	    	cssHeaderBar("Deleting an outage @ $stamp","grey");
		outageDelete($node,$start,$end,$change);
		$outage = "list";
		&outageMenu;
		&outageList;
	}
	rowEnd;
	tableEnd;
	pageEnd;

	sub outageMenu {
		# if end and start are blank set them to today!!!
		if ( $start eq "" or $start !~ / / ) { $start = returnDateStamp; }
		if ( $end eq "" or $end !~ / / ) { $end = returnDateStamp; }
		if ( $node ne "" and $outage eq "list" ) { $node = ""; }

		rowStart;
		cellStart("#FFFFFF",6);
		cssTableStart("white");

		rowStart;
		print "<form ACTION=\"$ENV{SCRIPT_NAME}\">\n";
		printCell("<input TYPE=submit value=\"GO\">");
		cellStart;
		print "Action <select name=outage SIZE=1>\n";
	        print "    <option value=\"list\">list</option>\n";
	        print "    <option value=\"add\">add</option>\n";
		print "</select>\n";
		cellEnd;

		cellStart;
		print "Node <select name=node SIZE=1>";
		print "		<option value=\"$node\">$node</OPTION>\n";
        	print "		<option value=\"--LAN--\">----LAN----</option>\n";
	        foreach $node (sort ( keys (%NMIS::nodeTable) ) )  {
			if ( $NMIS::nodeTable{$node}{net} eq "lan" ) {
		                print "		<option value=\"$node\">$node</option>\n";
			}
	        }

	        print "<option value=\"--WAN--\">----WAN----</option>\n";
	        foreach $node (sort ( keys (%NMIS::nodeTable) ) )  {
			if ( $NMIS::nodeTable{$node}{net} eq "wan" ) {
		                print "		<option value=\"$node\">$node</option>\n";
			}
	        }

		print "</select>";
		cellEnd;

		printCell("Start Date/Time <input name=start value=\"$start\">","#FFFFFF",1);
		printCell("End Date/Time <input name=end value=\"$end\">","#FFFFFF",1);
		printCell("Change Number <input name=change value=\"$change\">","#FFFFFF",1);

		print "</form>\n";
		rowEnd;

		tableEnd;
		cellEnd;
		rowEnd;
	}	

	sub outageList {
		my $outage;
		my $current;
		my $startStamp;
		my $endStamp;
		my $color;

		outageLoad;

		rowStart;

		cellStart("white",6);
		cssTableStart("white");
		rowStart;
		printHeadCell("");
		printHeadCell("Node");
		printHeadCell("Start");
		printHeadCell("End");
		printHeadCell("Change");
		printHeadCell("Status");
		rowEnd;
		
		# Get each of the nodes info in a HASH for playing with
	       	foreach $outage (sort(keys(%NMIS::outageTable))) {
			if ( $NMIS::outageTable{$outage}{node} ne "" ) {
				$startStamp = returnDateStamp($NMIS::outageTable{$outage}{start});
				$endStamp = returnDateStamp($NMIS::outageTable{$outage}{end});
				$current = outageCheck($NMIS::outageTable{$outage}{node},time);

			  	if ( $current eq "true" ) { 
					$color = "#00FF00";
					$current = "current"; 
				}
			  	elsif ( $current eq "pending" ) { 
					$color = "#FFFF00";
				}
				else { $color = "#FFFFFF"; }

				rowStart;
				printCell(	"<a href=\"$ENV{SCRIPT_NAME}?file=$conf&outage=delete".
							"&node=$NMIS::outageTable{$outage}{node}".
							"&start=$NMIS::outageTable{$outage}{start}".
							"&end=$NMIS::outageTable{$outage}{end}&".
							"change=$NMIS::outageTable{$outage}{change}\">Delete</a>"
							,$color);
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&type=summary&node=$NMIS::outageTable{$outage}{node}\">$NMIS::outageTable{$outage}{node}</a>",$color);
				printCell("$startStamp",$color);
				printCell("$endStamp",$color);
				printCell("$NMIS::outageTable{$outage}{change}",$color);
				printCell("$current",$color,1,"center");
				rowEnd;
			}
		}
		tableEnd;
		cellEnd;
	}
}

sub nmisMenuBar {
	print <<EO_HTML;
<tr>
	<td class="white">
		<table class="white">
		<tr>
			<td class="button"><a href="$ENV{SCRIPT_NAME}?file=$conf&type=find">Find</a></td>
			<td class="button"><a href="$ENV{SCRIPT_NAME}?file=$conf&type=event">Current Events</a></td>
			<td class="button"><a href="$NMIS::config{logs}?log=Event_Log&sort=descending&lines=100&level=normal">Event Log</a></td>
			<td class="button"><a href="$ENV{SCRIPT_NAME}?file=$conf&type=reports">Reports</a></td>
			<td class="button"><a href="$ENV{SCRIPT_NAME}?file=$conf&type=outage">Outages</a></td>
			<td class="button"><a href="$ENV{SCRIPT_NAME}?file=$conf&type=link">Links</a></td>
			<td class="button"><a href="$ENV{SCRIPT_NAME}?file=$conf&type=dns">DNS</a></td>
			<td class="button"><a href="$NMIS::config{map}?file=$conf">Map</a></td>
			<td class="button"><a href="$NMIS::config{ip}">IP</a></td>
			<td class="button"><a href="$NMIS::config{logs}">Logs</a></td>
			<td class="button"><a href="$NMIS::config{logs}?log=NMIS_Log&sort=descending&lines=100&level=normal">NMIS Log</a></td>
			<td class="button">Tables -></td>
			<td class="button"><a href="$NMIS::config{view}?file=$conf&table=Locations">Locations</a></td>
			<td class="button"><a href="$NMIS::config{view}?file=$conf&table=Contacts">Contacts</a></td>
			<td class="button"><a href="$NMIS::config{view}?file=$conf&table=Events">Event Policy</a></td>
			<td class="button"><a href="$NMIS::config{view}?file=$conf&table=Escalation">Escalation Policy</a></td>
			<td class="button"><a href="$NMIS::config{view}?file=$conf&table=Nodes">Nodes</a></td>
			<td class="button"><a href="$NMIS::config{view}?file=$conf&table=Links">Links</a></td>
			<td class="button"><a href="$NMIS::config{view}?file=$conf&table=Map">Map</a></td>
		</tr>
		</table>
	</td>
</tr>
EO_HTML

}

sub nmisMenuSmall {

	print <<EO_HTML;
<tr>
<td class="grey">
<table class="menu1">
<form ACTION="$ENV{SCRIPT_NAME}">
<tr>
	<td class="button"><a href="$ENV{SCRIPT_NAME}?file=$conf&menu=small">Dash</a></td>
	<td class="button"><a href="$ENV{SCRIPT_NAME}?file=$conf&menu=large">Large Dash</a></td>
	<td class="menugrey"><a href="$NMIS::config{'<url_base>'}/"><img alt=\"Help\" src=\"$NMIS::config{doc_icon}\" border=\"0\"></a>
	<a href="$NMIS::config{help_file}#$type"><img alt=\"Help\" src=\"$NMIS::config{help_icon}\" border=\"0\"></a></td>
	<td class="menugrey">
		Statistics Type
		<select name=type SIZE=1>
	            <option value="$type">$type</option>
		    <option value="find">find</option>
	            <option value="summary">summary</option>
		    <option value="health">health</option>
	            <option value="link">link</option>
		    <option value="event">event</option>
		</select>
	</td>
	<td class="menugrey">
		Node
		<select name=node SIZE=1>
		<option value="$node">$node</OPTION>
EO_HTML
	my ($node,$i,$line,@nodedetails);
	loadNodeDetails;

        print "<option value=\"--LAN--\">----LAN----</option>\n";

        foreach $node (sort ( keys (%NMIS::nodeTable) ) )  {
		if ( $NMIS::nodeTable{$node}{net} eq "lan" ) {
	                print "<option value=\"$node\">$node</option>\n";
		}
        }

        print "<option value=\"--WAN--\">----WAN----</option>\n";

        foreach $node (sort ( keys (%NMIS::nodeTable) ) )  {
		if ( $NMIS::nodeTable{$node}{net} eq "wan" ) {
	                print "<option value=\"$node\">$node</option>\n";
		}
        }

print <<EO_HTML;
	     	</select>
  	</td>
EO_HTML

print <<EO_HTML;
	<td class="menugrey">
		Group
		<select name=group SIZE=1>
		<option value="$group">$group</OPTION>
EO_HTML

        foreach $node (sort ( keys (%NMIS::groupTable) ) )  {
                print "<option value=\"$node\">$node</option>\n";
        }

print <<EO_HTML;
	     	</select>
  	</td>
EO_HTML

print <<EO_HTML;
	<td class="menugrey">Find <input type="text" size="20" name="find" value="$find"></td>
	<td class="menugrey"><input type=submit value="GO"></td>
	<td class="menugrey"><a href="http://www.sins.com.au/nmis/">NMIS Version<br>$NMIS::VERSION</a></td>
</tr>
</form>
</table>
</td>
</tr>
EO_HTML

} # end nmisMenuSmall

sub nmisMenuLarge {

	my $group;
	my $span = 1;

	loadEventState;
	loadNodeDetails;

	&nmisMenuSmall;
	&nmisMenuBar;
	&nmisSummary;

	rowStart;
	cssCellStart("white",$span);
	cssTableStart("white");

	rowStart;
	cssPrintCell("grey","Node List and Status",$span);
	rowEnd;

        foreach $group (sort ( keys (%NMIS::groupTable) ) ) {

		rowStart;
		cssPrintCell("grey","<A name=$group>$group Nodes</a>",$span);
		rowEnd;

		rowStart;
		cssCellStart("",$span);
		cssTableStart("white");
		printHeadRow("Node,NMIS,Tools,Net,Role,Group,Location,Outage,Status,Health,Reachability,Int Avail,Response Time","");
		&printNodeType($group);
		tableEnd;
		cellEnd;
		rowEnd;
		
	}
	tableEnd;
	cellEnd;
	rowEnd;
		
} # end nmisMenuLarge

sub nmisGroupSummary {
	my $group = shift;

	my $span = 1;

	loadEventState;
	loadNodeDetails;

	%summaryHash = &getGroupSummary($group);
	my %oldGroupSummary = &getGroupSummary($group,"-16 hours","-8 hours");
	my $overallStatus = overallNodeStatus($group);
	my $overallColor = eventColor($overallStatus);

	$oldGroupSummary{average}{metric_diff} = sprintf("%.3f",$summaryHash{average}{metric} - $oldGroupSummary{average}{metric});
	if ( $oldGroupSummary{average}{metric_diff} <= -1 ) {
		$oldGroupSummary{average}{metric_color} = "red";
		$summaryHash{average}{metric_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_down_big}\" border=\"0\" width=\"11\" height=\"10\">";
	}
	elsif ( $oldGroupSummary{average}{metric_diff} < 0 ) {
		$oldGroupSummary{average}{metric_color} = "red";
		$summaryHash{average}{metric_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">";
	}
	elsif ( $oldGroupSummary{average}{metric_diff} < 1 ) {
		$oldGroupSummary{average}{metric_color} = "#00FF00";
		$summaryHash{average}{metric_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">";
	}
	elsif ( $oldGroupSummary{average}{metric_diff} >= 1 ) {
		$oldGroupSummary{average}{metric_color} = "#00FF00";
		$summaryHash{average}{metric_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up_big}\" border=\"0\" width=\"11\" height=\"10\">";
	}
	# Arrow Icons for all the bits!
	if ( $oldGroupSummary{average}{reachable} <= $summaryHash{average}{reachable} ) { $summaryHash{average}{reachable_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
	else { $summaryHash{average}{reachable_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
	if ( $oldGroupSummary{average}{available} <= $summaryHash{average}{available} ) { $summaryHash{average}{available_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
	else { $summaryHash{average}{available_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
	if ( $oldGroupSummary{average}{health} <= $summaryHash{average}{health} ) { $summaryHash{average}{health_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
	else { $summaryHash{average}{health_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
	if ( $oldGroupSummary{average}{response} <= $summaryHash{average}{response} ) { $summaryHash{average}{response_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
	else { $summaryHash{average}{response_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
	$summaryHash{average}{reachable_diff} = sprintf("%.3f",$summaryHash{average}{reachable} - $oldGroupSummary{average}{reachable});
	$summaryHash{average}{available_diff} = sprintf("%.3f",$summaryHash{average}{available} - $oldGroupSummary{average}{available});
	$summaryHash{average}{health_diff} = sprintf("%.3f",$summaryHash{average}{health} - $oldGroupSummary{average}{health});
	$summaryHash{average}{response_diff} = sprintf("%.0f",$summaryHash{average}{response} - $oldGroupSummary{average}{response});

	&nmisMenuSmall;
	&nmisMenuBar;

		print <<EO_HTML;
<tr><td class="grey" colspan="3">$group Network Metrics</td>
<tr>
<td width="50%" class="white" colspan="1">
<table class="white">
	</tr>
	<tr>
		<td align="center" class="" rowspan="4" bgcolor="$overallColor">
		<font size="+1">$overallStatus</font>
		</td>
		<td align="center" class="" rowspan="4" bgcolor="$summaryHash{average}{metric_color}">
		<font size="+2">$summaryHash{average}{metric}</font> $summaryHash{average}{metric_icon}
		</td>
		<td align="center" class="" rowspan="4" bgcolor="$oldGroupSummary{average}{metric_color}">
		<font size="+1">was: $oldGroupSummary{average}{metric}<br>diff: $oldGroupSummary{average}{metric_diff}</font>
		</td>
		<td bgcolor="$summaryHash{average}{reachable_color}">Reachablility</td>
		<td bgcolor="$summaryHash{average}{reachable_color}" align="right">$summaryHash{average}{reachable} $summaryHash{average}{reachable_icon}</td>
		<td bgcolor="$summaryHash{average}{reachable_color}" align="right">was: $oldGroupSummary{average}{reachable} -> diff: $summaryHash{average}{reachable_diff}</td>
	</tr>
<tr>
	<td bgcolor="$summaryHash{average}{available_color}">Interface Availablility</td>
	<td bgcolor="$summaryHash{average}{available_color}" align="right">$summaryHash{average}{available} $summaryHash{average}{available_icon}</td>
	<td bgcolor="$summaryHash{average}{available_color}" align="right">was: $oldGroupSummary{average}{available} -> diff: $summaryHash{average}{available_diff}</td>
</tr>
<tr>
	<td bgcolor="$summaryHash{average}{health_color}">Health</td>
	<td bgcolor="$summaryHash{average}{health_color}" align="right">$summaryHash{average}{health} $summaryHash{average}{health_icon}</td>
	<td bgcolor="$summaryHash{average}{health_color}" align="right">was: $oldGroupSummary{average}{health} -> diff: $summaryHash{average}{health_diff}</td>
</tr>
<tr>
	<td bgcolor="$summaryHash{average}{response_color}" >Response Time</td>
	<td bgcolor="$summaryHash{average}{response_color}" align="right">$summaryHash{average}{response} ms $summaryHash{average}{response_icon}</td>
	<td bgcolor="$summaryHash{average}{response_color}" align="right">was: $oldGroupSummary{average}{response} ms -> diff: $summaryHash{average}{response_diff}</td>
</tr>
</table>
</td>
</tr>
EO_HTML

	print "<tr><td class=\"grey\">$group Node List and Status</td>\n";

	rowStart;
	cssCellStart("white",$span);
	cssTableStart("white");
	printHeadRow("Node,NMIS,Tools,Net,Role,Group,Location,Outage,Status,Health,Reachability,Interface<BR>Availability,Response Time","#FFFFFF");
	&printNodeType($group);
	tableEnd;
	cellEnd;
	rowEnd;
} # end nmisGroupSummary

sub nmisSummary {
	my $overallStatus;
	my $overallColor;
	my $group;
	my $span = 1;
	my %groupSummary;
	my %oldGroupSummary;

	loadEventState;
	loadNodeDetails;
	
	%summaryHash = &getGroupSummary();
	%oldGroupSummary = &getGroupSummary("","-16 hours","-8 hours");
	
	rowStart;
	print "\t<td width=\"100%\" class=\"white\" colspan=\"$span\">\n"; 
	cssTableStart("white");
	$oldGroupSummary{average}{metric_diff} = sprintf("%.3f",$summaryHash{average}{metric} - $oldGroupSummary{average}{metric});
	if ( $oldGroupSummary{average}{metric_diff} <= -1 ) {
		$oldGroupSummary{average}{metric_color} = "red";
		$summaryHash{average}{metric_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_down_big}\" border=\"0\" width=\"11\" height=\"10\">";
	}
	elsif ( $oldGroupSummary{average}{metric_diff} < 0 ) {
		$oldGroupSummary{average}{metric_color} = "red";
		$summaryHash{average}{metric_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">";
	}
	elsif ( $oldGroupSummary{average}{metric_diff} < 1 ) {
		$oldGroupSummary{average}{metric_color} = "#00FF00";
		$summaryHash{average}{metric_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">";
	}
	elsif ( $oldGroupSummary{average}{metric_diff} >= 1 ) {
		$oldGroupSummary{average}{metric_color} = "#00FF00";
		$summaryHash{average}{metric_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up_big}\" border=\"0\" width=\"11\" height=\"10\">";
	}
	# Arrow Icons for all the bits!
	if ( $oldGroupSummary{average}{reachable} <= $summaryHash{average}{reachable} ) { $summaryHash{average}{reachable_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
	else { $summaryHash{average}{reachable_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
	if ( $oldGroupSummary{average}{available} <= $summaryHash{average}{available} ) { $summaryHash{average}{available_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
	else { $summaryHash{average}{available_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
	if ( $oldGroupSummary{average}{health} <= $summaryHash{average}{health} ) { $summaryHash{average}{health_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
	else { $summaryHash{average}{health_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
	if ( $oldGroupSummary{average}{response} <= $summaryHash{average}{response} ) { $summaryHash{average}{response_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
	else { $summaryHash{average}{response_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
	
	print "\t<td valign=\"top\" width=\"30%\" bgcolor=\"white\" colspan=\"1\">\n"; 
		cssTableStart("white");
		print <<EO_HTML;
		<tr><td class="grey" colspan="2">Network Metrics</td></tr>
		<tr height="40">
		<td align="center" bgcolor="$summaryHash{average}{metric_color}">
		<font size="+2">$summaryHash{average}{metric}</font> $summaryHash{average}{metric_icon}</td> 
		<td align="center" bgcolor="$oldGroupSummary{average}{metric_color}">
		was: $oldGroupSummary{average}{metric}<br>diff: $oldGroupSummary{average}{metric_diff}</td> 
		</tr>
EO_HTML
			#rowStart;
			#printCell("Network Metric",$summaryHash{average}{metric_color},1);
			#printCell("$summaryHash{average}{metric}",$summaryHash{average}{metric_color},1,"right");
			#rowEnd;
			rowStart;
			printCell("Reachablility",$summaryHash{average}{reachable_color},1);
			printCell("$summaryHash{average}{reachable} $summaryHash{average}{reachable_icon}",$summaryHash{average}{reachable_color},1,"right");
			rowEnd;
			rowStart;
			printCell("Interface Availablility",$summaryHash{average}{available_color},1);
			printCell("$summaryHash{average}{available} $summaryHash{average}{available_icon}",$summaryHash{average}{available_color},1,"right");
			rowEnd;
			rowStart;
			printCell("Health",$summaryHash{average}{health_color},1);
			printCell("$summaryHash{average}{health} $summaryHash{average}{health_icon}",$summaryHash{average}{health_color},1,"right");
			rowEnd;
			rowStart;
			printCell("Response Time",$summaryHash{average}{response_color},1);
			printCell("$summaryHash{average}{response} ms $summaryHash{average}{response_icon}",$summaryHash{average}{response_color},1,"right");
			rowEnd;
		tableEnd;
	cellEnd;

	print "\t<td width=\"70%\" bgcolor=\"white\" colspan=\"1\">\n"; 
		cssTableStart("white");
			print "<tr><td class=\"grey\" colspan=\"9\">Current Network Status</td></tr>";
			printHeadRow("Group,Status,Count,Metric,Reach.,IntAvail,Health,RT","white",1);
			$overallStatus = overallNodeStatus;
			$overallColor = eventColor($overallStatus);
			rowStart;
			printCell("All Groups Status",$overallColor,1);
			printCell("$overallStatus",$overallColor,1,"center");
			printCell("$NMIS::nodeCount{total}",$overallColor,1,"right");
			printCell("$summaryHash{average}{metric} $summaryHash{average}{metric_icon}",$summaryHash{average}{metric_color},1,"right");
			printCell("$summaryHash{average}{reachable} $summaryHash{average}{reachable_icon}",$summaryHash{average}{reachable_color},1,"right");
			printCell("$summaryHash{average}{available} $summaryHash{average}{available_icon}",$summaryHash{average}{available_color},1,"right");
			printCell("$summaryHash{average}{health} $summaryHash{average}{health_icon}",$summaryHash{average}{health_color},1,"right");
			printCell("$summaryHash{average}{response} ms $summaryHash{average}{response_icon}",$summaryHash{average}{response_color},1,"right");
			rowEnd;
			foreach $group (sort ( keys (%NMIS::groupTable) ) ) {
				%groupSummary = getGroupSummary($group);
				%oldGroupSummary = &getGroupSummary($group,"-16 hours","-8 hours");
				if ( $oldGroupSummary{average}{metric} <= $groupSummary{average}{metric} ) { $groupSummary{average}{metric_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
				else { $groupSummary{average}{metric_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
				if ( $oldGroupSummary{average}{reachable} <= $groupSummary{average}{reachable} ) { $groupSummary{average}{reachable_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
				else { $groupSummary{average}{reachable_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
				if ( $oldGroupSummary{average}{available} <= $groupSummary{average}{available} ) { $groupSummary{average}{available_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
				else { $groupSummary{average}{available_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
				if ( $oldGroupSummary{average}{health} <= $groupSummary{average}{health} ) { $groupSummary{average}{health_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
				else { $groupSummary{average}{health_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
				if ( $oldGroupSummary{average}{response} <= $groupSummary{average}{response} ) { $groupSummary{average}{response_icon} = "<img alt=\"Up\" src=\"$NMIS::config{arrow_up}\" border=\"0\" width=\"11\" height=\"10\">"; }
				else { $groupSummary{average}{response_icon} = "<img alt=\"Down\" src=\"$NMIS::config{arrow_down}\" border=\"0\" width=\"11\" height=\"10\">"; }
				$overallStatus = overallNodeStatus($group);
				$overallColor = eventColor($overallStatus);
				rowStart;
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&group=$group\">$group</a>",$overallColor,1);
				printCell("$overallStatus",$overallColor,1,"center");
				printCell("$NMIS::nodeCount{$group}",$overallColor,1,"right");
				printCell("$groupSummary{average}{metric} $groupSummary{average}{metric_icon}",$groupSummary{average}{metric_color},1,"right");
				printCell("$groupSummary{average}{reachable} $groupSummary{average}{reachable_icon}",$groupSummary{average}{reachable_color},1,"right");
				printCell("$groupSummary{average}{available} $groupSummary{average}{available_icon}",$groupSummary{average}{available_color},1,"right");
				printCell("$groupSummary{average}{health} $groupSummary{average}{health_icon}",$groupSummary{average}{health_color},1,"right");
				printCell("$groupSummary{average}{response} ms $groupSummary{average}{response_icon}",$groupSummary{average}{response_color},1,"right");
				rowEnd;
			}
		tableEnd;
	cellEnd;

	tableEnd;
	cellEnd;
	rowEnd;
} # end nmisSummary

sub printNodeType {
	my $group = shift;
	
	my $outageExists;
	
        foreach $node (sort ( keys (%NMIS::nodeTable) ) ) {
		if ( $NMIS::nodeTable{$node}{group} eq "$group" ) {
			$outageExists = outageCheck($node,time);
			if (  $outageExists eq "true" ) {
				$outage = outageCheckHash($node,time);
				$outage = "<BR>Ref.=$NMIS::outageTable{$outage}{change}";
			}
			else { 
				$outage = ""; 
				#$outageExists = "";
			}
			rowStart;
				# Load the system table for the node.
				loadSystemFile($node);
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$node&type=summary\">$node</a>","#FFFFFF",1);
				printCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$node&type=summary\">Summary</a> ".
					"<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$node&type=health\">Health</a> "
					#"<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$node&type=interface\">Interface</a>"
					,"#FFFFFF",1);
				printCell(
					"<a href=\"telnet://$node\"><img alt=\"telnet to $node\" src=\"$NMIS::config{telnet_icon}\" border=\"0\">".
					"<a href=\"$NMIS::config{admin}?tool=ping&node=$node\"><img alt=\"ping $node\" src=\"$NMIS::config{ping_icon}\" BORDER=\"0\"></a>".
					"<a href=\"$NMIS::config{admin}?tool=trace&node=$node\"><img alt=\"traceroute $node\" src=\"$NMIS::config{trace_icon}\" BORDER=\"0\"></a>"
					,"#FFFFFF",1);
				printCell("$NMIS::nodeTable{$node}{net}");
				printCell("$NMIS::nodeTable{$node}{role}");
				printCell("<a href=\"$NMIS::config{view}?file=$conf&table=Locations&name=$NMIS::nodeTable{$node}{group}\">$NMIS::nodeTable{$node}{group}</a>");
				printCell("<a href=\"$NMIS::config{view}?file=$conf&table=Locations&name=$NMIS::systemTable{sysLocation}\">$NMIS::systemTable{sysLocation}</a>");
				printCell("$outageExists $outage");
				printCell("$summaryHash{$node}{event_status}",$summaryHash{$node}{event_color},1,"right");
				printCell("$summaryHash{$node}{health}",$summaryHash{$node}{health_color},1,"right");
				printCell("$summaryHash{$node}{reachable}",$summaryHash{$node}{reachable_color},1,"right");
				printCell("$summaryHash{$node}{available}",$summaryHash{$node}{available_color},1,"right");
				printCell("$summaryHash{$node}{response} ms",$summaryHash{$node}{response_color},1,"right");
			rowEnd;
		}
        }
}

sub displaySystemHeader {
	my $node = shift;

	my $event_status;
	my $event_color;
	my $configLink;
	my $outage;

	if ( $NMIS::systemTable{supported} eq "false" or $NMIS::systemTable{supported} eq "" ) { 
		$NMIS::systemTable{sysName} = "Device Not Supported";
	}

	if ( $NMIS::systemTable{nodeModel} eq "CiscoRouter" ) { $configLink = "Cisco_Router/$node.cfg"; }
	elsif ( $NMIS::systemTable{nodeModel} =~ /MSM|RSM/i ) { $configLink = "Cisco_Catalyst_L2L3_Switch/$node.cfg"; }
	elsif ( $NMIS::systemTable{nodeModel} =~ /1010/ ) { $configLink = "Cisco_LS1010_Switch/$node.cfg"; }
	elsif ( $NMIS::systemTable{nodeModel} =~ /Catalyst/ ) { $configLink = "Cisco_Catalyst_Switch/$node.cfg"; }

	if ( -r "/var/adm/CSCOpx/files/archive/shadow/$configLink" ) {
		$configLink = "<a href=\"/configs/$configLink\">View Config</a>";
	}
	else { $configLink = ""; }

	if ( eventExist($node,"Node Down","") eq "true" ) {
		($event_status,$event_color) = eventLevel("Node Down",$NMIS::nodeTable{$node}{role});
	}
	else {
		($event_status,$event_color) = eventLevel("Node Up",$NMIS::nodeTable{$node}{role});
	}

	cssCellStart("white");
	cssTableStart("white");
	rowStart;
	cssPrintCell("button","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$node&type=summary\">Summary</a>");

	#if ( $NMIS::nodeTable{$node}{collect} eq "true" ) {
	#	printHeadCell("<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$node&type=interface\">Interface Statistics</a>","#AAAAAA");
	#}

	cssPrintCell("button","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$node&type=health\">Health Statistics</a>");
	cssPrintCell("button","<a href=\"$ENV{SCRIPT_NAME}?file=$conf&node=$node&type=link\">Link List</a>");
	rowEnd;
	tableEnd;
	cellEnd;
	rowEnd;

	if ( outageCheck($node,time) eq "true" ) {
		$outage = outageCheckHash($node,time);
		$outage = "- Planned Outage change=$NMIS::outageTable{$outage}{change}";
	}

	print <<EO_HTML;
<tr>
	<td class="white">
      		<table class="white">
	       		<tr><A name=INFO>
				<td class="sh_head" colspan="1" bgcolor="$event_color" align="center">Name</td>
				<td class="sh_head" colspan="2" bgcolor="$event_color" align="center"><a href="$ENV{SCRIPT_NAME}?file=$conf&type=event&node=$node">$NMIS::systemTable{sysName}</a></td>
				<td class="sh_head" colspan="1">UpTime</td>
				<td class="sh_head" colspan="1" align="center">$NMIS::systemTable{sysUpTime}</td>
				<td class="sh_head" colspan="1">
					<a href="$NMIS::config{admin}?tool=ping&node=$node"><img alt="ping $node" src="$NMIS::config{ping_icon}" border="0" align="top"></a>
					<a href="$NMIS::config{admin}?tool=trace&node=$node"><img alt="traceroute $node" src="$NMIS::config{trace_icon}" BORDER="0" align="top"></a>
					<a href="telnet://$node"><img alt="telnet to $node" src="$NMIS::config{telnet_icon}" border="0" align="top"></a>
				</td>
			</tr>
	       		<tr>
				<td class="sh_head" colspan="1" bgcolor=$event_color><b>Status</b></td>
				<td class="sh_head" colspan="2" bgcolor=$event_color><a href="$ENV{SCRIPT_NAME}?file=$conf&type=event&node=$node">$event_status $outage</a></td>
				<td class="sh_body" colspan="1">$configLink</td>
				<td class="sh_body" colspan="1"><a href="$NMIS::config{logs}?log=Event_Log&search=$node&lines=100&level=normal">Event Log</a></td>
				<td class="sh_body" colspan="1"><a href="$NMIS::config{logs}?log=Cisco_Syslog&search=$node&lines=100&level=normal">Cisco Syslog</a></td>
			</tr>
	      		<tr>
				<td class="sh_bold" colspan="1"><b>Node Type</b></td>
				<td class="sh_body" colspan="1" align="center"><a href="$NMIS::config{view}?file=$conf&table=Events&column=Type&value=$NMIS::systemTable{nodeType}">$NMIS::systemTable{nodeType}</a></td>
				<td class="sh_bold" colspan="1"><b>Node Vendor</b></td>
				<td class="sh_body" colspan="1" align="center">$NMIS::systemTable{nodeVendor}</td>
				<td class="sh_bold" colspan="1"><b>Node Model</b></td>
				<td class="sh_body" colspan="1" align="center">$NMIS::systemTable{nodeModel}</td>
			</tr>
			<tr>
				<td class="sh_bold"><b>Description</b></td>
				<td class="sh_body" colspan="5">$NMIS::systemTable{sysDescr}</td>
			</tr>
	       		<tr>
				<td class="sh_bold" width="10%"><b>Net Type</b></td>
				<td class="sh_body" align="center" width="13%">$NMIS::systemTable{netType}</td>
				<td class="sh_bold" width="10%"><b>Role Type</b></td>
				<td class="sh_body" align="center" width="13%"><a href="$NMIS::config{view}?file=$conf&table=Events&column=Role&value=$NMIS::systemTable{roleType}">$NMIS::systemTable{roleType}</a></td>
				<td class="sh_bold" width="10%"><b>Interfaces</b></td>
				<td class="sh_body" align="center" width="13%">$NMIS::systemTable{ifNumber}</td>
			</tr>
			<tr>
	       			<td class="sh_bold"><b>Contact</b></td>
				<td class="sh_body" colspan="2"><a href=\"$NMIS::config{view}?file=$conf&table=Contacts&name=$NMIS::systemTable{sysContact}\">$NMIS::systemTable{sysContact}</a></td>
	      			<td class="sh_bold"><b>Location</b></td>
				<td class="sh_body" colspan="2"><a href=\"$NMIS::config{view}?file=$conf&table=Locations&name=$NMIS::systemTable{sysLocation}\">$NMIS::systemTable{sysLocation}</a></td>
			</tr>
EO_HTML
	if ( 	$NMIS::systemTable{nodeVendor} =~ "Cisco" and
		$NMIS::systemTable{nodeType} eq "router"
	) {
		print <<EO_HTML;
			<tr>
				<td class="sh_bold" width="8%"><b>Serial Number</b></td>
				<td class="sh_body" align="center" width="8%">$NMIS::systemTable{serialNum}</td>
				<td class="sh_bold" width="8%"><b>Chassis Version</b></td>
				<td class="sh_body" align="center" width="8%">$NMIS::systemTable{chassisVer}</td>
				<td class="sh_bold" width="8%"><b>Processor Mem</b></td>
				<td class="sh_body" align="center" width="8%">$NMIS::systemTable{procMem}</td>
			</tr>
EO_HTML
	}
	elsif ( $NMIS::systemTable{nodeVendor} =~ "Cisco" and
		$NMIS::systemTable{nodeType} eq "switch"
	) {
		print <<EO_HTML;
			<tr>
				<td class="sh_bold" width="8%"><b>Serial Number</b></td>
				<td class="sh_body" align="center" width="8%">$NMIS::systemTable{serialNum}</td>
				<td class="sh_bold" width="8%"><b>Traffic Peak</b></td>
				<td class="sh_body" align="center" width="8%">$NMIS::systemTable{sysTrafficPeak}%</td>
				<td class="sh_bold" width="8%"><b>Peak Time (ago)</b></td>
				<td class="sh_body" align="center" width="8%">$NMIS::systemTable{sysTrafficPeakTime}</td>
			</tr>
EO_HTML
	}
	print <<EO_HTML;
		</table>
	</td>
EO_HTML
	close (DATAFILE);
} # displaySystemHeader

sub drawrrd {
        my %args = @_;
        my $node = $args{node};
	my $type = $args{type};
	my $length = $args{length};
	my $start = $args{start};
	my $end = $args{end};
	my $width = $args{width};
	my $height = $args{height};
	my $imagefile = $args{imagefile};
	my $interface = $args{interface};

	my $weight;
	my $imageDir;
	my $database;
	my $tmpifDescr;
	my $ERROR;
	my $graphret;
	my $xs;
	my $ys;
	my $speed;

	loadSystemFile($node);
	loadInterfaceFile($node);

	if ( $end == 0 ) { $end = time; }
	if ( $start == 0 ) { $start = convertTime($length); }

	my $datestamp_start = &returnDateStamp($start);
	my $datestamp_end = &returnDateStamp($end);
	my $datestamp = &returnDateStamp(time);

	# if the filename starts with / in it must be a absolute directory
	if ( $imagefile =~ /^\// ) { $imageDir = ""; }
	else { $imageDir = "$NMIS::config{web_root}/tmp"; }

	if ( $NMIS::debug eq "verbose" ) { print "Graphing: node=$node type=$type length=$length start=$start end=$end width=$width height=$height imagefile=$imagefile interface=$interface speed=$NMIS::interfaceTable{$interface}{ifSpeed}\n";  }

	print "Content-type: image/png\n\n";

	# Run the approiate graph subrouting
	if ( $type eq "util" ) { 
		$tmpifDescr = &convertIfName($interface);
		if ( $NMIS::interfaceTable{$tmpifDescr}{ifSpeed} eq "auto" ) {
			 $NMIS::interfaceTable{$tmpifDescr}{ifSpeed} = 10000000;
		} 
		$speed = &convertIfSpeed($NMIS::interfaceTable{$tmpifDescr}{ifSpeed});
		$database = "$NMIS::config{database_root}/interface/$NMIS::systemTable{nodeType}/$node/$node-$tmpifDescr.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node: $interface - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", '% Utilisation', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:input=$database:ifInOctets:AVERAGE", 
		"DEF:output=$database:ifOutOctets:AVERAGE", 
		"DEF:status=$database:ifOperStatus:AVERAGE", 
		"CDEF:inputUtil=input,8,*,$NMIS::interfaceTable{$tmpifDescr}{ifSpeed},/,100,*", 
		"CDEF:outputUtil=output,8,*,$NMIS::interfaceTable{$tmpifDescr}{ifSpeed},/,100,*", 
		"CDEF:totalUtil=outputUtil,inputUtil,+,2,/", 
		"LINE1:inputUtil#0000ff:In % Util", 
		"LINE1:outputUtil#00ff00:Out % Util", 
		"LINE2:totalUtil#000000:Total % Util", 
		"LINE3:status#aa0000:Availability", 
		"GPRINT:status:AVERAGE:Avg Availability %1.2lf",
		"GPRINT:totalUtil:AVERAGE:Avg Total %1.2lf",
		"GPRINT:inputUtil:AVERAGE:Avg In %1.2lf",
		"GPRINT:outputUtil:AVERAGE:Avg Out %1.2lf",
		"COMMENT:Interface Speed $speed"
		;
	}
	elsif ( $type eq "abits" ) { 
		$tmpifDescr = &convertIfName($interface);
		$speed = &convertIfSpeed($NMIS::interfaceTable{$tmpifDescr}{ifSpeed});
		$database = "$NMIS::config{database_root}/interface/$NMIS::systemTable{nodeType}/$node/$node-$tmpifDescr.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node: $interface - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", 'Utilisation Bits/Second', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:input=$database:ifInOctets:AVERAGE", 
		"DEF:output=$database:ifOutOctets:AVERAGE", 
		"DEF:status=$database:ifOperStatus:AVERAGE", 
		"CDEF:inputBits=input,8,*", 
		"CDEF:outputBits=output,8,*", 
		"LINE1:inputBits#0000ff:In bits/sec", 
		"LINE1:outputBits#00ff00:Out bits/sec", 
		"STACK:inputBits#aaaaaa:Agg In and Out bits/sec", 
		"GPRINT:status:AVERAGE:Avg Availability %1.2lf",
		"GPRINT:inputBits:AVERAGE:Avg In bits/sec %1.2lf",
		"GPRINT:inputBits:MAX:Max In bits/sec %1.2lf",
		"GPRINT:outputBits:AVERAGE:Avg Out bits/sec %1.2lf",
		"GPRINT:outputBits:MAX:Max Out bits/sec %1.2lf",
		"HRULE:$NMIS::interfaceTable{$tmpifDescr}{ifSpeed}#ff0000",
		"COMMENT:Interface Speed $speed",
		;
	}
	elsif ( $type eq "bits" ) { 
		#sample code for this came from Andres Kroonmaa [andre@online.ee]
		#new-mavg = ( (last-mavg - datasample) * weight + datasample
		#weight=0.983
		#rrdtool graph $png -a PNG \
		#  -h 400 \
		#  -w 1000 \
		#  --alt-y-grid  \
		#  --base 1000 \
		#  -s -7week     \
		#  DEF:max=$rrdfile:ds1:MAX     \
		#  CDEF:inp2=max,UN,1,max,IF      \
		#  CDEF:mavg=PREV,UN,inp2,PREV,IF,inp2,-,$weight,*,inp2,+      \
		#  "LINE1:inp2#FF0000:Max values\n"       \
		#  "LINE3:mavg#0000FF:Moving average, weight $weight"
		$weight=0.983;
		$tmpifDescr = &convertIfName($interface);
		$speed = &convertIfSpeed($NMIS::interfaceTable{$tmpifDescr}{ifSpeed});
		$database = "$NMIS::config{database_root}/interface/$NMIS::systemTable{nodeType}/$node/$node-$tmpifDescr.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node: $interface - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", 'Utilisation Bits/Second', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:input=$database:ifInOctets:AVERAGE", 
		"DEF:output=$database:ifOutOctets:AVERAGE", 
		"DEF:maxinput=$database:ifInOctets:MAX", 
		"DEF:maxoutput=$database:ifOutOctets:MAX", 
		"DEF:status=$database:ifOperStatus:AVERAGE", 
		"CDEF:inputBits=input,8,*", 
		"CDEF:outputBits=output,8,*", 
		"CDEF:maxin=maxinput,8,*",
		"CDEF:maxout=maxoutput,8,*",
		"CDEF:inp2in=maxin,UN,1,maxin,IF",
		"CDEF:inp2out=maxout,UN,1,maxout,IF",
		"CDEF:mavgin=PREV,UN,inp2in,PREV,IF,inp2in,-,$weight,*,inp2in,+",
		"CDEF:mavgout=PREV,UN,inp2out,PREV,IF,inp2out,-,$weight,*,inp2out,+",
		"LINE2:inputBits#0000ff:In bits/sec", 
		"LINE2:outputBits#00ff00:Out bits/sec", 
		"STACK:inputBits#aaaaaa:Agg In and Out bits/sec", 
		#"LINE1:inp2#FF0000:Max values\n",
		"LINE3:mavgin#0000AA:Input Moving average, weight $weight",
		"LINE2:mavgout#00AA00:Output Moving average, weight $weight",
		"GPRINT:status:AVERAGE:Avg Availability %1.2lf",
		"GPRINT:inputBits:AVERAGE:Avg In bits/sec %1.2lf",
		"GPRINT:inputBits:MAX:Max In bits/sec %1.2lf",
		"GPRINT:outputBits:AVERAGE:Avg Out bits/sec %1.2lf",
		"GPRINT:outputBits:MAX:Max Out bits/sec %1.2lf",
		"HRULE:$NMIS::interfaceTable{$tmpifDescr}{ifSpeed}#ff0000",
		"COMMENT:Interface Speed $speed",
		;
	}
	elsif ( $type eq "health" ) { 
		if ( $title eq "short" ) { $title = "$node health $length"; }
		else { $title = "$node  - $length from $datestamp_start to $datestamp_end"; }
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-reach.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$title", 
		"--vertical-label", '% Health Statistics', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:reach=$database:reachability:AVERAGE", 
		"DEF:avail=$database:availability:AVERAGE", 
		"DEF:health=$database:health:AVERAGE", 
		"LINE2:avail#00ff00:Availability", 
		"LINE2:health#ff9900:Health", 
		"LINE2:reach#0000ff:Reachability", 
		"GPRINT:reach:AVERAGE:Avg Reachable %1.2lf",
		"GPRINT:avail:AVERAGE:Avg Available %1.2lf",
		"GPRINT:health:AVERAGE:Avg Health %1.2lf",
		;
	}
	elsif ( $type eq "response" ) { 
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-reach.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", 'Response Time in ms', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:response=$database:responsetime:AVERAGE", 
		"LINE2:response#00ff00",
		"HRULE:250#00ff00", 
		"HRULE:500#0000ff",
		"HRULE:1000#ff0000",
		"GPRINT:response:AVERAGE:Avg Response Time %1.2lf",
		;
	}
	elsif ( $type eq "cpu" ) {  
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", '% CPU Utilisation', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:avgBusy1=$database:avgBusy1:AVERAGE", 
		"DEF:avgBusy5=$database:avgBusy5:AVERAGE", 
		"LINE1:avgBusy1#00ff00:Avg Busy 1min", 
		"LINE1:avgBusy5#0000ff:Avg Busy 5min",
		"GPRINT:avgBusy1:AVERAGE:Avg Busy 1min %1.2lf",
		"GPRINT:avgBusy1:MAX:Max Busy 1min %1.2lf",
		"GPRINT:avgBusy5:AVERAGE:Avg Busy 5min %1.2lf",
		"GPRINT:avgBusy5:MAX:Max Busy 5min %1.2lf",
		;
	}
	elsif ( $type eq "traffic" ) {  
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", '% System Traffic', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:sysTraffic=$database:sysTraffic:AVERAGE", 
		"LINE1:sysTraffic#00ff00:Avg Traffic",
		"GPRINT:sysTraffic:AVERAGE:Avg Traffic %1.2lf",
		"GPRINT:sysTraffic:MAX:Max Traffic %1.2lf",
		;
	}
	elsif ( $type eq "ip" ) {  
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", 'IP Routing Packets', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:ipInReceives=$database:ipInReceives:AVERAGE", 
		"DEF:ipForwDatagrams=$database:ipForwDatagrams:AVERAGE", 
		"DEF:ipInDelivers=$database:ipInDelivers:AVERAGE", 
		"DEF:ipOutNoRoutes=$database:ipOutNoRoutes:AVERAGE", 
		"AREA:ipInReceives#cccccc:IP Packets Received", 
		"LINE1:ipForwDatagrams#0000ff:IP Packets Forwarded", 
		"STACK:ipInDelivers#000000:IP Packets Local",
		"LINE2:ipInDelivers#ff0000:IP No Routes",
		"GPRINT:ipInReceives:AVERAGE:Avg Received %1.2lf",
		"GPRINT:ipInReceives:MAX:Max Received %1.2lf",
		"GPRINT:ipForwDatagrams:AVERAGE:Avg Forwarded %1.2lf",
		"GPRINT:ipForwDatagrams:MAX:Max Forwarded %1.2lf",
		"GPRINT:ipInDelivers:AVERAGE:Avg Local %1.2lf",
		"GPRINT:ipInDelivers:MAX:Max Local %1.2lf",
		"GPRINT:ipOutNoRoutes:AVERAGE:Avg No Routes %1.2lf",
		"GPRINT:ipOutNoRoutes:MAX:Max No Routes %1.2lf"
		;
	}
	elsif ( $type eq "topo" ) {  
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", 'SpanT Topo Changes', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:top=$database:TopChanges:AVERAGE", 
		"LINE1:top#0000ff:Number Changes", 
		"GPRINT:top:AVERAGE:Avg Topo Changes %1.2lf",
		"GPRINT:top:MAX:Max Topo Changes %1.2lf",
		;
	}
	elsif ( $type eq "buffer" ) {  
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", 'Buffer Utilisation', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:bufferElFree=$database:bufferElFree:AVERAGE", 
		"DEF:bufferElHit=$database:bufferElHit:AVERAGE", 
		"DEF:bufferFail=$database:bufferFail:AVERAGE",  
		"LINE2:bufferElFree#0000ff:Buffers Free", 
		"LINE2:bufferElHit#00ff00:Buffers Hit", 
		"LINE2:bufferFail#ff0000:Buffers Failed",
		"GPRINT:bufferElFree:AVERAGE:Avg Buffers Free %1.2lf",
		"GPRINT:bufferElFree:MAX:Max Buffers Free %1.2lf",
		"GPRINT:bufferElHit:AVERAGE:Avg Buffers Hit %1.2lf",
		"GPRINT:bufferElHit:MAX:Max Buffers Hit %1.2lf",
		"GPRINT:bufferFail:AVERAGE:Avg Buffers Fail %1.2lf",
		"GPRINT:bufferFail:MAX:Max Buffers Fail %1.2lf",
		;
	}
	elsif ( $type eq "mem-proc" ) {  
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", '% Memmory Utilisation', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:MemUsed=$database:MemoryUsedPROC:AVERAGE", 
		"DEF:MemFree=$database:MemoryFreePROC:AVERAGE", 
		"CDEF:totalMem=MemUsed,MemFree,+",
		"CDEF:perUsedMem=MemUsed,totalMem,/,100,*",
		"CDEF:perFreeMem=MemFree,totalMem,/,100,*",
		"AREA:perFreeMem#cccccc", 
		"STACK:perUsedMem#cccccc",
		"LINE2:perFreeMem#0000ff:% Processor Mem Free", 
		"LINE2:perUsedMem#000000:% Processor Mem Used", 
		"GPRINT:MemUsed:AVERAGE:Proc Mem Used %1.0lf bytes",
		"GPRINT:MemFree:AVERAGE:Proc Mem Free %1.0lf bytes",
		"GPRINT:totalMem:AVERAGE:Total Proc Mem %1.0lf bytes",
		"GPRINT:perUsedMem:AVERAGE:Proc Mem Used %1.2lf",
		"GPRINT:perFreeMem:AVERAGE:Proc Mem Free %1.2lf",
		;
	}
	elsif ( $type eq "mem-io" ) {  
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", '% Memmory Utilisation', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:MemUsed=$database:MemoryUsedIO:AVERAGE", 
		"DEF:MemFree=$database:MemoryFreeIO:AVERAGE", 
		"CDEF:totalMem=MemUsed,MemFree,+",
		"CDEF:perUsedMem=MemUsed,totalMem,/,100,*",
		"CDEF:perFreeMem=MemFree,totalMem,/,100,*",
		"AREA:perFreeMem#cccccc", 
		"STACK:perUsedMem#cccccc",
		"LINE2:perFreeMem#0000ff:% IO Mem Free", 
		"LINE2:perUsedMem#000000:% IO Mem Used", 
		"GPRINT:MemUsed:AVERAGE:IO Mem Used %1.0lf bytes",
		"GPRINT:MemFree:AVERAGE:IO Mem Free %1.0lf bytes",
		"GPRINT:totalMem:AVERAGE:Total IO Mem %1.0lf bytes",
		"GPRINT:perUsedMem:AVERAGE:IO Mem Used %1.2lf",
		"GPRINT:perFreeMem:AVERAGE:IO Mem Free %1.2lf",
		;
	}
	elsif ( $type eq "mem-dram" ) {  
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", '% Memmory Utilisation', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:MemUsed=$database:MemoryUsedDRAM:AVERAGE", 
		"DEF:MemFree=$database:MemoryFreeDRAM:AVERAGE", 
		"CDEF:totalMem=MemUsed,MemFree,+",
		"CDEF:perUsedMem=MemUsed,totalMem,/,100,*",
		"CDEF:perFreeMem=MemFree,totalMem,/,100,*",
		"AREA:perFreeMem#cccccc", 
		"STACK:perUsedMem#cccccc",
		"LINE2:perFreeMem#0000ff:% DRAM Mem Free", 
		"LINE2:perUsedMem#000000:% DRAM Mem Used", 
		"GPRINT:MemUsed:AVERAGE:DRAM Mem Used %1.0lf bytes",
		"GPRINT:MemFree:AVERAGE:DRAM Mem Free %1.0lf bytes",
		"GPRINT:totalMem:AVERAGE:Total DRAM Mem %1.0lf bytes",
		"GPRINT:perUsedMem:AVERAGE:DRAM Mem Used %1.2lf %",
		"GPRINT:perFreeMem:AVERAGE:DRAM Mem Free %1.2lf %",
		;
	}
	elsif ( $type eq "mem-mbuf" ) {  
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", '% Memmory Utilisation', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:MemUsed=$database:MemoryUsedMBUF:AVERAGE", 
		"DEF:MemFree=$database:MemoryFreeMBUF:AVERAGE", 
		"CDEF:totalMem=MemUsed,MemFree,+",
		"CDEF:perUsedMem=MemUsed,totalMem,/,100,*",
		"CDEF:perFreeMem=MemFree,totalMem,/,100,*",
		"AREA:perFreeMem#cccccc", 
		"STACK:perUsedMem#cccccc",
		"LINE2:perFreeMem#0000ff:% MBUF Mem Free", 
		"LINE2:perUsedMem#000000:% MBUF Mem Used", 
		"GPRINT:MemUsed:AVERAGE:MBUF Mem Used %1.0lf bytes",
		"GPRINT:MemFree:AVERAGE:MBUF Mem Free %1.0lf bytes",
		"GPRINT:totalMem:AVERAGE:Total MBUF Mem %1.0lf bytes",
		"GPRINT:perUsedMem:AVERAGE:MBUF Mem Used %1.2lf %",
		"GPRINT:perFreeMem:AVERAGE:MBUF Mem Free %1.2lf %",
		;
	}
	elsif ( $type eq "mem-cluster" ) {  
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", '% Memmory Utilisation', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:MemUsed=$database:MemoryUsedCLUSTER:AVERAGE", 
		"DEF:MemFree=$database:MemoryFreeCLUSTER:AVERAGE", 
		"CDEF:totalMem=MemUsed,MemFree,+",
		"CDEF:perUsedMem=MemUsed,totalMem,/,100,*",
		"CDEF:perFreeMem=MemFree,totalMem,/,100,*",
		"AREA:perFreeMem#cccccc", 
		"STACK:perUsedMem#cccccc",
		"LINE2:perFreeMem#0000ff:% CLUSTER Mem Free", 
		"LINE2:perUsedMem#000000:% CLUSTER Mem Used", 
		"GPRINT:MemUsed:AVERAGE:CLUSTER Mem Used %1.0lf bytes",
		"GPRINT:MemFree:AVERAGE:CLUSTER Mem Free %1.0lf bytes",
		"GPRINT:totalMem:AVERAGE:Total CLUSTER Mem %1.0lf bytes",
		"GPRINT:perUsedMem:AVERAGE:CLUSTER Mem Used %1.2lf %",
		"GPRINT:perFreeMem:AVERAGE:CLUSTER Mem Free %1.2lf %",
		;
	}
	elsif ( $type eq "mem-switch" ) {  
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", '% Mem. Util.', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:MemUsedD=$database:MemoryUsedDRAM:AVERAGE", 
		"DEF:MemFreeD=$database:MemoryFreeDRAM:AVERAGE", 
		"DEF:MemUsedM=$database:MemoryUsedMBUF:AVERAGE", 
		"DEF:MemFreeM=$database:MemoryFreeMBUF:AVERAGE", 
		"DEF:MemUsedC=$database:MemoryUsedCLUSTER:AVERAGE", 
		"DEF:MemFreeC=$database:MemoryFreeCLUSTER:AVERAGE", 
		"CDEF:totalMemD=MemUsedD,MemFreeD,+",
		"CDEF:perUsedMemD=MemUsedD,totalMemD,/,100,*",
		"CDEF:totalMemM=MemUsedM,MemFreeM,+",
		"CDEF:perUsedMemM=MemUsedM,totalMemM,/,100,*",
		"CDEF:totalMemC=MemUsedC,MemFreeC,+",
		"CDEF:perUsedMemC=MemUsedC,totalMemC,/,100,*",
		"LINE2:perUsedMemD#0000ff:% DRAM Mem Used", 
		"LINE2:perUsedMemM#00ff00:% MBUF Mem Used", 
		"LINE2:perUsedMemC#ffff00:% CLUSTER Mem Used", 
		"GPRINT:perUsedMemD:AVERAGE:DRAM Mem Used %1.2lf",
		"GPRINT:perUsedMemM:AVERAGE:MBUF Mem Used %1.2lf",
		"GPRINT:perUsedMemC:AVERAGE:CLUSTER Mem Used %1.2lf",
		"GPRINT:MemUsedD:AVERAGE:DRAM Mem Used %1.0lf bytes",
		"GPRINT:MemFreeD:AVERAGE:DRAM Mem Free %1.0lf bytes",
		"GPRINT:totalMemD:AVERAGE:Total DRAM Mem %1.0lf bytes",
		"GPRINT:MemUsedM:AVERAGE:MBUF Mem Used %1.0lf bytes",
		"GPRINT:MemFreeM:AVERAGE:MBUF Mem Free %1.0lf bytes",
		"GPRINT:totalMemM:AVERAGE:Total MBUF Mem %1.0lf bytes",
		"GPRINT:MemUsedC:AVERAGE:CLUSTER Mem Used %1.0lf bytes",
		"GPRINT:MemFreeC:AVERAGE:CLUSTER Mem Free %1.0lf bytes",
		"GPRINT:totalMemC:AVERAGE:Total CLUSTER Mem %1.0lf bytes",
		;
	}
	elsif ( $type eq "mem-router" ) {  
		$database = "$NMIS::config{database_root}/health/$NMIS::systemTable{nodeType}/$node-health.rrd";
		($graphret,$xs,$ys) = RRDs::graph "-", 
		"--title", "$node - $length from $datestamp_start to $datestamp_end", 
		"--vertical-label", '% Mem. Util.', 
		"--start", "$start", 
		"--end", "$end", 
		"--width", "$width",
		"--height", "$height",
		"--imgformat", "PNG",
		"--interlace",
		"DEF:MemUsedD=$database:MemoryUsedPROC:AVERAGE", 
		"DEF:MemFreeD=$database:MemoryFreePROC:AVERAGE", 
		"DEF:MemUsedM=$database:MemoryUsedIO:AVERAGE", 
		"DEF:MemFreeM=$database:MemoryFreeIO:AVERAGE", 
		"CDEF:totalMemD=MemUsedD,MemFreeD,+",
		"CDEF:perUsedMemD=MemUsedD,totalMemD,/,100,*",
		"CDEF:totalMemM=MemUsedM,MemFreeM,+",
		"CDEF:perUsedMemM=MemUsedM,totalMemM,/,100,*",
		"LINE2:perUsedMemD#0000ff:% Proc Mem Used", 
		"LINE2:perUsedMemM#00ff00:% IO Mem Used", 
		"GPRINT:perUsedMemD:AVERAGE:Proc Mem Used %1.2lf",
		"GPRINT:perUsedMemM:AVERAGE:IO Mem Used %1.2lf",
		"GPRINT:MemUsedD:AVERAGE:Proc Mem Used %1.0lf bytes",
		"GPRINT:MemFreeD:AVERAGE:Proc Mem Free %1.0lf bytes",
		"GPRINT:totalMemD:AVERAGE:Total Proc Mem %1.0lf bytes",
		"GPRINT:MemUsedM:AVERAGE:IO Mem Used %1.0lf bytes",
		"GPRINT:MemFreeM:AVERAGE:IO Mem Free %1.0lf bytes",
		"GPRINT:totalMemM:AVERAGE:Total IO Mem %1.0lf bytes",
		;
	}

	if ($ERROR = RRDs::error) { 
  		logMessage("$node, graph error database=$database: $ERROR");
	} else {
	       	#return "GIF Size: ${xs}x${ys}\n";
		#print "Graph Return:\n",(join "\n", @$graphret),"\n\n";    
	}
} # end graph

